 ; Ŀ
 ;   Aasup - Rocket Setup.                                                 
 ;   Copyright 2011 by Rocket Software Ltd.                                
 ;   Legitimate Authority - who invented this strange idea?                
 ; 

 ; Ŀ
 ;   Subroutine Mono - Partial menu Load and Install.                      
 ;   Arguments: Namu, a menu file name (no extension).                     
 ;              Menm, a menu group name.                                   
 ;              Patha, the path to the menu file.                          
 ;   Returns nothing, calls nothing.                                       
 ; 
 (DEFUN MONO (namu menm patha /)
 ; Ŀ
 ;   Unload a menu by the menu group name, not the file name.              
 ;   Loading an already loaded menu hangs the command with an error, so    
 ;   have to unload it first.                                              
 ;   It isn't necessary to reload a partial menu unless it has changed -   
 ;   partial loads are saved between acad sessions.                        
 ; 
  (command "menuunload" menm)      ; by group name in the file, not file name
 ; Ŀ
 ;   Load the partial menu.                                                
 ;   Search for menu files in reverse version order, load the first one    
 ;   found.                                                                
 ;   Rocket contains a cuix file - pre-2010 doesn't use it.                
 ; 
; (command "menuload" (strcat patha namu ".mnu"))   ; use to force a recompile
  (cond ((and (>= (read (substr (getvar "acadver") 1 2)) 18)
              (setq fnam (findfile (strcat patha namu ".cuix"))))
         (command "menuload" fnam))
        ((setq fnam (findfile (strcat patha namu ".cui")))
         (command "menuload" fnam))
        ((setq fnam (findfile (strcat patha namu ".mnc")))
         (command "menuload" fnam)))
 ; Ŀ
 ;   Put the pop sections from the partial menu in front of pop24.         
 ;   Since there is no pop24 in the main file these become the last        
 ;   (furthest right) menus.                                               
 ; 
  (menucmd (strcat "P24=+" menm ".Pop1"))
  (menucmd (strcat "P24=+" menm ".Pop2"))
  (menucmd (strcat "P24=+" menm ".Pop3"))
  (menucmd (strcat "P24=+" menm ".Pop4"))
  (menucmd (strcat "P24=+" menm ".Pop5"))
  (menucmd (strcat "P24=+" menm ".Pop6"))
  (menucmd (strcat "P24=+" menm ".Pop7"))
  (menucmd (strcat "P24=+" menm ".Pop8"))
  (menucmd (strcat "P24=+" menm ".Pop9"))
  (menucmd (strcat "P24=+" menm ".Pop10"))
 (princ))
 ; Ŀ
 ;   Subroutine Mono end.                                                  
 ; 

 ; Ŀ
 ;   Omno - remove the Rocket menu.                                        
 ; 
 (DEFUN C:OMNO (/ menm)
  (setvar "cmdecho" 0)
  (setq menm "Rocket")         ; group name
  (command "menuunload" menm)  ; by group name in the file, not file name
 (princ))

 ; Ŀ
 ;   Sonar - see if a string contains a substring.                         
 ;   Arguments:  Loc, the substring.                                       
 ;               Txt, the string.                                          
 ;               Cas, if this is non-nil then the search                   
 ;                                is non-case-sensitive.                   
 ;   Returns the number of occurrences of the substring.                   
 ; 
 (DEFUN SONAR (loc txt cas / chflg ln sta st)
  (setq chflg 0)
  (if cas 
      (progn
           (setq loc (strcase loc t))
           (setq txt (strcase txt t))))
  (setq ln (strlen loc))
  (setq sta 1)
  (while (= ln (strlen (setq st (substr txt sta ln))))
         (if (= st loc) (setq chflg (1+ chflg)))
         (setq sta (1+ sta)))
 chflg)
 ; Ŀ
 ;   Sonar end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Splat - divide a text string into a list of substrings.    
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ;                                                                         
 ;   Completely rewritten 11.19.2000.                                      
 ; 
 (DEFUN SPLAT (sepchr linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the string Linn.      
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Aasup.                                                                
 ; 
 (DEFUN C:AASUP (/ namu menm patha prefoo liz gnustr num strub patha2 patha3
                                                                      patha4)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Typically the menu file name and group name should be the same.       
 ; 
  (setq namu "Rocket")      ; file name
  (setq menm "Rocket")      ; group name
  (setq patha "c:\\rocket\\")
 ; Ŀ
 ;   See if a menu file is available.                                      
 ; 
  (if (or (findfile (strcat patha namu ".cui"))
          (findfile (strcat patha namu ".cuix"))
          (findfile (strcat patha namu ".mnu"))
          (findfile (strcat patha namu ".mns"))
          (findfile (strcat patha namu ".mnc")))
      (mono namu menm patha)
      (prompt (strcat "Can't find menu file "
                      (strcase (substr namu 1 1))
                      (strcase (substr namu 2) t) ".")))
 ; Ŀ
 ;   Other stuff you can do.                                               
 ; 
 ; (menucmd "P15.5=~")             ; disable pop15 item 5
 ; (menucmd "P15.5=!")             ; checkmark pop15 item 5
 ; (menucmd "P15.5=")              ; restore pop15 item 5 to base state
 ; (menucmd "p5=-")                ; remove a pop menu
 ; (menucmd "Rocket.pop5=-")       ; doesn't seem to work
 ; Ŀ
 ;   Get the Acadprefix and split it into paths.                           
 ; 
  (setq prefoo (getvar "acadprefix"))
  (setq liz (splat ";" prefoo))
  (setq gnustr "")
  (setq num 0)
  (while (setq strub (nth num liz))
         (setq num (1+ num))
         (if (zerop (sonar "rocket" strub t))
             (setq gnustr (strcat gnustr ";" strub))))
 ; Ŀ
 ;   Prefix the rocket paths to it.                                        
 ; 
  (setq patha2 (strcat patha "lisp"))
  (setq patha3 (strcat patha "blocks"))
  (setq patha4 (strcat patha "terminals"))
  (setq gnustr (strcat patha ";" patha2 ";" patha3 ";" patha4 gnustr))
 ; Ŀ
 ;   Save it back to the sysvar.                                           
 ; 
  (setenv "ACAD" gnustr)
 ; Ŀ
 ;   Set up a few system variables.                                        
 ;   These could be tied to the version number...                          
 ; 
  (setvar "apbox" 0)                 ; don't display an aperture box
  (setvar "cursorsize" 100)          ; make the crosshairs full screen
  (setvar "dblclkedit" 1)            ; double click editing on
  (if (getvar "dynmode")
      (setvar "dynmode" -3))         ; dynamic input off but saved
  (if (getvar "dynprompt")
      (setvar "dynprompt" 0))        ; dynamic prompts in tooltips off
  (setvar "gripobjlimit" 032767)     ; number of grips to show (max allowed)
  (if (getvar "hatchgrip")
      (setvar "hatchgrip" 0))        ; hatches only have one grip each
  (setvar "insunits" 0)              ; don't scale inserted blocks
  (setvar "insunitsdefsource" 0)     ; really don't scale inserted blocks
  (setvar "insunitsdeftarget" 0)     ; don't scale if wblocking ... I think ...
  (if (getvar "layerdlgmode")
      (setvar "layerdlgmode" 1))     ; use the old style layer dialog box
  (if (getvar "isaveback")
      (setvar "isaveback" 1))        ; make backup files
  (if (getvar "menubar")
      (progn
          (setvar "menubar" 1)       ; pulldown menus on
          (command ".ribbonclose"))) ; close the ribbon
  (setvar "mirrtext" 0)              ; don't mirror text
  (setvar "previeweffect" 0)         ; just dash if selectionpreview is on
  (if (getvar "rollovertips")
      (setvar "rollovertips" 0))     ; turn off tooltips
 ; (setvar "savefilepath" "")        ; where should this be?
  (setvar "savetime" 9)              ; save every nine minutes
  (setvar "selectionpreview" 0)      ; stop twitching on crosshair contact
  (setvar "shortcutmenu" 25)         ; time delayed popups on right click etc.
  (setvar "tooltips" 0)              ; turn off tool tips (vs rollovertips?)
 ; (setvar "xloadpath" "")           ; 
  (setvar "xloadctl" 2)              ; demand load xrefs - enabled with copy
 ; Ŀ
 ;   Environment variables.                                                
 ; 
  (setenv "AutomaticPlotLog" "0")      ; don't save a plot log
  (setenv "CmdHistLines" "1000")       ; command history lines - max 2048
  (setenv "DefaultFormatForSave" "36") ; save as 2007
  (setenv "Scrollbars" "0")            ; turn off scrollbars
  (setenv "ShowFullPathInTitle" "1")   ; 
  (setenv "ShowPaperBackground" "0")   ; no paper outline in ps
  (setenv "ShowPaperMargins" "0")      ; no margins in ps
  (setenv "ShowPrintBorder" "0")       ; no print area in ps ffs
  (setenv "Background" "0")            ; set model space background to black
  (setenv "Layout background" "0")     ; set paper space background to black
 (princ))

(c:aasup)   ; run on load, so this file can be dragged into a drawing

