 ; Ŀ
 ;   Bc - fix block names to be leading caps.                              
 ;   Copyright 2003, 2005, 2009, 2013 by Rocket Software Ltd.              
 ;   Doesn't this duplicate Sarn?                                          
 ; 

 ; Ŀ
 ;   Subroutine Fdash - add initial capitals to a text string.             
 ;   Split a string at any character in a list, capitalise each resulting  
 ;   substring, reassemble the string.                                     
 ;   Also watches for certain special cases.                               
 ;   Arguments: Astr: the string to process.                               
 ;              Chra: the list of separator characters.                    
 ;   Recursive.                                                            
 ; 
 (DEFUN FDASH (chra astr / sub prlist nustra nump)
  (if (and astr 
           (car chra)
           (> (length (setq prlist (splat (car chra) astr))) 0))
      (progn
           (setq nustra "")
           (while (setq sub (car prlist))
                  (setq prlist (cdr prlist))
                  (setq sub (strcase sub t))
                  (cond ((= (substr sub 1 1) "(")               ; balance: )
                         (setq sub (strcat "(" (car (hug (substr sub 2)))))) ;)
                        ((or (and (> (setq nump (sonar "." sub t)) 0)
                                  (/= (substr sub (strlen sub)) "."))
                             (> nump 1))
                         (setq sub (strcase sub)))
                        ((member sub '("vsat" "mds" "vavcu"))
                         (setq sub (strcase sub)))
                        (T (setq sub (car (hug sub)))))
                  (setq sub (fdash (cdr chra) sub))             ; recurse
                  (setq nustra (strcat nustra (car chra) sub)))
           (if (= (substr nustra 1 1) (car chra))
               (setq nustra (substr nustra 2))))
      (setq nustra astr))
 nustra)
 ; Ŀ
 ;   Fdash.                                                                
 ; 

 ; Ŀ
 ;   Hug - string capitaliser.  Takes one argument, a string, and returns  
 ;   a list: the string with the first letter changed to upper case and    
 ;   T if this changed the string, () if not.                              
 ; 
 (DEFUN HUG (exstr / nustr)
  (setq nustr (strcat (strcase (substr exstr 1 1))
                      (strcase (substr exstr 2) t)))
 (list nustr (if (= exstr nustr) () t)))
 ; Ŀ
 ;   Hug - end.                                                            
 ; 

 ; Ŀ
 ;   Sonar - see if a string contains a substring.                         
 ;   Arguments:  Loc, the substring.                                       
 ;               Txt, the string.                                          
 ;               Cas, if this is non-nil then the search                   
 ;                                is non-case-sensitive.                   
 ;   Returns the number of occurrences of the substring.                   
 ; 
 (DEFUN SONAR (loc txt cas / chflg ln sta st)
  (setq chflg 0)
  (if cas 
      (progn
           (setq loc (strcase loc t))
           (setq txt (strcase txt t))))
  (setq ln (strlen loc))
  (setq sta 1)
  (while (= ln (strlen (setq st (substr txt sta ln))))
         (if (= st loc) (setq chflg (1+ chflg)))
         (setq sta (1+ sta)))
 chflg)
 ; Ŀ
 ;   Sonar end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Splat - divide a text string into a list of substrings.    
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ;                                                                         
 ;   Completely rewritten 11.19.2000.                                      
 ; 
 (DEFUN SPLAT (sepchr linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the string Linn.      
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Bc - convert all block names to lower case with initial capitals.     
 ; 
 (DEFUN C:BC (/ rew llist blnam gnunam)
  (setvar "cmdecho" 0)
  (command "undo" "be")
 ; Ŀ
 ;   Step through the block tables and impart neatness.                    
 ; 
  (setq rew t)
  (while (setq llist (tblnext "block" rew))
         (setq rew ())
         (setq blnam (cdr (setq asoc2 (assoc 2 llist))))
         (setq gnunam (fdash (list " " "-" "/") blnam))
         (command ".rename" "block" blnam gnunam))
  (command "undo" "end")
 (princ))
