 ; Ŀ
 ;   Bp: see if a pile of loose crap could pass for an accurate dimension. 
 ;   Copyright 2017 by Rocket Software Ltd.                                
 ;                                                                         
 ; 

 ; Ŀ
 ;   Lk - see if the layer on which an entity lies is locked.              
 ;   Takes one argument, the entity ename, returns a string.               
 ; 
 (DEFUN LK (enam / lay sev)
  (setq lay (cdr (assoc 8 (entget enam))))
  (setq sev (cdr (assoc 70 (tblsearch "layer" lay))))
  (if (= 4 (logand sev 4)) "Locked " ""))
 ; Ŀ
 ;   Lk end.                                                               
 ; 

 ; Ŀ
 ;   Bazang - given a list of three points, find the angle between a       
 ;   line from the first one to a line between the two others.             
 ;   Arguments: Ptlist, a list of three points.                            
 ;              Colo, a grdraw colour for test lines, ignore if nil.       
 ;   Calls nothing.                                                        
 ;   Returns a list: the angle difference and the angle between the first  
 ;   point and the point midway between the two others.                    
 ; 
 (DEFUN BAZANG (ptlist colo / pa pb pc basex basey basez base arang bangle)
  (setq pa (car ptlist))
  (setq pb (cadr ptlist))
  (setq pc (caddr ptlist))
 ; Ŀ
 ;   Get the middle of the side defined by the pb and pc.                  
 ; 
  (setq basex (/ (+ (car pb) (car pc)) 2))
  (setq basey (/ (+ (cadr pb) (cadr pc)) 2))
  (setq basez (/ (+ (caddr pb) (caddr pc)) 2))
  (setq base (list basex basey basez))  ; middle of back edge
 ; Ŀ
 ;   Get the angle from pa to the middle of the opposite side.             
 ; 
  (setq arang (angle pa base))
 ; Ŀ
 ;   Get the opposite side angle (pb to pc).                               
 ; 
  (setq bangle (angle pb pc))
 ; Ŀ
 ;   Make some nice test graphic lines.                                    
 ; 
  (if colo
      (progn
           (grdraw pa base colo)
           (grdraw pb pc colo)))
 ; Ŀ
 ;   Return the smallest difference between the two angles.                
 ; 
 (list (abs (rem (- arang bangle) pi)) arang))
 ; Ŀ
 ;   Bazang end.                                                           
 ; 

 ; Ŀ
 ;   Cindi - mark each entity in an ss.                                    
 ;   Arguments: Ss, a selection set.                                       
 ;              Rad - the marker segment length.                           
 ;              Colo - the marker grdraw line colour.                      
 ; 
 (DEFUN CINDI (ss rad colo / len num so pa)
  (setq len (strcat "/" (itoa (sslength ss))))
  (setq num 0)
  (while (setq so (ssname ss num))
         (setq num (1+ num))
         (grtext -2 (strcat (itoa num) len))
         (setq pa (cdr (assoc 10 (entget so))))
         (mark pa rad colo))
 (princ))
 ; Ŀ
 ;   Cindi end.                                                            
 ; 

 ; Ŀ
 ;   Mark - mark a point.                                                  
 ;   Arguments: Pa - the point to mark.                                    
 ;              Rad - the marker segment length.                           
 ;              Colo - the marker grdraw line colour.                      
 ; 
 (DEFUN MARK (pa rad colo /)
  (grdraw (polar pa (/ pi 4) rad) (polar pa (* 1.25 pi) rad) colo)
  (grdraw (polar pa (* pi 0.75) rad) (polar pa (* pi 1.75) rad) colo)
 (princ))
 ; Ŀ
 ;   Mark end.                                                             
 ; 

 ; Ŀ
 ;   Sonar - see if a string contains a substring.                         
 ;   Arguments:  Loc, the substring.                                       
 ;               Txt, the string.                                          
 ;               Cas, if this is non-nil then the search                   
 ;                                is non-case-sensitive.                   
 ;   Returns the number of occurrences of the substring.                   
 ; 
 (DEFUN SONAR (loc txt cas / chflg ln sta st)
  (setq chflg 0)
  (if cas 
      (progn
           (setq loc (strcase loc t))
           (setq txt (strcase txt t))))
  (setq ln (strlen loc))
  (setq sta 1)
  (while (= ln (strlen (setq st (substr txt sta ln))))
         (if (= st loc) (setq chflg (1+ chflg)))
         (setq sta (1+ sta)))
 chflg)
 ; Ŀ
 ;   Sonar end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Splat - divide a text string into a list of substrings.    
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ;                                                                         
 ;   Completely rewritten 11.19.2000.                                      
 ; 
 (DEFUN SPLAT (sepchr linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the string Linn.      
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Triarp - see if a solid is triangular, if so return the endpoint      
 ;   and the angle to the middle of the opposite side.                     
 ;   Arguments: Arrnam, the solid ename.                                   
 ;   Calls Bazang.                                                         
 ;   Returns a point point and an angle or nil.                            
 ; 
 (DEFUN TRIARP (arrnam / arr plist num pa gnulis angg arang thangl axang panu)
 ; Ŀ
 ;   If the user ends the solid command after three points then the 12     
 ;   and 13 groups are the same point.  On the other hand, they may have   
 ;   just dragged one point onto another, so you can't assume this.        
 ; 
  (setq arr (entget arrnam))
  (setq plist (list (cdr (assoc 10 arr)) (cdr (assoc 11 arr))
                    (cdr (assoc 12 arr)) (cdr (assoc 13 arr))))
 ; Ŀ
 ;   Make a new list without duplicate points.                             
 ; 
  (setq num 0)
  (while (setq pa (nth num plist))
         (setq num (1+ num))
         (if (not (member pa gnulis))
             (setq gnulis (cons pa gnulis))))
 ; Ŀ
 ;   Find the point for which a line between it and the midpoint of the    
 ;   opposite side is closest to making a right angle with the opposite    
 ;   side.                                                                 
 ; 
  (if (= (length gnulis) 3)
      (progn
           (repeat 3
                   (setq angg (bazang gnulis nil))
                   (setq arang (cadr angg))
                   (setq angg (car angg))
                   (setq angg (abs (- angg (/ pi 2))))
                   (if (or (null thangl) (< angg thangl))
                       (progn
                            (setq thangl angg)
                            (setq axang arang)
                            (setq panu (car gnulis))))
                   (setq gnulis (cons (last gnulis)
                                      (reverse (cdr (reverse gnulis))))))
 ;         (grdraw panu (polar panu axang 12) 142)
 ; Ŀ
 ;   Return a list of the point and the angle to the back midpoint or nil. 
 ; 
           (list panu axang))))
 ; Ŀ
 ;   Triarp end.                                                           
 ; 

 ; Ŀ
 ;   Bp.                                                                   
 ; 
 (defun c:bp (/ *error* snapp ss num enam typ entt stra sol1 sol2 pa1 pa2
                                                     extlis llist rad elay)
  (setvar "cmdecho" 0)
  (setq snapp (getvar "snapmode"))
  (setq rad (/ (getvar "viewsize") 45))
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (setvar "snapmode" snapp)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Turn off snap.                                                        
 ; 
  (setvar "snapmode" 0)
 ; Ŀ
 ;   Get an ss.                                                            
 ; 
  (setq ss (ssget (list (cons 0 "text,mtext,point,line,solid,dimension")
;                        (cons 8 "urb-dim,rur-dim,urb-det-dim,rur-det-dim")
)))
 ; Ŀ
 ;   See if the ss is an exploded dimension.                               
 ; 
  (setq num 0)
  (if (and ss (null (zerop (sslength ss))))
      (progn
           (While (and ss (setq enam (ssname ss num)))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq typ (cdr (assoc 0 entt)))
                  (setq elay (cdr (assoc 8 entt)))
 ; Ŀ
 ;   Sort the entities by type.                                            
 ; 
                  (cond ((or (= typ "TEXT") (= typ "MTEXT"))
                         (setq stra (cdr (assoc 1 entt))))
                        ((and (null sol1) (= typ "SOLID"))
                         (setq sol1 enam))
                        ((= typ "SOLID")
                         (setq sol2 enam))
                        ((= typ "LINE")
                         (setq llist (cons enam llist)))
                        ((= typ "POINT")
                         (setq plist (cons enam plist)))))
 ; Ŀ
 ;   Get the number from the text string.                                  
 ; 
           (if (>= (sonar ";" stra nil) 1)
               (setq stra (cadr (splat ";" stra))))
 ; Ŀ
 ;   Get the width and angle from the arrowheads.  (Solids.)               
 ; 
           (setq pa1 (car (triarp sol1)))
           (setq pa2 (car (triarp sol2)))
           (setq disa (distance pa1 pa2))
           (setq anga (angle pa1 pa2))
 ; Ŀ
 ;   Identify lines not running from arrowhead to arrowhead.               
 ; 
           (setq num 0)
           (while (and llist (setq enam (nth num llist)))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq lang (angle (cdr (assoc 10 entt))
                                    (cdr (assoc 11 entt))))
                  (if (not (or (equal lang anga 0.015)
                               (equal lang (+ anga pi) 0.015)))
                      (setq dlist (cons enam dlist))))
 ; Ŀ
 ;   Get the other two lines which are at right angles to that and pass    
 ;   through the points of the arrowheads.  Probably two.                  
 ; 
           (setq num 0)
           (while (setq enam (nth num dlist))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq disp (/ disa 20))
                  (setq lpa (cdr (assoc 10 entt)))
                  (setq lpb (cdr (assoc 11 entt)))
                  (setq pa11 (polar pa1 anga disp))
                  (setq pa12 (polar pa1 (+ anga pi) disp))
                  (setq pa21 (polar pa2 anga disp))
                  (setq pa22 (polar pa2 (+ anga pi) disp))
                  (if (or (inters lpa lpb pa11 pa12)
                          (inters lpa lpb pa21 pa22))
                      (setq extlis (cons enam extlis))))
 ; Ŀ
 ;   Get the length of the longest line in extlist.                        
 ; 
           (setq enaml1 (car extlis))
           (setq entt1 (entget enaml1))
           (setq enaml2 (cadr extlis))
           (setq entt2 (entget enaml2))
           (setq len1 (distance (cdr (assoc 10 entt1))
                                (cdr (assoc 11 entt1))))
           (setq len2 (distance (cdr (assoc 10 entt2))
                                (cdr (assoc 11 entt2))))
           (setq len (max len1 len2))
 ; Ŀ
 ;   Figure out the error bar outline.                                     
 ; 
           (setq paaxa pa1)
           (setq paaxb (polar pa1 (+ anga pi) 5))
           (setq paaxa2 (polar paaxa (+ anga (/ pi 2)) len))
           (setq paaxb2 (polar paaxb (+ anga (/ pi 2)) len))
           (setq paaxa (polar paaxa (- anga (/ pi 2)) len))
           (setq paaxb (polar paaxb (- anga (/ pi 2)) len))
           (bex (list paaxa paaxb paaxb2 paaxa2) 8 9 5)
           (setq pabxa pa1)
           (setq pabxb (polar pa1 anga 5))
           (setq pabxa2 (polar pabxa (+ anga (/ pi 2)) len))
           (setq pabxb2 (polar pabxb (+ anga (/ pi 2)) len))
           (setq pabxa (polar pabxa (- anga (/ pi 2)) len))
           (setq pabxb (polar pabxb (- anga (/ pi 2)) len))
           (bex (list pabxa pabxb pabxb2 pabxa2) 9 8 5)
           (setq pacxa pa2)
           (setq pacxb (polar pa2 (+ anga pi) 5))
           (setq pacxa2 (polar pacxa (+ anga (/ pi 2)) len))
           (setq pacxb2 (polar pacxb (+ anga (/ pi 2)) len))
           (setq pacxa (polar pacxa (- anga (/ pi 2)) len))
           (setq pacxb (polar pacxb (- anga (/ pi 2)) len))
           (bex (list pacxa pacxb pacxb2 pacxa2) 8 9 5)
           (setq padxa pa2)
           (setq padxb (polar pa2 anga 5))
           (setq padxa2 (polar padxa (+ anga (/ pi 2)) len))
           (setq padxb2 (polar padxb (+ anga (/ pi 2)) len))
           (setq padxa (polar padxa (- anga (/ pi 2)) len))
           (setq padxb (polar padxb (- anga (/ pi 2)) len))
           (bex (list padxa padxb padxb2 padxa2) 9 8 5)))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (setvar "snapmode" snapp)
 (princ))