 ; Ŀ
 ;   Chall - chgtext for every text entity and attribute in a drawing.     
 ;   Copyright 1995, 2010, 2018 by Rocket Software Ltd.                    
 ;   Science has yet to find any fossilized jellyfish.                     
 ; 

 ; Ŀ
 ;   Subroutine Chall.                                                     
 ;   Arguments: Cc, the old string.                                        
 ;              Dd, the new string.                                        
 ;              Ss, the selection set of things to change.                 
 ;              Rad, the marker radius, nil = no markers.                  
 ;              Notouch, ths list of blocks to ignore.                     
 ;   Calls Chug.                                                           
 ;   Returns nothing.                                                      
 ; 
 (DEFUN CHALL (cc dd ss rad notouch / chlin chnum len num changs strch enam
                        txt entt altr mods newtxt pa chaags attch esub chg)
  (setq chlin 0)
  (setq chnum 0)
 ; Ŀ
 ;   Change the text and mtext as required.                                
 ; 
  (if ss0 (setq ss (ssget "p" '((-4 . "<or") (0 . "text")
                                             (0 . "mtext") (-4 . "or>")))))
  (if ss (setq len (strcat "/" (itoa (sslength ss)) ":Txt")))
  (setq num 0)
  (setq changs 0)
  (setq strch 0)
  (while (and ss (setq enam (ssname ss num)))
         (grtext -2 (strcat (itoa (setq num (1+ num))) len))
         (setq txt (cdr (assoc 1 (setq entt (entget enam)))))
         (setq altr (chug cc dd txt))
         (if (> (setq mods (cadr altr)) 0)           ; if any changes made
                (progn
                     (setq newtxt (car altr))
                     (setq changs (+ changs mods))
                     (setq strch (1+ strch))
                     (setq pa (cdr (assoc 10 entt)))
                     (if rad
                         (progn
                              (grdraw (polar pa (/ pi 4) rad)
                                      (polar pa (* 1.25 pi) rad) 1)
                              (grdraw (polar pa (* pi 0.75) rad)
                                      (polar pa (* pi 1.75) rad) 1)))
                     (entmod (subst (cons 1 newtxt) (assoc 1 entt) entt)))))
 ; Ŀ
 ;   Change the blocks.                                                    
 ; 
  (setq num 0)
  (setq chaags 0)
  (setq attch 0)
  (if ss0
      (progn
           (command "select" ss0 "")
           (setq ss (ssget "p" '((0 . "insert"))))))
  (if ss (setq len (strcat "/" (itoa (sslength ss)) ":Att")))
  (while (and ss (setq enam (ssname ss num)))
         (setq esub (entnext enam))
         (grtext -2 (strcat (itoa (setq num (1+ num))) len))
         (setq chg ())
         (while (and (not (member (cdr (assoc 2 (entget enam))) notouch))
                     (/= "SEQEND" (cdr (assoc 0 (setq entt (entget esub))))))
                (if (setq txt (cdr (assoc 1 entt)))
                    (progn
                         (setq altr (chug cc dd txt))
                         (if (> (setq mods (cadr altr)) 0) ; if changes made
                             (progn
                                  (setq chg T)
                                  (setq newtxt (car altr))
                                  (setq chaags (+ chaags mods))
                                  (setq attch (1+ attch))
                                  (setq pa (cdr (assoc 10 entt)))
                                  (if rad 
                                      (progn
                                           (grdraw (polar pa (/ pi 4) rad)
                                                (polar pa (* 1.25 pi) rad) 7)
                                           (grdraw (polar pa (* pi 0.75) rad)
                                              (polar pa (* pi 1.75) rad) 7)))
                                  (entmod (subst (cons 1 newtxt)
                                                 (assoc 1 entt) entt))))))
                (setq esub (entnext esub)))
         (if chg (entupd enam)))
 ; Ŀ
 ;   Debrief the user.                                                     
 ; 
  (write-line (strcat (itoa changs) " change" (if (/= changs 1) "s" "") " in "
                      (itoa strch) " text string"  (if (/= strch 1) "s." ".")))
  (write-line (strcat (itoa chaags) " change" (if (/= chaags 1) "s" "") " in "
                      (itoa attch) " attribute" (if (/= attch 1) "s." ".")))
 (princ))
 ; Ŀ
 ;   Chall end.                                                            
 ; 

 ; Ŀ
 ;   Chug - string substitution engine.  Takes the search string, the      
 ;   replacement string, and the target string as arguments, and returns   
 ;   the (possibly modified) target string and the number of changes made. 
 ; 
 (DEFUN CHUG (oldstr newstr exstr / pos chnum changd newlen chunk)
  (setq pos 1)
  (setq chnum 0)
  (setq changd ())
  (setq newlen (strlen newstr))
  (setq oldlen (strlen oldstr))
  (while (= oldlen (strlen (setq chunk (substr exstr pos oldlen))))
         (if (= chunk oldstr)
             (progn
                  (setq exstr (strcat (substr exstr 1 (1- pos))
                                       newstr
                                      (substr exstr (+ pos oldlen))))
                  (setq changd t)
                  (setq chnum (1+ chnum))
                  (setq pos (+ pos newlen)))
             (setq pos (1+ pos))))
 (list exstr chnum))
 ; Ŀ
 ;   Chug - end.                                                           
 ; 

 ; Ŀ
 ;   Chall - the controller.                                               
 ; 
 (DEFUN C:CHALL (/ picks *error* ccp cc cont dd notouch rad glist ss0)
  (setvar "cmdecho" 0)
  (command "undo" "be")
 ; Ŀ
 ;   This turns off group selection - otherwise entities in groups may     
 ;   be changed twice with bad things happening as a result.               
 ; 
  (setq picks (getvar "pickstyle"))
  (setvar "pickstyle" 0)
 ; Ŀ
 ;   Something is bound to go wrong.                                       
 ; 
  (defun *error* (shk)
   (setvar "pickstyle" picks)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Get existing and replacement strings.                                 
 ; 
  (if (= (type cc) 'STR)
      (progn
           (setq ccp (getstring t (strcat "\nExisting string <" cc ">:")))
           (if (/= ccp "") (setq cc ccp)))
      (progn
           (setq cont t)
           (while cont
                 (setq cc (getstring t "\nOld string:"))
                 (if (= cc "")
                 (princ "Can't search for nothing.")
                 (setq cont nil)))))
  (setq dd (getstring t "Replacement string:"))
 ; Ŀ
 ;   Make a list of blocks not to change, deduce the marker size.          
 ; 
  (setq notouch ())
  (setq rad (/ (getvar "viewsize") 45))
 ; Ŀ
 ;   Get entities to change.                                               
 ; 
  (prompt "\nSelect text and blocks or <Return> for entire drawing: ")
  (setq glist '((-4 . "<or")
                 (0 . "text") (0 . "mtext")
                 (-4 . "<and") (66 . 1) (0 . "insert") (-4 . "and>")
                (-4 . "or>")))
  (if (null (setq ss0 (ssget glist)))
      (setq ss0 (ssget "x" glist)))
 ; Ŀ
 ;   Call Chall to change the entities.                                    
 ; 
  (chall cc dd ss rad notouch)
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (*error* nil)
 (princ))