 ; Ŀ
 ;   Coo - draws lines until it hears a <Return>, then inserts a N & E     
 ;   coordinate string based on the angle of the first line segment.       
 ;   Cot - update existing coordinate.                                     
 ;   Copyright 1994, 1997, 2012 by Rocket Software Ltd.                    
 ;   Armadillo popsicles, the fad food of 2003 - get your tongue ready.    
 ; 

 ; Ŀ
 ;   Subroutine Crodin - insert coordinate text.                           
 ;   Takes five arguments: the angle, insertion point, string, scale, and  
 ;   the associated polyline ename.                                        
 ; 
 (DEFUN CRODIN (angrad pa str scal phandl /)
 ; Ŀ
 ;   Change to the correct layer, make it if it doesn't exist.             
 ; 
  (if (tblsearch "layer" "text2")
      (setvar "clayer" "text2")
      (command "layer" "m" "text2" "c" "1" "" ""))
 ; Ŀ
 ;   Find the correct text position and angle, draw it.                    
 ; 
  (cond ((or (>= angrad (* pi 1.75)) (< angrad (* pi 0.25)))  ; right
         (setq pa (polar pa 0 (* scal 2)))
         (command "text" "ml" pa (* 2.5 scal) "0" str))
        ((and (> angrad (* pi 0.25)) (<= angrad (* pi 0.75))) ; top
         (setq pa (polar pa (/ pi 2) (* scal 2)))
         (command "text" "ml" pa (* 2.5 scal) "90" str))
        ((and (> angrad (* pi 0.75)) (<= angrad (* pi 1.25))) ; left
         (setq pa (polar pa pi (* scal 2)))
         (command "text" "mr" pa (* 2.5 scal) "0" str))
        ((and (>= angrad (* pi 1.25)) (< angrad (* pi 1.75))) ; down
         (setq pa (polar pa (* pi 1.5) (* scal 2)))
         (command "text" "mr" pa (* 2.5 scal) "90" str)))
 (princ))
 ; Ŀ
 ;   Crodin end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Ptaer - convert the X and Y coordinates of a point to      
 ;   N and E offsets from 0,0, return as a list.                           
 ; 
 (DEFUN PTAER (pa / xx yy len)
 ; Ŀ
 ;   If we're in model space use the point as is, if we're in ps then      
 ;   translate them into model space.                                      
 ;   So 1 or three use as is, 2 translate.                                 
 ; 
  (setq cspac (wasp))
  (if (= cspac 2)
      (setq pa (trans pa 3 2)))
 ; Ŀ
 ;   The next two lines have the (read (rtos)) because Autolisp insisted   
 ;   on fixing 165143.0 in the LL&E drawing 92107718 to 165142.  This      
 ;   addition sets the value in amber before fixing it.                    
 ;   But why the bug?                                                      
 ;   Fix truncates rather than rounds off a real.  The position of the     
 ;   block in question was 165142.999999999, which was displayed rounded   
 ;   up but Fixed down.  One could apply a rounding correction, but this   
 ;   way, as in life, what you see is what you expect.                     
 ; 
  (setq xx (itoa (fix (read (rtos (car pa) 2 2)))))
  (setq yy (itoa (fix (read (rtos (cadr pa) 2 2)))))
 ; Ŀ
 ;   (setq xx (itoa (fix (car pa))))       The original code.              
 ;   (setq yy (itoa (fix (cadr pa))))                                      
 ; 
  (while (< (strlen xx) 5)
         (setq xx (strcat "0" xx)))
  (while (< (strlen yy) 5)
         (setq yy (strcat "0" yy)))
  (if (< (strlen xx) 6)
         (setq xx (strcat "0" xx)))
  (if (< (strlen yy) 6)
         (setq yy (strcat "0" yy)))
  (setq len (strlen xx))
  (setq xx (strcat "E " (substr xx 1 (- len 3))
                    "." (substr xx (- len 2) 2)))   ; change the last 2
  (setq len (strlen yy))
  (setq yy (strcat "N " (substr yy 1 (- len 3))
                    "." (substr yy (- len 2) 2)))   ; to a 3 for mm
 (list xx yy))
 ; Ŀ
 ;   Ptaer end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Toaxt - Text To something.                                 
 ;   Arguments: Rasp, a replacement string.                                
 ;              Proma, a prompt, if nil then use the string in Rasp.       
 ;   Calls nothing, returns the ename of the selected text or nil.         
 ;   Breeds budgies.                                                       
 ; 
 (defun TOAXT (rasp proma / snapp nent enam entt typ outer)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq snapp (getvar "snapmode"))
  (setvar "snapmode" 0)
  (defun *error* (shk /)
   (setvar "snapmode" snapp)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
  (princ))
  (if (null proma)
      (setq proma (strcat rasp ": ")))
  (setq enam (car (setq nent (nentsel proma))))
  (setq typ (cdr (assoc 0 (setq entt (entget enam)))))
  (if (or (= "TEXT" typ) (= "MTEXT" typ) (= "ATTDEF" typ) (= "ATTRIB" typ))
      (progn
           (setq outer (car (reverse (car (reverse nent)))))
           (entmod (subst (cons 1 rasp) (assoc 1 entt) entt))
           (entupd enam)
           (if (= (type outer) 'ENAME) (entupd outer))))
  (setvar "snapmode" snapp)
  (command "undo" "end")
 enam)
 ; Ŀ
 ;   Subroutine Toaxt end.                                                 
 ; 

 ; Ŀ
 ;   Subroutine Wasp - are we in the Model Space Tab, Paper Space, or      
 ;   a viewport in Paper Space?                                            
 ;   Brooks no Arguments.                                                  
 ;   Calls nothing.                                                        
 ;   Returns: 1 - Model space.                                             
 ;            2 - Paper space.                                             
 ;            3 - A Viewport in Paper Space.                               
 ; 
 (DEFUN WASP ()
  (cond ((= (getvar "tilemode") 1) 1)  ; model
        ((= (getvar "cvport") 1) 2)    ; paper space
        (t 3)))                        ; a ps viewport
 ; Ŀ
 ;   Wasp end.                                                             
 ; 

 ; Ŀ
 ;   Cot.                                                                  
 ; 
 (DEFUN C:COT ( / clay disc aa bb stangl ang str ptlst)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq aa (getpoint "Point: "))
  (setq ptlst (ptaer aa))
  (setq str (strcat "(" (car ptlst) ", " (cadr ptlst) ")"))
  (toaxt str nil)
  (command "undo" "end")
 (princ))

 ; Ŀ
 ;   Coo.                                                                  
 ; 
 (DEFUN C:COO ( / clay disc aa bb stangl ang str ptlst)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq clay (getvar "clayer"))
  (setq disc (getvar "dimscale"))
  (setq aa (getpoint "Start point:\n"))
  (setq ptlst (ptaer aa))
  (setq str (strcat "(" (car ptlst) ", " (cadr ptlst) ")"))
  (command "pline" aa)
  (while (setq bb (getpoint aa "Next point:\n"))
         (if (= (distance aa bb) 0) (write-line "Try moving the cursor\n")
             (progn
                  (if (null stangl) (setq stangl (angle aa bb)))
                  (command bb)
                  (setq ang (angle aa bb))
                  (if bb (setq aa bb)))))
  (command "")
  (setq plenam (entlast))
  (setq phandl (cdr (assoc 5 (entget plenam))))
  (setq aa (polar aa ang (* disc 2)))
  (crodin ang aa str disc phandl)
  (setvar "clayer" clay)
  (command "undo" "end")
 (princ))

(princ)
