 ; Ŀ
 ;   Cotx - neaten text by row and column.                                 
 ;   Copyright 2000, 2002, 2003, 2013 by Rocket Software Ltd.              
 ;                                                                         
 ; 

 ; Ŀ
 ;   Subroutine Colly: sort entities into columns.      2013.10.02         
 ;   Text is sorted on the insertion, other entity types by ten point.     
 ;   Arguments: ss, an ss of entities to columnise.                        
 ;              Slop, a maximum acceptable slop distance.                  
 ;                                                                         
 ;   Calls: Horiz/Lowest, Vtol/Spit, Spit, and Liss.                       
 ;   Returns a list of lists: each sublist contains all the enames in a    
 ;   column in vertical order, preceded by the average column x position.  
 ; 
 (DEFUN COLLY (ss slop / num enam entt ten xpos xlist xa totalx numofx sub
                                                                column malist)
 ; Ŀ
 ;   Make the ((x_insert ename) ...) list.                                 
 ; 
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (setq entt (entget enam))
         (if (member (cdr (assoc 0 entt)) '("TEXT" "ATTDEF"))
             (setq ten (spit entt))
             (setq ten (cdr (assoc 10 entt))))
         (setq xpos (car ten))
         (setq xlist (cons (list xpos enam) xlist)))
 ; Ŀ
 ;   Order the list Xlist by first value.                                  
 ; 
  (setq xlist (horiz xlist))
 ; Ŀ
 ;   Columnise:                                                            
 ;   Add to the first sublist until reach the cutoff difference in X pos,  
 ;   then start a new sublist, continue until run out of sublists.         
 ; 
  (setq xa (caar xlist))
  (setq num 0)
  (setq totalx 0)
  (setq numofx 0)
  (while (setq sub (nth num xlist))
         (setq num (1+ num))
         (setq xpos (car sub))
 ; Ŀ
 ;   If the x position is still within the slop distance of the first      
 ;   one in the column, add the ename to the current column sublist.       
 ; 
         (if (<= (abs (- xpos xa)) slop)
             (progn
                  (setq numofx (1+ numofx))
                  (setq totalx (+ totalx xpos))
                  (setq column (cons (cadr sub) column)))
 ; Ŀ
 ;   If the x position is outside the slop distance then start a new       
 ;   column sublist.                                                       
 ; 
             (progn
                  (setq column (vtol (liss column) "y" t))
                  (setq column (cons (/ totalx numofx) column))
                  (setq malist (append malist (list column)))
                  (setq totalx xpos)
                  (setq numofx 1)
                  (setq xa xpos)
                  (setq column (list (cadr sub))))))
 ; Ŀ
 ;   There last column won't by definition set off the distance checker,   
 ;   so add it to malist.                                                  
 ; 
  (setq column (vtol (liss column) "y" t))
  (setq column (cons (/ totalx numofx) column))
  (setq malist (append malist (list column)))
 ; Ŀ
 ;   Return the list of lists.                                             
 ; 
 malist)
 ; Ŀ
 ;   Subroutine Colly end.                                                 
 ; 

 ; Ŀ
 ;   Subroutine Horiz - put a list in order by the first number in each    
 ;   sublist.  Takes one argument, a list, which it returns in order       
 ;   from smallest to largest first element.                               
 ;   Rehashed 2007.10.02: the first number may not be a string.            
 ; 
 (DEFUN HORIZ (nexlst / low lowp nxtsub hrzlst newlst orderd)
  (while nexlst
        (setq low (lowest nexlst))                 ; lowest leading number
        (while (and nexlst (setq nxtsub (nth 0 nexlst)))
               (setq lowp (car nxtsub))
               (if (= 'str (type lowp)) (setq lowp (read lowp)))
               (if (equal low lowp)
                   (setq hrzlst (append hrzlst (list nxtsub)))
                   (setq newlst (append newlst (list nxtsub))))
               (setq nexlst (cdr nexlst)))          ; remove 1st ent from list
        (setq orderd (append orderd hrzlst))        ; add lev sublst to levels
        (setq hrzlst ())                            ; set to () for next loop
        (setq nexlst newlst)                        ; nexlst reconstituted
        (setq newlst ()))                           ; empty new list & reuse
  orderd)
 ; Ŀ
 ;   Horiz end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Liss: make a list of entity names into an ss.              
 ;   Arguments: Liz, a list of enames.                                     
 ;   Returns an ss.                                                        
 ; 
 (DEFUN LISS (liz / num ss enam)
  (setq num 0)
  (setq ss (ssadd))
  (while (setq enam (nth num liz))
         (ssadd enam ss)
         (setq num (1+ num)))
 ss)
 ; Ŀ
 ;   Subroutine Liss end.                                                  
 ; 

 ; Ŀ
 ;   Subroutine Lowest - find the smallest leading number in a sublist     
 ;   of the list Nexlst which is the sole argument.                        
 ;   Revised 2007.10.02 - leading numbers can be ints, reals, or number    
 ;   strings.                                                              
 ; 
 (DEFUN LOWEST (nexlst / num minlst neth carna)
  (setq num 0)
  (setq minlst (list min))
  (while (setq neth (nth num nexlst))
         (setq carna (car neth))
         (if (= 'str (type carna)) (setq carna (read carna)))
         (if neth (setq minlst (append minlst (list carna))))
         (setq num (1+ num)))
  (eval minlst))
 ; Ŀ
 ;   Lowest end.                                                           
 ; 

 ; Ŀ
 ;   Milt - middle or centre rejustify a text or attdef entity.            
 ; 
 (DEFUN MILT (enam justa / aa ten new10 dist angl new11 nu11)
  (setq aa (entget enam))
  (setq ten (cdr (assoc 10 aa)))                   ; starting 10 pos
  (setq aa (subst (cons 72 justa) (assoc 72 aa) aa))   ; change to M
  (entmod (subst (cons 73 0) (assoc 73 aa) aa))    ; change MM to M normal
  (setq aa (entget enam))                          ; get the changed ent. data
  (setq new10 (cdr (assoc 10 aa)))                 ; new 10 point
  (setq dist (distance ten new10))                 ; distance moved
  (setq angl (angle new10 ten))                    ; and angle
  (setq new11 (cdr (assoc 11 aa)))                 ; new centre point
  (setq nu11 (polar new11 angl dist))              ; move centre same as 10 was
  (entmod (subst (cons 11 nu11) (assoc 11 aa) aa))
 (princ))
 ; Ŀ
 ;   Milt end.                                                             
 ; 

 ; Ŀ
 ;   Sna - put a number on snap.                                           
 ;   Takes two arguments, the number and the matching snap increment.      
 ;   Returns the nearest snap value.                                       
 ; 
 (DEFUN SNA (pax snapx / nxtup nxtdn distup distdn)
  (if (minusp pax)
      (progn
           (setq nxtup (* snapx (fix (/ pax snapx))))       ; next snap pt up
           (setq nxtdn (* snapx (1- (fix (/ pax snapx)))))) ; next snap pt dn
      (progn
           (setq nxtup (* snapx (1+ (fix (/ pax snapx)))))  ; next snap pt up
           (setq nxtdn (* snapx (fix (/ pax snapx))))))     ; next snap pt dn
  (setq distup (abs (- nxtup pax)))               ; distance to next higher
  (setq distdn (abs (- pax nxtdn)))               ; distance to next lower
  (if (> distup distdn)                           ; if further up
      (setq pax nxtdn)                            ; then go down
      (setq pax nxtup))                           ; else go up
 pax)
 ; Ŀ
 ;   Sna end.                                                              
 ; 

 ; Ŀ
 ;   Spit - returns the insertion point of the text entity whose data was  
 ;   passed as its sole argument.  Note that this is not necessarily the   
 ;   same as the 10 association code.                                      
 ; 
 (DEFUN SPIT (entt / xjust yjust)
  (setq xjust (cdr (assoc 72 entt)))
  (if (= (cdr (assoc 0 entt)) "ATTDEF")
      (setq yjust (cdr (assoc 74 entt)))
      (setq yjust (cdr (assoc 73 entt))))
  (if (or (/= xjust 0) (/= yjust 0))
      (cdr (assoc 11 entt))
      (cdr (assoc 10 entt))))
 ; Ŀ
 ;   Spit end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Steve: calculate a standard deviation.                     
 ;   Arguments: Nums - a list of numeric values.                           
 ;   Returns a number.                                                     
 ; 
 (DEFUN STEVE (nums / totl digit len averag sum)
 ; Ŀ
 ;   Get an average value.                                                 
 ; 
  (setq totl 0)
  (mapcar '(lambda (digit)
                   (setq digit (float digit))
                   (setq totl (+ totl digit))) nums)
  (setq averag (/ totl (setq len (length nums))))
 ; Ŀ
 ;   Sum all: square of (value -average).                                  
 ; 
  (setq sum 0)
  (mapcar '(lambda (digit)
                   (setq sum (+ sum (expt (- digit averag) 2))))
            nums)
 ; Ŀ
 ;   Divide by (n-1), get the square root.                                 
 ; 
 (sqrt (/ sum (1- len))))
 ; Ŀ
 ;   Steve end.                                                            
 ; 


 ; Ŀ
 ;   VBCX - centre rejustify a column of text.                             
 ;   Not currently used.                                                   
 ;   Saved in case need to make centre rather than middle rejustify the    
 ;   text, or if want to middle one line and centre > 1.                   
 ;   Takes two arguments - an ss and a centre point x coordinate.          
 ;   Returns zilch.                                                        
 ; 
 (DEFUN VBCX (ss xa / xa num enam entt pty)
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (setq entt (entget enam))
         (setq pty (cddr (assoc 10 entt)))
         (setq pa (cons xa pty))
         (setq entt (subst (cons 72 1) (assoc 72 entt) entt))
         (if (= typ "ATTDEF")
             (setq entt (subst (cons 74 0) (assoc 74 entt) entt))
             (setq entt (subst (cons 73 0) (assoc 73 entt) entt)))
         (entmod (subst (cons 11 pa) (assoc 11 entt) entt)))
 (princ))
 ; Ŀ
 ;   Vbcx end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Vtol: returns a list of enames ordered entity position.    
 ;   Arguments: Ss, a selection set of entities to order.                  
 ;              Dir, a direction - if this is either "X" or "Y" then the   
 ;                   entities are assumed to be arrayed in that direction, 
 ;                   if anything else then the routine uses the direction  
 ;                   in which they are most spread out.                    
 ;              Insa, if T and the entity is text or an attribute then     
 ;                    sort based on the insertion point rather than the    
 ;                    ten point.                                           
 ;                                                                         
 ;   This is the latest version: 2006.10.05, which sorts text by           
 ;   insertion point rather than ten point and in which setting the        
 ;   direction to nil doesn't cause a crash.                               
 ;   Also it works with attdefs as well as text.                           
 ;   It should replace all other uses of Vtol and Stol.                    
 ;                                                                         
 ;   Revamped 2009.07.28 to use Apply rather than Eval Cons 'Max List etc. 
 ;   This is less elegant but removes the 256 entity limitation.           
 ;   Also added the ability to sort by either ten point or insertion.      
 ;                                                                         
 ; 
 (DEFUN VTOL (ss dir insa / xposnam yposnam numm ent entt ten xpos ypos xx yy
                            pn maxx minx maxy miny xdif ydif poslst posnam
                                                       direct pos lastt order)
  (setq xposnam ())                      ; initialize (xpos & name list) list
  (setq yposnam ())                      ; initialize (ypos & name list) list
 ; Ŀ
 ;   Now see if the entities are arranged horizontally or vertically.      
 ; 
  (setq numm 0)                             ; start at the ss beginning again
  (while (setq ent (ssname ss numm))
         (setq entt (entget ent))
         (if (and insa (member (cdr (assoc 0 entt)) '("TEXT" "ATTDEF")))
             (setq ten (spit entt))
             (setq ten (cdr (assoc 10 entt))))
         (setq xpos (car ten))
         (setq ypos (cadr ten))
         (setq xx (append xx (list xpos)))  ; add x insert to list
         (setq yy (append yy (list ypos)))  ; and y to y list
 ; Ŀ
 ;   Also make the position and name list.  Have to make one for the X     
 ;   values and one for the Ys and use the appropriate one later.          
 ; 
         (setq pn (cons xpos ent))
         (setq xposnam (append xposnam (list pn)))
         (setq pn (cons ypos ent))
         (setq yposnam (append yposnam (list pn)))
         (setq numm (1+ numm)))             ; next entity
 ; Ŀ
 ;   Now evaluate the four lists.  The result will be the max and min      
 ;   values for the X and Y lists.                                         
 ; 
  (setq maxx (apply 'max xx))
  (setq minx (apply 'min xx))
  (setq maxy (apply 'max yy))
  (setq miny (apply 'min yy))
  (setq xdif (- maxx minx))
  (setq ydif (- maxy miny))
 ; Ŀ
 ;   Set direction variables to match whichever direction was given in     
 ;   the argument, if it was nil then deduce a direction.                  
 ; 
  (cond ((and (= (type dir) 'STR)
              (= (strcase dir) "X"))
          (setq poslst xx)                ; positions from X coord list
          (setq posnam xposnam)           ; position & ename list with X coord
          (setq direct 'min))             ; edit from smallest to largest X
        ((and (= (type dir) 'STR)
              (= (strcase dir) "Y"))
         (setq poslst yy)
         (setq posnam yposnam)
         (setq direct 'max))
        (T
 ; Ŀ
 ;   The default case: figure it out yourself.                             
 ;   Set vert to T if vertical, nil if horizontal.                         
 ;   If not sure, assume vertical.                                         
 ;   Could set strip to Quit and thus do so...                             
 ; 
         (cond ((> xdif ydif)             ; if (Xmax - Xmin) > (Ymax - Ymin)
                (setq poslst xx)          ; positions from X coord list
                (setq posnam xposnam)     ; position & ename list with X coord
                (setq direct 'min))       ; edit from smallest to largest X
               ((< xdif ydif)
                (setq poslst yy)
                (setq posnam yposnam)
                (setq direct 'max))
               (T                         ; if not sure then call it vertical
                (setq poslst yy)
                (setq posnam yposnam)
                (setq direct 'max)))))
 ; Ŀ
 ;   Now make the list of enames in order by increasing X or decreasing Y  
 ;   coordinate depending on whether the array is horizontal or vertical.  
 ;   Already Have Posnam: a list of (list position ename).                 
 ;   Using the original list of either x or y positions, get the first or  
 ;   last as appropriate, extract the ename from Posnam using              
 ;   (cdr (assoc (largest Y or smallest X) posnam))                        
 ;   and append the ename to the end of the enames in order list: Order.   
 ;   Then remove that position from the position list.                     
 ; 
  (while (> (length poslst) 0)
 ; Ŀ
 ;   Get the largest Y or smallest X value in the position list.           
 ; 
         (setq maxx (apply direct poslst))
 ; Ŀ
 ;   Having found Maxx, want to remove that value from poslst.             
 ;   Get the list from Maxx on, and the position of Maxx within the list.  
 ; 
         (setq pos (- (length poslst)
                      (length (setq lastt (member maxx poslst)))))
 ; Ŀ
 ;   Get the list after maxx.                                              
 ; 
         (setq lastt (cdr lastt))
 ; Ŀ
 ;   And add the list members before maxx.                                 
 ;   One could use (cdr (member (reverse poslist))) but if there were two  
 ;   values the same in the list this would result in a longer rather      
 ;   than a shorter poslist.                                               
 ; 
         (setq pos (1- pos))     ; subtract one: nth is zero based
         (while (>= pos 0)
                (setq lastt (append (list (nth pos poslst)) lastt))
                (setq pos (1- pos)))
         (setq poslst lastt)      ; poslst becomes lastt
 ; Ŀ
 ;   Now get the matching ename from posnam and add it to the end of the   
 ;   order list.                                                           
 ; 
         (setq order (append order (list (cdr (assoc maxx posnam)))))
 ; Ŀ
 ;   If there are two entities with the same position then assoc will      
 ;   always return the first one.  Must delete the first one each time -   
 ;   subst (nil) for it.                                                   
 ; 
         (setq posnam (subst (list nil) (assoc maxx posnam) posnam)))
 order)
 ; Ŀ
 ;   Vtol end.                                                             
 ; 

 ; Ŀ
 ;   VVB - vertically respace a column of text.                            
 ;   Takes two arguments - an ss and a centre point y coordinate.          
 ;   Returns nothing.                                                      
 ; 
 (DEFUN VVB (ss yins incr / txa enn nna ya txb yb txh yy nn yins)
  (while (setq txa (ssname ss 0))             ; first entity name
         (setq enn 1)                         ; entity to test - initialize
         (setq nna (entget txa))              ; the whole thing
         (setq ya (cdr (assoc 10 nna)))       ; Y insertion
 ; Ŀ
 ;   Find the highest entity.                                              
 ; 
         (while (setq txb (ssname ss enn))                 ; next entity
                (setq yb (cdr (assoc 10 (entget txb))))    ; Y insertion
                (if (> (cadr yb) (cadr ya))                ; if txb highest
                    (progn
                         (setq txa txb)                    ; next becomes txa
                         (setq nna (entget txa))           ; get whole thing
                         (setq ya (cdr (assoc 10 nna)))))  ; and Y insertion
                (setq enn (1+ enn)))                       ; next entity
 ; Ŀ
 ;   And move it.                                                          
 ; 
         (if (or (= (cdr (assoc 72 nna)) 2)
                 (= (cdr (assoc 72 nna)) 4)
                 (= (cdr (assoc 72 nna)) 1))
             (progn
                   (if (= (cdr (assoc 72 nna)) 4)
                       (progn
                             (setq txh (cdr (assoc 40 (entget (ssname ss 0)))))
                             (setq yy (cdr (assoc 11 nna)))
                             (setq nn (list (car yy) (+ (/ txh 2) yins)))
                             (command "move" txa "" yy nn))
                       (progn
                             (setq yy (cdr (assoc 11 nna)))
                             (setq nn (list (car yy) yins))
                             (command "move" txa "" yy nn))))
             (progn
                   (setq yy (cdr (assoc 10 nna)))
                   (setq nn (list (car yy) yins))
                   (command "move" txa "" yy nn)))
 ; Ŀ
 ;   Increment insertion point, remove entity from ss, loop.               
 ; 
         (setq yins (- yins incr))
         (ssdel txa ss))
 (princ))
 ; Ŀ
 ;   VVB end.                                                              
 ; 

 ; Ŀ
 ;   Subroutine Wits - find the width of a text entity.                    
 ;   Takes one argument: the text entity data list.  Returns a width.      
 ; 
 (DEFUN WITS (entt / tblist cc dd)
  (setq tblist (textbox entt))
  (setq cc (car tblist))                    ; ll offset from 10 of text
  (setq dd (cadr tblist))                   ; ur offset from 10 of text
  (- (car dd) (car cc)))
 ; Ŀ
 ;   Wits end.                                                             
 ; 

 ; Ŀ
 ;   Cotx.                                                                 
 ; 
 (DEFUN C:COTX (/ picks snapp osna snaps snapx snapy ss pa pay num enam pax
                                        widx wid0 plist fuzz malist sub incr)
  (command "undo" "be")
 ; Ŀ
 ;   Turn off group selection - otherwise entities in groups may           
 ;   be changed twice with bad things happening as a result.               
 ; 
  (setq picks (getvar "pickstyle"))
  (setvar "pickstyle" 0)
  (setq snapp (getvar "snapmode"))
  (setvar "snapmode" 0)
  (setq osna (getvar "osmode"))
  (setvar "osmode" 0)
 ; Ŀ
 ;   Something is bound to go wrong.                                       
 ; 
  (defun *error* (shk)
   (setvar "pickstyle" picks)
   (setvar "snapmode" snapp)
   (setvar "osmode" osna)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Make the variables for point snapping.                                
 ; 
  (setq snaps (getvar "snapunit"))
  (setq snapx (car snaps))
  (setq snapy (cadr snaps))
 ; Ŀ
 ;   Get entities.                                                         
 ; 
  (if (setq ss (ssget (list (cons 0 "text"))))
      (progn
 ; Ŀ
 ;   Get the Y ordinate of the insertion point of the highest one.         
 ; 
           (setq pa (spit (entget (car (vtol ss "y" t)))))
           (setq pay (sna (cadr pa) snapy))
 ; Ŀ
 ;   Convert them all to centre justified in place in case they're all     
 ;   left or all different ...                                             
 ; 
           (setq num 0)
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (milt enam 1))
 ; Ŀ
 ;   Extract the insertion points, get a standard deviation, use this as   
 ;   the allowable distance before an entity is considered to be in        
 ;   a new column.                                                         
 ; 
           (setq num 0)
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq pax (cadr (spit (entget enam))))
                  (setq plist (cons pax plist)))
           (setq fuzz (steve plist))
 ; Ŀ
 ;   If there are a small number of entities the standard deviation can    
 ;   end up being much less than the text width, so a column can be        
 ;   treated as two and the entities end up on top of each other - at      
 ;   the same Y and near Xs.                                               
 ;   So don't let Fuzz be less than half the smallest text width.          
 ;   Typically text doesn't overlap, but people do strange things.         
 ;   a new column.                                                         
 ; 
           (setq num 0)
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq wixd (wits (entget enam)))
                  (if (or (null wid0)
                          (< wixd wid0))
                      (setq wid0 wixd)))
           (setq wid0 (/ wid0 4))
           (if (> wid0 fuzz)
               (setq fuzz wid0))
 ; Ŀ
 ;   Call Colly to extract the entities into columns.                      
 ; 
          (setq malist (colly ss fuzz))))
 ; Ŀ
 ;   Centre each column on the centre point, put it on snap first.         
 ; 
  (setq num 0)
  (while (setq sub (car malist))
         (setq malist (cdr malist))
         (setq num (1+ num))
         (setq pax (sna (car sub) snapx))
         (setq sub (cdr sub))
         (setq incr (* 1.65 (cdr (assoc 40 (entget (car sub))))))
         (vbcx (liss sub) pax)
         (vvb (liss sub) pay incr))
 ; Ŀ
 ;   End so the pigs aren't killed by the impact.                          
 ; 
  (*error* nil)
 (princ))
