 ; Ŀ
 ;   Crr - put an arrowhead inside a circle where it intersects a line.    
 ;   Ccr - put an arrowhead outside a circle where it intersects a line.   
 ;   Copyright 1994, 2001, 2010, 2011 by Rocket Software Ltd.              
 ;   Rocket - the barnacles of software.                                   
 ; 

 ; Ŀ
 ;   Gent - get the first entity of a given type at a point.               
 ;   Arguments: Pa, a point.                                               
 ;              Diss, the search area size, theoretically dimscale.        
 ;              Etype, the entity type string.  Apparently "line,pline"    
 ;                     etc. can be used.                                   
 ;   Calls nothing, returns an ename.                                      
 ; 
 (DEFUN GENT (pa diss etype / pa0 pa1)
  (setq pa0 (polar pa (/ pi 4) diss))
  (setq pa1 (polar pa (* 1.25 pi) diss))
 (ssname (ssget "c" pa0 pa1 (list (cons 0 etype))) 0))
 ; Ŀ
 ;   Gent end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Crr - make an arrowhead given a circle and line.           
 ;   Arguments: Ins, t = inside the circle, nil = outside.                 
 ;   Calls Gent, Returns nothing.                                          
 ; 
 (DEFUN CRR (ins / osmo snapp blip *error* disc bxsiz cept inpt lin ang s1 s2
                                                                       entt)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
  (setq osmo (getvar "osmode"))
  (setq snapp (getvar "snapmode"))
  (setvar "snapmode" 0)
  (setq blip (getvar "blipmode"))
  (setvar "blipmode" 0)
 ; Ŀ
 ;   Make a nice local error handler.                                      
 ; 
  (defun *error* (shk /)
   (setvar "osmode" osmo)
   (setvar "snapmode" snapp)
   (setvar "blipmode" blip)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
   (if shk (write-line shk))
  (princ))
 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))
 ; Ŀ
 ;   Get the scale, depending on which space we are in and other things.   
 ; 
  (if misps
      (setq disc (misps))
      (setq disc (getvar "dimscale")))
  (setq bxsiz disc)
  (setq disc (* disc (getvar "dimasz")))
 ; Ŀ
 ;   Ask nicely for a circle centre.  Probably an arc will do too.         
 ; 
  (setvar "osmode" 5)
  (setq cept (getpoint "Centre point/Line start: "))
 ; Ŀ
 ;   And then the intersection between the line and the circle.            
 ; 
  (setvar "osmode" 32)
  (setq inpt (getpoint cept "Line and Circle Intersection: "))
 ; Ŀ
 ;   Get the line-like entity at the intersection point.                   
 ;   Hope it will be the line you want.                                    
 ; 
  (setq lin (entget (gent inpt bxsiz "line,leader,polyline,lwpolyline")))
 ; Ŀ
 ;   Draw the arrowhead.                                                   
 ; 
  (if ins
      (setq ang (angle inpt cept))
      (setq ang (angle cept inpt)))
  (setq s1 (polar inpt ang disc))
  (setq s2 (polar s1 (+ ang (/ pi 2)) (/ disc 6)))
  (setq s1 (polar s2 (+ ang (* pi 1.5)) (/ disc 3)))
  (setvar "osmode" 0)
  (command "solid" s1 s2 inpt "" "")
  (setvar "osmode" osmo)
 ; Ŀ
 ;   Make it the same colour as the line.                                  
 ; 
  (setq entt (entget (entlast)))
  (cond ((and (assoc 62 entt) (assoc 62 lin))
         (setq entt (subst (assoc 62 lin) (assoc 62 entt) entt)))
        ((assoc 62 lin)
         (setq entt (append entt (list (assoc 62 lin))))))
 ; Ŀ
 ;   And put it on the layer with the line.                                
 ; 
  (entmod (subst (assoc 8 lin) (assoc 8 entt) entt))
 ; Ŀ
 ;   End neatly.                                                           
 ; 
  (emp ())
 (princ))
 ; Ŀ
 ;   Subroutine Crr end.                                                   
 ; 

 ; Ŀ
 ;   Ccr.                                                                  
 ; 
 (DEFUN C:CCR ()
  (crr nil)
 (princ))

 ; Ŀ
 ;   Crr.                                                                  
 ; 
 (DEFUN C:CRR ()
  (crr t)
 (princ))
