 ; Ŀ
 ;   Endevi - insert a loop style 2/3/4 wire device block.                 
 ;   Copyright 2013 by Rocket Software Ltd.                                
 ;                                                                         
 ; 

 ; Ŀ
 ;   Subroutine Gnang - get a point and angle for a line.                  
 ;   Argument: Pa, a point.                                                
 ;             Snark, put the point on snap.                               
 ;   Returns a list: a point and an angle or nil.                          
 ; 
 (DEFUN GNANG (pa snark / pe px pm angg)
 ; Ŀ
 ;   Find the nearest endpoint.                                            
 ; 
  (if (setq px (osnap pa "end"))                   ; look for an endpoint
      (setq pe px))                                ; if one then use it
 ; Ŀ
 ;   And the midpoint.                                                     
 ; 
  (if (setq px (osnap pa "mid"))                   ; look for a midpoint
      (setq pm px))                                ; if one then use it
 ; Ŀ
 ;   See if there's an angle.                                              
 ; 
  (if (and pe pm)
      (progn
           (setq angg (angle pm pe))))
           (setq angg (/ (* angg 180) pi))
 ; Ŀ
 ;   If snark is set then put the end point Pe on snap.                    
 ; 
  (if snark (setq pe (snale pe t)))
 ; Ŀ
 ;   Return what you have.                                                 
 ; 
 (if angg
     (list pe angg)
     pa))
 ; Ŀ
 ;   Subroutine Gnang end.                                                 
 ; 

 ; Ŀ
 ;   Grizz - a replacement for the entsel function - returns a list in the 
 ;   (ename point) format unless there was no entity at that point, in     
 ;   which case (nil point) is returned.                                   
 ;   Takes one argument, a prompt string.                                  
 ; 
 (DEFUN GRIZZ (prom / pa ss)
  (write-line prom)
  (while (and (setq pa (grread t 5 2))
              (/= (car pa) 3)))
  (if (setq ss (ssget (setq pa (cadr pa))))
      (list (ssname ss 0) pa)
      (list () pa)))
 ; Ŀ
 ;   Grizz end.                                                            
 ; 

 ; Ŀ
 ;   Sna - put a number on snap.                                           
 ;   Takes two arguments, the number and the matching snap increment.      
 ;   Returns the nearest snap value.                                       
 ; 
 (DEFUN SNA (pax snapx / nxtup nxtdn distup distdn)
  (if (minusp pax)
      (progn
           (setq nxtup (* snapx (fix (/ pax snapx))))       ; next snap pt up
           (setq nxtdn (* snapx (1- (fix (/ pax snapx)))))) ; next snap pt dn
      (progn
           (setq nxtup (* snapx (1+ (fix (/ pax snapx)))))  ; next snap pt up
           (setq nxtdn (* snapx (fix (/ pax snapx))))))     ; next snap pt dn
  (setq distup (abs (- nxtup pax)))               ; distance to next higher
  (setq distdn (abs (- pax nxtdn)))               ; distance to next lower
  (if (> distup distdn)                           ; if further up
      (setq pax nxtdn)                            ; then go down
      (setq pax nxtup))                           ; else go up
 pax)
 ; Ŀ
 ;   Sna end.                                                              
 ; 

 ; Ŀ
 ;   Snale - put an association list on snap.                              
 ;   Arguments: Sub, a list.                                               
 ;              Do1, if t then do all elements, else leave nth 0 as is.    
 ;   Returns the modified list.                                            
 ; 
 (DEFUN SNALE (sub do1 / snaps snapx snapy sub1 sub2 sub3)
  (setq snaps (getvar "snapunit"))
  (setq snapx (car snaps))
  (setq snapy (cadr snaps))
  (if do1
      (progn
           (setq sub1 (sna (car sub) snapx))
           (setq sub2 (sna (cadr sub) snapy))
           (if (setq sub3 (caddr sub))
               (setq sub3 (sna sub3 snapx))))
      (progn
           (setq sub1 (car sub))
           (setq sub2 (sna (cadr sub) snapx))
           (if (setq sub3 (caddr sub))
               (setq sub3 (sna sub3 snapy)))))
 (if (= (length sub) 3)
     (list sub1 sub2 sub3)
     (list sub1 sub2 sub3)))
 ; Ŀ
 ;   Snale end.                                                            
 ; 

 ; Ŀ
 ;   Endevi.                                                               
 ; 
 (DEFUN C:ENDEVI (/ osmo clay *error* blnam pa angg)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq osmo (getvar "osmode"))
  (setq clay (getvar "clayer"))
  (defun *error* (shk)
   (setvar "attreq" 1)
   (setvar "osmode" osmo)
   (setvar "clayer" clay)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
  (princ))
 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))
 ; Ŀ
 ;   Ask for a file name.                                                  
 ; 
  (if (null blpnam)
      (setq blpnam "2wireloopdevice"))
  (setq blnam (strcase (getstring t (strcat "Block Name <" blpnam ">: ")) t))
  (if (= blnam "")
      (setq blnam blpnam))
  (if (not (wcmatch blnam "*wireloopdevice"))
      (write-line "Unacceptable Block Name.")
      (progn
           (setq blpnam blnam)
 ; Ŀ
 ;   Get a point.                                                          
 ; 
           (setq pa (grizz "Insertion: "))
 ; Ŀ
 ;   See what's there.                                                     
 ; 
           (if (car pa)
               (progn
                    (setq angg (gnang (cadr pa) nil))  ; t to put on snap
                    (setq pa (car angg))
                    (setq angg (cadr angg)))
               (progn
                    (setq pa (cadr pa))
                    (setq pa (snale pa t))  ; put on snap
                    (setq angg 180)))       ; so doesn't mirror
           (setvar "attreq" 0)
           (command "insert" blnam pa 1 "" 0)
           (setvar "attreq" 1)
 ; Ŀ
 ;   The block may need to be mirrored.                                    
 ; 
           (if (or (null angg)
                   (< angg 90)
                   (> angg 270))
               (command ".mirror" (entlast) "" pa (polar pa (/ pi 2) 10) "y"))
 ; Ŀ
 ;   Offer to edit the block insertion.                                    
 ; 
           (command ".ddatte" (entlast))))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (*error* nil)
 (princ))
