 ; Ŀ
 ;   Fook - Font convert - make all fonts using Romans.shx into Standard.  
 ;   Copyright 1992 - 2012 by Rocket Software Ltd.                         
 ;   Notes: 1. Vertical styles are ignored.  Backwards styles should be    
 ;             as well but they are vanishingly uncommon.                  
 ;          2. Attributes may have different styles in the block tables    
 ;             and in the inserts, but this can be allowed for if it       
 ;             ever happens.                                               
 ;          3. This program assumes that Standard uses Romans and is the   
 ;             desired style.  It could be modified to use Simplex.        
 ;          4. Xref styles, blocks, and dimstyles are ignored.             
 ;                                                                         
 ; 

 ; Ŀ
 ;   Blook - change a text style in the block tables.                      
 ;   Notes: 1. Entnext returns nil after the last entity in a block        
 ;             definition.                                                 
 ;          2. An empty block has one subentity of type Endblk.            
 ;   Arguments: Stylis, a list of styles to change.                        
 ;              Gnust, a new style name.                                   
 ;   Calls nothing, Returns a list of the names of blocks in which         
 ;   attdefs were changed.  (Text and mtext don't matter because they      
 ;   update when the block definition is changed.)                         
 ; 
 (DEFUN BLOOK (stylis gnust / reww bldat blnam namm etype cstyl blist isxrf)
  (setq reww T)
  (while (setq bldat (tblnext "block" reww))
         (setq reww ())
 ; Ŀ
 ;   See if the block is an xref and thus can't have attributes and        
 ;   shouldn't be messed with anyway.                                      
 ; 
         (setq isxrf (= 4 (logand (cdr (assoc 70 bldat)) 4)))
         (if (and (/= "*" (substr (setq blnam (cdr (assoc 2 bldat))) 1 1))
                  (null isxrf))
             (progn
                  (setq namm (cdr (assoc -2 bldat)))
                  (while (and namm (setq entt (entget namm)))
                         (setq etype (cdr (assoc 0 entt)))
                         (setq cstyl (cdr (assoc 7 entt)))
                         (if (and (member cstyle stylis)
                                  (member etype '("TEXT" "ATTRIB" "MTEXT")))
                             (progn
                                  (entmod (subst (cons 7 gnust)
                                                 (assoc 7 entt) entt))
                                  (if (and (not (member blnam blist))
                                           (= etype "ATTRIB"))
                                      (setq blist (cons blnam blist)))))
                         (setq namm (entnext namm))))))
 blist)
 ; Ŀ
 ;   Blook end.                                                            
 ; 

 ; Ŀ
 ;   Chook - change the style of attributes in an ss of block insertions.  
 ;   Arguments: Ss, the selection set of blocks.                           
 ;              Stylis, the list of style names to change.                 
 ;              Gnust, the new style name.                                 
 ;   Calls nothing, Returns a bag of rhubarb.                              
 ; 
 (DEFUN CHOOK (ss stylis gnust / num enam entt cstyl asoc7 pa)
  (setq rad (/ (getvar "viewsize") 55))
  (setq num 0)   ; position in ss
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (while (/= "SEQEND" (cdr (assoc 0 (setq entt
                                       (entget (setq enam (entnext enam)))))))
                (setq cstyl (strcase (cdr (setq asoc7 (assoc 7 entt))) t))
                (if (member cstyl stylis)
                    (progn
                         (entmod (subst (cons 7 gnust) asoc7 entt))
                         (setq pa (spit entt))
                         (stack pa rad)))))
 (princ))
 ; Ŀ
 ;   Chook end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Cstar - draw an individual grstar (centred).               
 ;   Takes four arguments: centre point, side length, rotation (radians),  
 ;   and colour.  Returns nothing, but draws a star.                       
 ; 
 (DEFUN CSTAR (pa sidlen rota colo / anginc angg hafang pb)
  (setq px pa)
  (setq pa (polar pa (+ rota (/ pi 2)) (* sidlen 1.37638192)))
  (setq anginc (* 1.6 pi))
  (setq angg (+ rota (* 1.6 pi)))
  (setq hafang (* 0.8 pi))
  (repeat 5
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg anginc))
         (setq pa pb)
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg hafang))
         (setq pa pb))
 (princ))
 ; Ŀ
 ;   Subroutine Cstar end.                                                 
 ; 

 ; Ŀ
 ;   Dook - change text styles in the dimstyle tables.                     
 ;   Arguments: Stylis, a list of styles to change.                        
 ;              Gnust, a new style name.                                   
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN DOOK (stylis gnust / reww ddat stlnam cstyl dentt)
 ; Ŀ
 ;   Step through the dimstyle tables.                                     
 ; 
  (setq reww T)
  (while (setq ddat (tblnext "dimstyle" reww))
         (setq reww ())
 ; Ŀ
 ;   See if the dimstyle is part of an xref.                               
 ; 
         (if (/= 16 (logand (cdr (assoc 70 ddat)) 16))
             (progn
                  (setq dimnm (cdr (assoc 2 ddat)))
                  (setq stlnam (cdr (setq asoc34 (assoc 340 ddat))))
                  (setq cstyl (strcase (cdr (assoc 2 (entget stlnam))) t))
                  (if (member cstyl stylis)
                      (progn
                           (command "dimtxsty" gnust)
                           (command "dimstyle" "save" dimnm "y"))))))
  (princ))
 ; Ŀ
 ;   Dook end.                                                             
 ; 

 ; Ŀ
 ;   Fok - get a list of all text styles using a given font.               
 ;   Ignores Standard and any vertical styles.                             
 ;   Arguments: Fontx, the font name to find.                              
 ;              Stylx, the desired style - i.e. one to ignore.             
 ;   Returns a list of styles.                                             
 ; 
 (DEFUN FOK (fontx stylx / rew stylx stylc stylst isvert isxrf)
  (setq fontx (strcase fontx t))
  (setq stylx (strcase stylx t))
 ; Ŀ
 ;   Get a list of styles defined in the drawing.                          
 ; 
  (setq rew t)
  (while (setq stylx (tblnext "style" rew))
         (setq stylc (strcase (cdr (assoc 2 stylx)) t))
         (setq isvert (= 4 (logand 4 (cdr (assoc 70 stylx)))))
         (setq isxrf (= 16 (logand 16 (cdr (assoc 70 stylx)))))
         (if (and (= fontx (strcase (cdr (assoc 3 stylx)) t))
                  (null isvert)
                  (null isxrf)
                  (/= stylc stylx))
             (setq stylst (cons (strcase (cdr (assoc 2 stylx)) t) stylst)))
         (setq rew ()))
 stylst)
 ; Ŀ
 ;   Subroutine Fok end.                                                   
 ; 

 ; Ŀ
 ;   Subroutine Ici - grdraw circle maker.                                 
 ; 
 (DEFUN ICI (pa radd reps colo / angg incr pa1 pa2)
  (setq angg 0)
  (setq incr (/ pi (/ reps 2)))
  (setq pa1 (polar pa angg radd))
  (repeat reps
          (setq angg (+ angg incr))
          (setq pa2 (polar pa angg radd))
          (grdraw pa1 pa2 colo)
          (setq pa1 pa2))
 (princ))
 ; Ŀ
 ;   Ici end.                                                              
 ; 

 ; Ŀ
 ;   Isxnam: see if a given block is an xref.                              
 ;   Argument: Blnam, the block name.                                      
 ;   Returns T: it was an xref, or nil: it wasn't, or no such block is     
 ;   is defined in the drawing.                                            
 ; 
 (DEFUN ISXNAM (blnam / isxrf xp dat)
  (if (setq dat (tblsearch "block" blnam))
      (progn
           (setq xp (cdr (assoc 70 dat)))
           (setq isxrf (logand xp 4))))
 (if (= isxrf 4) T ()))
 ; Ŀ
 ;   Isxnam end.                                                           
 ; 

 ; Ŀ
 ;   Mark - mark a point.                                                  
 ;   Arguments: Pa - the point to mark.                                    
 ;              Rad - the marker segment length.                           
 ;              Colo - the marker grdraw line colour.                      
 ; 
 (DEFUN MARK (pa rad colo /)
  (grdraw (polar pa (/ pi 4) rad) (polar pa (* 1.25 pi) rad) colo)
  (grdraw (polar pa (* pi 0.75) rad) (polar pa (* pi 1.75) rad) colo)
 (princ))
 ; Ŀ
 ;   Mark end.                                                             
 ; 

 ; Ŀ
 ;   Spit - returns the insertion point of the text entity whose data was  
 ;   passed as its sole argument.  Note that this is not necessarily the   
 ;   same as the 10 association code.                                      
 ; 
 (DEFUN SPIT (entt / xjust yjust)
  (setq xjust (cdr (assoc 72 entt)))
  (if (= (cdr (assoc 0 entt)) "ATTDEF")
      (setq yjust (cdr (assoc 74 entt)))
      (setq yjust (cdr (assoc 73 entt))))
  (if (or (/= xjust 0) (/= yjust 0))
      (cdr (assoc 11 entt))
      (cdr (assoc 10 entt))))
 ; Ŀ
 ;   Spit end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Stack - draw a grstar and two circles.                     
 ;   Arguments: Pa, the common centre point.                               
 ;              Rad, the side length of a star.                            
 ;   Calls Ici and Cstar, Returns nothing.                                 
 ; 
 (DEFUN STACK (pa rad)
  (ici pa (* rad 0.75) 32 2)
  (ici pa (* rad 0.9) 32 8)
  (cstar pa rad 0 1)
 (princ))
 ; Ŀ
 ;   Stack end.                                                            
 ; 

 ; Ŀ
 ;   Stixx - make sure a text style is correct.                            
 ;   Arguments: Stylx, the style to fix.                                   
 ;              Fontx, the font to use.                                    
 ; 
 (DEFUN STIXX (stylx fontx)
  (command ".style"
           stylx        ; style name
           fontx        ; font file
           "0"          ; don't preset height (ever)
           "1.0"        ; width scale
           "0"          ; obliquing angle
           "n"          ; backwards
           "n"          ; upside down
           "n")         ; vertical (only for vertical enabled .shx fonts)
  (setvar "textstyle" stylx)
 (princ))
 ; Ŀ
 ;   Stixx end.                                                            
 ; 

 ; Ŀ
 ;   Took - change the style in text entities.                             
 ;   Arguments: Stylis, a list of styles to change.                        
 ;              Gnust, a new style name.                                   
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN TOOK (stylis gnust / ss enam entt num asoc7)
 ; Ŀ
 ;   Get an ss of all textlike entities.                                   
 ; 
  (setq ss (ssget "x" '((-4 . "<or") (0 . "text")
                                     (0 . "mtext") (0 . "attdef")
                        (-4 . "or>"))))
 ; Ŀ
 ;   Process the text.                                                     
 ; 
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (setq entt (entget enam))
         (if (member (strcase (cdr (setq asoc7 (assoc 7 entt))) t) stylis)
             (entmod (subst (cons 7 gnust) asoc7 entt))))
 (princ))
 ; Ŀ
 ;   Subroutine Took end.                                                  
 ; 

 ; Ŀ
 ;   Fook.                                                                 
 ; 
 (DEFUN C:FOOK (/ fontx stylx stylis blist num ss)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq fontx "Romans.shx")  ; the font to use
  (setq stylx "Standard")    ; the desired style
 ; Ŀ
 ;   Make sure the desired (usaually Standard) text style is correct.      
 ; 
  (stixx stylx fontx)
 ; Ŀ
 ;   Get a list of styles other than Standard which use Romans.shx.        
 ; 
  (setq stylis (fok fontx stylx))
 ; Ŀ
 ;   Change all text, attdefs and mtext which use one of these styles      
 ;   to use Standard.                                                      
 ; 
  (took stylis stylx)
 ; Ŀ
 ;   Similarly change all text, attdefs and mtext in the block tables.     
 ;   Blook returns a list of block names which have been changed.          
 ; 
  (setq blist (blook stylis stylx))
 ; Ŀ
 ;   Change attributes in insertions of these blocks.                      
 ; 
  (setq num 0)
  (while (and blist (setq blnam (nth num blist)))
         (setq num (1+ num))
         (setq ss (ssget "x" (list (cons 2 blnam))))
         (chook ss stylis stylx))        
 ; Ŀ
 ;   Change text styles in dimstyles.                                      
 ; 
  (dook stylis stylx)
 ; Ŀ
 ;   End neatly.                                                           
 ; 
  (command "undo" "end")
 (princ))