 ; Ŀ
 ;   Fuu: fix the width of an existing block to fit its attribute string.  
 ;   Fuj: make them all the same (maximum required) width.                 
 ;   Copyright 1994, 1995, 2007, 2013 by Rocket Software Ltd.              
 ;                                                                         
 ; 

 ; Ŀ
 ;   Blasz - find the size of a rectangle which will surround a block.     
 ;   Argument: Tagblk, a block name.  If the block isn't available then    
 ;             blasz will try to load it, if it can't it returns nil.      
 ;   Returns a list: (ins_point left_dist right_dist up_dist down_dist)    
 ;   (Presumably the insertion point has some practical use.)              
 ; 
 (DEFUN BLASZ (tagblk / plist pta lhdis rhdis updis dndis)
 ; Ŀ
 ;   If the tag block isn't available then insert it so the definition     
 ;   can be looked at.                                                     
 ; 
  (if (null (tblsearch "block" tagblk))
      (progn
           (command ".insert" tagblk)
           (command)))
 ; Ŀ
 ;   Load the block corner point finder from Puss.lsp.                     
 ; 
  (if (not bent) (load "puss"))
 ; Ŀ
 ;   Get a list of the insertion point and block extents.                  
 ;   Bent returns a list: (insertion_point  x-max  x-min  y-max  y-min).   
 ; 
  (setq plist (bent tagblk))
  (setq pta (car plist))
  (setq rhdis (+ (car pta) (cadr plist)))         ; right hand (+x) distance
  (setq lhdis (abs (+ (car pta) (caddr plist))))  ; left hand (-x) distance
  (setq updis (+ (cadr pta) (cadddr plist)))      ; top (+y) distance
  (setq dndis (abs (+ (cadr pta) (nth 4 plist)))) ; bottom (-y) distance
 (if plist (list pta lhdis rhdis updis dndis)))
 ; Ŀ
 ;   Blasz end                                                             
 ; 

 ; Ŀ
 ;   Twid - find the width of a hypothetical text string.                  
 ;   Takes one argument, the text entity data list.                        
 ;   Returns a width (a number).                                           
 ; 
 (DEFUN TWID (entt / tblist cc dd bwidth)
  (setq tblist (textbox entt))
  (setq cc (car tblist))                    ; ll offset from 10 of text
  (setq dd (cadr tblist))                   ; ur offset from 10 of text
  (setq bwidth (- (car dd) (car cc))))
 ; Ŀ
 ;   Twid end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Bwx.                                                       
 ;   Arguments: Enam, the ename of the block to process.                   
 ;              Attwid, the attribute width at width scale 1,              
 ;                      or nil to get it from the current block.           
 ;   Calls Blasz and Twid.                                                 
 ;   Returns nothing.                                                      
 ; 
 (DEFUN BWX (enam attwid / entt pa tagblk blins dalist blwid stra etta attwid
                                             deltax dimscl lefty ettpa asoc11)
  (setq entt (entget enam))
  (setq pa (cdr (assoc 10 entt)))
  (setq tagblk (cdr (assoc 2 entt)))
  (setq blins (cdr (assoc 10 entt)))
 ; Ŀ
 ;   Call Blasz to find how big the block is.                              
 ;   Blasz returns a list:                                                 
 ;   (insertion_point  left_distance  right_distance  up_dist  down_dist)  
 ; 
  (setq dalist (blasz tagblk))
  (setq blwid (+ (cadr dalist) (caddr dalist)))        ; overall block width
 ; Ŀ
 ;   Extract the attribute string.                                         
 ; 
  (setq stra (cdr (assoc 1 (setq etta (entget (entnext enam))))))
  (if (null attwid)
 ; Ŀ
 ;   Find the attribute width using the given string.                      
 ; 
      (setq attwid (twid (list (cons 1 stra) (cons 40 2.5) (cons 41 1.0)))))
 ; Ŀ
 ;   Overall required block width is the attribute/string width plus 4,    
 ;   and the block X scale is that divided by the overall block width.     
 ; 
  (setq attwid (+ attwid 4))             ; required block new width in units
  (setq scalf (/ attwid blwid))          ; block x scale to get that
  (setq deltax (- (* (cadr dalist) scalf)
                  (cadr dalist)))  ; movement of block edge
 ; Ŀ
 ;   Rescale the block.                                                    
 ; 
  (setq dimscl (getvar "dimscale"))
  (entmod (subst (cons 41 (* scalf dimscl)) (assoc 41 entt) entt))
 ; Ŀ
 ;   Adjust the attribute to 1 width scale.                                
 ; 
  (entmod (subst (cons 41 1) (assoc 41 etta) etta))
  (entupd enam)
 ; Ŀ
 ;   Move the attribute back into place.  This assumes that it is left     
 ;   justified rather than centre.                                         
 ;   The block will have increased in width by deltax so move the          
 ;   attribute to the left by half of that.                                
 ;   No - that doesn't work if you run the program again.                  
 ;   Calculate the location of the left edge.                              
 ;   The block lefthand width is the left block definition width x the     
 ;   x block scale -2.                                                     
 ; 
  (setq blins (cdr (assoc 10 (setq entt (entget enam)))))
  (setq lefty (* (cadr dalist) (cdr (assoc 41 entt))))
  (setq ettpa (polar blins pi (- lefty 2)))
  (setq asoc11 (assoc 11 etta))
  (entmod (subst (cons 11 ettpa) asoc11 etta))
 (princ))
 ; Ŀ
 ;   Subroutine Bwx end.                                                   
 ; 

 ; Ŀ
 ;   Fuu.                                                                  
 ; 
 (DEFUN C:FUU (/ ss num enam)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
  (setq ss (ssget (list (cons 2 "cabletag"))))
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (bwx enam nil))
  (command ".undo" "end")
 (princ))

 ; Ŀ
 ;   Fuj - set all selected cable tags to the same (adequate) width.       
 ; 
 (DEFUN C:FUJ (/ ss num enam stra etta attwid maxwid)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
  (setq ss (ssget (list (cons 2 "cabletag"))))
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
 ; Ŀ
 ;   Extract the attribute string.                                         
 ; 
         (setq stra (cdr (assoc 1 (setq etta (entget (entnext enam))))))
 ; Ŀ
 ;   Find the attribute width using the given string.                      
 ; 
         (setq attwid (twid (list (cons 1 stra) (cons 40 2.5) (cons 41 1.0))))
 ; Ŀ
 ;   Save the maximum width.                                               
 ; 
         (if (or (null maxwid)
                 (> attwid maxwid))
             (setq maxwid attwid)))
 ; Ŀ
 ;   Process the ss.                                                       
 ; 
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (bwx enam maxwid))
  (command ".undo" "end")
 (princ))