 ; Ŀ
 ;   Fw - Adjust Hocs date attribute widths.                               
 ;   Copyright 1995, 1997, 1999, 2002 - 2010 by Rocket Software Ltd.       
 ;   Fix Width ... or something else.                                      
 ; 

 ; Ŀ
 ;   Subroutine Hlister - make the list of blocks, attributes, and widths. 
 ;   The format is:                                                        
 ;   '(Blockname (Attribute_tag1 Maximum_width Ideal_width_scale) ...)     
 ;   Notes: 1. Don't forget the leading quote.                             
 ;          2. Block and attribute names are enclosed in double quotes,    
 ;             attribute width isn't.                                      
 ;          3. Widths are automatically adjusted for block scale.          
 ;          4. Names must be in uppercase.                                 
 ;          5. You can rewidth some attributes and ignore others in the    
 ;             same block.                                                 
 ;          6. Named attributes which don't exist in the block aren't a    
 ;             problem.                                                    
 ;          7. Blocks which aren't present in the drawing are ok.          
 ;          8. If this seems pretty foolproof then you don't know your     
 ;             coworkers very well.                                        
 ;                                                                         
 ;   Updated: now allows wild cards in tag names.                          
 ;   To do: allow wild cards in block names.                               
 ; 
 (DEFUN HLISTER ()
  (list '("HOCS ATTRIBUTES TEXT"  ("REV_DATE_*" 14 1)
                                  ("REF_DWG_TITLE_*" 103 1)
                                  ("DRAWN_DATE" 15 1)
                                  ("REV_CHK_*" 8 1)
                                  ("REV_APP_*" 8 1)
                                  ("CHK_DATE" 15 1)
                                  ("APPROVED_DATE" 15 1))
        '("HOCS ATTRIBUTE TEXT"   ("REV_DATE_*" 14 1)
                                  ("REF_DWG_TITLE_*" 103 1)
                                  ("DRAWN_DATE" 15 1)
                                  ("REV_CHK_*" 8 1)
                                  ("REV_APP_*" 8 1)
                                  ("CHK_DATE" 15 1)
                                  ("APPROVED_DATE" 15 1))))
 ; Ŀ
 ;   Hlister end.                                                          
 ; 

 ; Ŀ
 ;   Subroutine Wits - find the width of an attribute.                     
 ;   Takes one argument: the attribute entity data list.  Returns a width. 
 ; 
 (DEFUN WITS (entt / tblist cc dd bwidth)
  (setq tblist (textbox entt))
  (setq cc (car tblist))                    ; ll offset from 10 of text
  (setq dd (cadr tblist))                   ; ur offset from 10 of text
  (setq bwidth (- (car dd) (car cc))))
 ; Ŀ
 ;   Wits end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Vise - squeeze or stretch attributes as required.          
 ;   Arguments: Enam, a block ename.                                       
 ;              Widlst, the corresponding list of attrib. and width lists. 
 ;              Blscal, the block scale.                                   
 ; 
 (DEFUN VISE (enam widlst blscal / esav entt enam num sublst namp width realwd
                                                          scalfc widscl prev41)
  (setq esav enam)
 ; Ŀ
 ;   Repeat until come to the block end marker.                            
 ; 
  (while (/= (cdr (assoc 0 (setq entt (entget (setq enam (entnext enam))))))
                                                                   "SEQEND")
 ; Ŀ
 ;   Get the attribute name.                                               
 ; 
         (setq namm (cdr (assoc 2 entt)))
 ; Ŀ
 ;   Call Wcasoc to see if there is a matching sublist.                    
 ; 
         (if (setq sublst (wcasoc namm widlst ()))
             (progn
 ; Ŀ
 ;   Find the correct width from the sublist, allow for the block scale.   
 ; 
                  (setq width (* (cadr sublst) blscal))
 ; Ŀ
 ;   Find the desired width scale factor from the sublist.                 
 ; 
                  (setq txtscl (caddr sublst))
 ; Ŀ
 ;   Call Wits to find the actual string width.                            
 ; 
                  (setq realwd (wits entt))
 ; Ŀ
 ;   Find the attribute width scale factor.                                
 ; 
                  (setq widscl (cdr (setq prev41 (assoc 41 entt))))
 ; Ŀ
 ;   Compare the actual and desired widths.                                
 ; 
                  (if (and (> realwd width)
                           (not (equal realwd width 0.1)))
 ; Ŀ
 ;   If the actual width is greater than the allowed width in the          
 ;   sublist, then adjust the width scale factor to make it fit.           
 ;   Wait: must also check to see if the width scale is greater than the   
 ;   ideal - if the attribute is too wide and the width scale is too       
 ;   large, then shrinking the attribute to fit may result in it just      
 ;   filling the space but still being too wide.                           
 ; 
                      (progn
 ; Ŀ
 ;   Compare the actual width scale to the ideal width scale.              
 ; 
                           (if (<= widscl txtscl)
 ; Ŀ
 ;   If the actual is less than or equal to the ideal, then make it fit.   
 ; 
                               (progn
                                    (setq scalfc (/ width realwd))
                                    (setq widscl (* widscl scalfc))
                                    (entmod (subst (cons 41 widscl)
                                                    prev41 entt)))
 ; Ŀ
 ;   If the actual width scale is greater than the ideal, see if the       
 ;   attribute will be too wide if if set to the ideal.                    
 ;   If so then squash to fit, if not then set to the ideal.               
 ; 
                               (progn
                                    (if (> (* realwd (/ txtscl widscl)) width)
 ; Ŀ
 ;   Squash to fit.                                                        
 ; 
                                        (progn
                                             (setq scalfc (/ width realwd))
                                             (setq widscl (* widscl scalfc))
                                             (entmod (subst (cons 41 widscl)
                                                             prev41 entt)))
 ; Ŀ
 ;   Set to the ideal width scale factor.                                  
 ; 
                                        (entmod (subst (cons 41 txtscl)
                                                        prev41 entt))))))
 ; Ŀ
 ;   Else the actual width is narrower than or equal to the available      
 ;   space.                                                                
 ; 
                      (progn
 ; Ŀ
 ;   See if the attribute is narrower than it should be - if setting the   
 ;   width scale factor to the desired value would leave the attribute     
 ;   wider than the allowable space, then increase it to fill the space.   
 ; 
                           (if (> (* realwd (/ txtscl widscl)) width)
                               (progn
                                    (setq scalfc (/ width realwd))
                                    (setq widscl (* widscl scalfc))
                                    (entmod (subst (cons 41 widscl)
                                                           prev41 entt)))
 ; Ŀ
 ;   Otherwise set it to the ideal width scale value.                      
 ; 
                               (progn
                                    (entmod (subst (cons 41 txtscl)
                                                       prev41 entt)))))))))
  (entupd esav)
 (princ))
 ; Ŀ
 ;   Vise end.                                                             
 ; 

 ; Ŀ
 ;   WcAsoc - Assoc function with wild cards.                              
 ;   Arguments: Carra, a hypothetical leading string.                      
 ;              Lisa, a list of lists, possibly containing wild cards.     
 ;   In other words this thing is backwards.                               
 ;   Would checking a b and b a work?  Maybe.........                      
 ;              Casa, case sensitive-p: t = yes, nil = no.                 
 ;   Won't crash if the leading atom in each list isn't a string, but      
 ;   may be baffled.                                                       
 ;   Returns a sublist or nil.                                             
 ; 
 (DEFUN WCASOC (carra lisa casa / sub num aso str)
  (if (null casa) (setq carra (strcase carra)))
  (setq num 0)
  (while (and (null aso) (setq sub (nth num lisa)))
         (setq num (1+ num))
         (setq str (car sub))
         (if (= (type str) 'STR)
             (progn
                  (if (null casa) (setq str (strcase str)))
                  (if (wcmatch carra str)
                      (setq aso sub)))))
 aso)
 ; Ŀ
 ;   WcAsoc end.                                                           
 ; 

 ; Ŀ
 ;   FW.                                                                   
 ; 
 (DEFUN C:FW (/ blscal blist num sub ss ssnum enam entt)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq blist (hlister))
  (setq num 0)
  (while (setq sub (nth num blist))
         (setq num (1+ num))
         (if (setq ss (ssget "X" (list (cons 2 (car sub)))))
             (progn
                  (setq ssnum 0)
                  (while (setq enam (ssname ss ssnum))
                         (setq ssnum (1+ ssnum))
                         (setq entt (entget enam))
                         (setq blscal (abs (cdr (assoc 41 entt))))
                         (if (assoc 66 entt)
                             (vise enam (cdr sub) blscal))))))
  (command "undo" "end")
 (princ))