 ; Ŀ
 ;   Gore - draw spherical gores.                                          
 ;   Copyright 2013, 2014 by Rocket Software Ltd.                          
 ;                                                                         
 ; 
 (DEFUN C:GORE (/ ppa goresp slicsp slice hite circum1 dize radia dizh hypot
                                               circp width radi0 enama pb)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
  (setvar "osmode" 0)
 ; Ŀ
 ;   Get the centre point.                                                 
 ; 
  (if (null pa) (setq pa (getvar "viewctr")))
  (setq ppa (getpoint pa "Gore Base Centre <Current>: "))
  (if ppa (setq pa ppa))
 ; Ŀ
 ;   Get the circumference.                                                
 ; 
  (if (/= (type circum) 'REAL) (setq circum 27.5))
  (setq circp (getreal (strcat "Circumference <" (rtos circum 2 2) ">: ")))
  (if circp (setq circum circp))
  (setq hypot (/ circum pi 2))
  (write-line (strcat "\nRadius: " (rtos hypot)))
 ; Ŀ
 ;   The gore height is 1/4 of the circumference.                          
 ; 
  (setq goreht (/ circum 4))
 ; Ŀ
 ;   Get the number of slices.  Initget 6 = positive nonzero only.         
 ; 
  (initget 6)
  (if (/= (type slices) 'INT) (setq slices (fix (* goreht 10))))
  (setq slicsp (getint (strcat "\nNumber of Line Segments <"
                               (itoa slices) ">: ")))
  (if slicsp (setq slices slicsp))
 ; Ŀ
 ;   Get the slice thickness and the slice increment angle.                
 ; 
  (setq slice (/ goreht slices))
  (setq slang (/ pi 2 slices))
 ; Ŀ
 ;   Get the number of gores.                                              
 ; 
  (initget 6)
  (if (/= (type gores) 'INT) (setq gores 4))
  (setq goresp (getint (strcat "\nNumber of Gores <" (itoa gores) ">: ")))
  (if goresp (setq gores goresp))
 ; Ŀ
 ;   Get the vertical stretch factor.                                      
 ; 
  (initget 6)
  (if (/= (type vsfac) 'real) (setq vsfac 1.35))
  (setq vsfacp (getreal (strcat "\nVertical Stretch X <" (rtos vsfac) ">: ")))
  (if vsfacp (setq vsfac vsfacp))
 ; Ŀ
 ;   Initialize the height and angle counters, start the Polyline command. 
 ; 
  (setq hite 0)
  (setq anga 0)
  (command ".pline")
 ; Ŀ
 ;   Draw the gore half-outline.                                           
 ;   For each slice get the circle diameter at that height (Pythagorus)    
 ;   calculate the circumference, divide by the number of gores, that      
 ;   gives the gore width.                                                 
 ; 
  (repeat (1+ slices)
 ; Ŀ
 ;   First get the height above the y axis from the slice angle.           
 ;   This is the height on the base circle, not on the gore.               
 ; 
          (setq chite (* hypot (sin anga)))
 ; Ŀ
 ;   Find the distance across the circle at that height, i.e. the radius.  
 ; 
          (setq dizh (- (* hypot hypot) (* chite chite)))
          (if (< 0.00000001 dizh)   ; hack: sqrt fails on very small numbers
              (setq radia (sqrt dizh))
              (setq radia dizh))
 ; Ŀ
 ;   Calculate the circumference.                                          
 ; 
          (setq circm1 (* pi radia 2))
 ; Ŀ
 ;   The gore width is the circumference divided by the number of gores.   
 ; 
          (setq width (/ circm1 gores))
 ; Ŀ
 ;   Save the base width - the first value to go past.                     
 ; 
          (if (null radi0) (setq radi0 width))
 ; Ŀ
 ;   The height on the gore for that width will of course be the total     
 ;   of all the gore slices so far.                                        
 ;   Also the vertical scale factor is included.                           
 ; 
          (command (polar (polar pa (/ pi 2) (* vsfac hite)) 0 (/ width 2)))
          (setq hite (+ slice hite))
          (setq anga (+ anga slang)))
  (command "")
 ; Ŀ
 ;   Mirror the pline, join the two, close the resulting gore.             
 ; 
  (command ".mirror" (setq enama (entlast)) "" pa (polar pa (/ pi 2) 10) "n")
  (command ".pedit" enama "join" (entlast) "" "c" "")
 ; Ŀ
 ;   Add some descriptive text.                                            
 ; 
  (command ".text" "j" "m" (polar pa (/ pi 2) 3.75) 0.2 0
           (strcat "Circumference: " (rtos circum 2 2)))
  (command ".text" "" (strcat "Vertical Scale: " (rtos vsfac 2 2)))
  (command ".text" "" (strcat "2 x " (itoa gores) " Gores, join at base."))
  (command ".text" "" (strcat "Gore Height: " (rtos (* vsfac goreht) 2 2)))
  (command ".text" "" (strcat "Gore Width: " (rtos goreht 2 2)))
  (command ".text" "" "Pattern Size for 2\" Brim:")
  (command ".text" "" (strcat "Height: " (rtos (+ (* vsfac goreht 2) 7) 2 2)))
  (command ".text" "" (strcat "Width: " (rtos circum 2 2)))
  (command ".text" "" "Brim Section Height: 7")
  (setq daet (getvar "date"))
  (setq daet (fix (* 10000000 (- daet (fix daet)))))
  (command ".text" "" (strcat "Run No.: " (itoa daet)))
 ; Ŀ
 ;   Copy it (1- gores) times.                                             
 ; 
  (setq pb pa)
  (repeat (1- gores)
          (setq pb (polar pb 0 radi0))
          (command ".copy" enama "" pa pb))
 ; Ŀ
 ;   End politely.                                                         
 ; 
  (command ".undo" "end")
 (princ))