 ; Ŀ
 ;   Ido - make an iso dimension.                                          
 ;   Copyright 2013 by Rocket Software Ltd.                                
 ;   Isometrics - proof that no decision is without consequences.          
 ; 

 ; Ŀ
 ;   Subroutine Blug - dissect a bulged pline segment.                     
 ;   Arguments: Liza, the starting vertex data list.                       
 ;              Lizb, the ending vertex data list.                         
 ;              Diag, draw some temporary markers.                         
 ;   Returns a list:                                                       
 ;   (centre radius midpt mid_angle chord inside_ang start_ang end_ang     
 ;   point_where_tangents_to_both_ends_meet)                               
 ;   unless it's not a bulged segment, in which case nil.                  
 ; 
 (DEFUN BLUG (liza lizb diag / bulge incang pa pb chord rad chang chmid
                     pa1 pb1 ptanx cenang apothg cen midang arcmid arcdet)
  (if (and (setq bulge (cdr (assoc 42 liza)))
           (/= bulge 0))
      (progn
           (setq incang (* 4 (atan bulge))) ; included Angle
           (setq pa (cdr (assoc 10 liza)))  ; start point
           (setq pb (cdr (assoc 10 lizb)))  ; end point
           (setq chord (distance pa pb))    ; chord length
           (setq rad (/ (/ chord 2) (sin (* (abs incang) 0.5)))) ; arc radius
           (setq chang (angle pa pb))       ; chord angle
           (setq chmid (polar pa chang (/ chord 2))) ; chord midpoint
           (setq cenang (+ chang (/ pi 2)))          ; angle 90 from the chord
           (if (minusp incang) (setq cenang (+ cenang pi)))
           (setq apothg (* rad (cos (/ (abs incang) 2))))  ; apothegm - cen to chmid
           (setq cen (polar chmid cenang apothg))    ; the centre point
           (setq midang (angle cen chmid))           ; centre to arc mid angle
           (setq arcmid (polar cen midang rad))      ; arc midpoint
 ; Ŀ
 ;   Get the point at which lines tangent to both ends meet.               
 ; 
           (setq pa1 (polar pa (+ (/ pi 2) (angle cen pa)) 10))
           (setq pb1 (polar pb (+ (/ pi 2) (angle cen pb)) 10))
           (setq ptanx (inters pa pa1 pb pb1 nil))
 ; Ŀ
 ;   Draw some markers if diag is set.                                     
 ; 
           (if diag
               (progn
                    (grdraw pa pb 4)
                    (grdraw cen arcmid 2)
                    (grdraw cen pa 1)
                    (grdraw cen pb 1)
                    (grdraw pa ptanx 8)
                    (grdraw pb ptanx 8)))
           (setq arcdet (list cen rad arcmid midang chord incang
                              (angle cen pa) (angle cen pb) ptanx))))
 arcdet)
 ; Ŀ
 ;   Blug end.                                                             
 ; 

 ; Ŀ
 ;   Dimsty - make the two dim text styles.                                
 ;   Calls nothing, Returns nothing, Accepts no arguments.  Antisocial.    
 ; 
 (DEFUN DIMSTY ()
  (command ".style" "isoleft"  "simplex.shx" "0" "1.0" "-30" "n" "n" "n")
  (command ".style" "isoright" "simplex.shx" "0" "1.0"  "30" "n" "n" "n")
  (setvar "textstyle" "standard")
 (princ))
 ; Ŀ
 ;   Dimsty end.                                                           
 ; 

 ; Ŀ
 ;   Goto - grdraw an arrow.                                               
 ;   Arguments: bb, the start end of the arrow                             
 ;              aa, the point to which the arrow points.                   
 ;              colo, the arrow colour.                                    
 ;              hi, highlight the arrow if /= 0.                           
 ;   Notes:                                                                
 ;   1. any negative colour is equivalent to xor colour - dashed white -   
 ;      which erases itself on overwrite.                                  
 ;   2. 0 erases whatever is under it, and is undocumented.                
 ;   3. Highlighting must be turned on in the first grdraw call in the     
 ;      routine, and can't be turned off in the routine.                   
 ;   These may just be anomalies in this video system.                     
 ; 
 (DEFUN GOTO (bb aa colo hi / rad basic dist bhasic ang pa pb1 pb2)
  (setq rad (/ (getvar "viewsize") 50))
  (setq basic (/ (setq dist (distance aa bb)) 4))
  (if (> basic rad) (setq basic rad))
  (if (> basic (* dist 0.75)) (setq basic (* dist 0.75)))
  (setq bhasic (/ basic 2.25))
  (setq ang (angle aa bb))
  (setq pa (polar aa ang basic))
  (setq pb1 (polar pa (+ ang (/ pi 2)) bhasic))
  (setq pb2 (polar pa (+ ang (* pi 1.5)) bhasic))
  (grdraw aa pb1 colo hi) ; hilight must be in first call, doesn't turn off
  (grdraw pb1 pb2 colo)
  (grdraw aa pb2 colo)
  (grdraw bb pa colo)
 (princ))
 ; Ŀ
 ;   Goto end.                                                             
 ; 

 ; Ŀ
 ;   ***** Not fixed for lwplines yet. *****                               
 ;                                                                         
 ;   Pfx: find the lwpline, pline, or line segment a point lies on.        
 ;   Takes two arguments: the point and the pline ename.                   
 ;   Returns a list of lists: the two endpoints of each segment and the    
 ;   vertex entity name.  This is prefaced with a bulge factor if the      
 ;   segment was bulged, or "X" if it contained the pick point.            
 ;   Other entity types return nil.                                        
 ; 
 (DEFUN PFX (pa enam / typ entt closed pwid num entt2 enam end1 pasav end2 angg
                                   pb pc pint suba bulge seglst entt1 vnum sub)
  (setq typ (cdr (assoc 0 (setq entt (entget enam)))))
 ; Ŀ
 ;   Polyline.                                                             
 ;   The crossing length for intersections is the lw/pline width.          
 ; 
  (cond ((= typ "POLYLINE")
         (setq closed (if (= 1 (logand 1 (cdr (assoc 70 entt)))) t))
         (if (or (null (setq pwid (cdr (assoc 40 entt))))
                 (zerop pwid))
             (setq pwid 1))
         (setq num 0)
         (while (/= "SEQEND" (cdr (assoc 0 (setq entt2
                                       (entget (setq enam (entnext enam)))))))
                (setq num (1+ num))
                (if (and entt1 entt2)
                    (progn
                         (setq end1 (cdr (assoc 10 entt1)))
                         (if (null pasav) (setq pasav end1))
                         (setq end2 (cdr (assoc 10 entt2)))
                         (setq angg (angle end1 end2))
                         (setq pb (polar pa (+ angg (/ pi 2)) pwid))
                         (setq pc (polar pa (- angg (/ pi 2)) pwid))
                         (cond ((setq pint (inters end1 end2 pc pb))
                                (setq suba (list "X" end1 end2 enam1)))
                               ((and (setq bulge (cdr (assoc 42 entt1)))
                                     (null (zerop bulge)))
                                (setq suba (list bulge end1 end2 enam1)))
                               (t (setq suba (list end1 end2 enam1))))
                         (setq seglst (cons suba seglst))))
                (setq enam1 enam)
                (setq entt1 entt2))
 ; Ŀ
 ;   If the polyline was closed, add the implicit closed segment.          
 ; 
         (if closed
             (progn
                  (setq end1 pasav)
                  (setq angg (angle end1 end2))
                  (setq pb (polar pa (+ angg (/ pi 2)) pwid))
                  (setq pc (polar pa (- angg (/ pi 2)) pwid))
                  (setq pint (inters end1 end2 pc pb))))
                  (cond ((setq pint (inters end1 end2 pc pb))
                         (setq suba (list "X" end1 end2 enam1)))
                        ((and (setq bulge (cdr (assoc 42 entt1)))
                              (null (zerop bulge)))
                         (setq suba (list bulge end1 end2 enam1)))
                        (t (setq suba (list end1 end2 enam1))))
                  (setq seglst (cons suba seglst)))
 ; Ŀ
 ;   Lwpolyline.                                                           
 ; 
        ((= typ "LWPOLYLINE")
         (setq closed (if (= 1 (logand 1 (cdr (assoc 70 entt)))) t))
         (if (null (setq pwid (cdr (assoc 43 entt))))
             (setq pwid 1))
         (setq vnum (setq num 0))
         (while (setq sub (nth num entt))
                (setq num (1+ num))
                (if (= (car sub) 10)
                    (progn
                         (setq vnum (1+ vnum))
                         (setq end2 (cdr sub))
                         (if (null pasav) (setq pasav end2))
                         (if (and end1 end2 (null pint))
                             (progn
                                  (setq angg (angle end1 end2))
                                  (setq pb (polar pa (+ angg (/ pi 2)) pwid))
                                  (setq pc (polar pa (- angg (/ pi 2)) pwid))
                                  (if (inters end1 end2 pc pb)
                                      (setq pint (list end1 end2)))))
                         (if (null pint) (setq end1 end2)))))
 ; Ŀ
 ;   If the point wasn't on a segment and the lwpolyline was closed, try   
 ;   the implicit closed segment.                                          
 ; 
         (if (and (null pint) closed)
             (progn
                  (setq end2 pasav)
                  (setq angg (angle end1 end2))
                  (setq pb (polar pa (+ angg (/ pi 2)) pwid))
                  (setq pc (polar pa (- angg (/ pi 2)) pwid))
                  (if (inters end1 end2 pc pb)
                      (setq pint (list end1 end2)))))
         (if pint
            (cons vnum pint)
            (list vnum ())))
 ; Ŀ
 ;   Line.                                                                 
 ; 
        ((= typ "LINE")
         (setq end1 (cdr (assoc 10 entt)))
         (setq end2 (cdr (assoc 11 entt)))
         (setq angg (angle end1 end2))
         (setq pb (polar pa (+ angg (/ pi 2)) 0.000001))
         (if (inters end1 end2 pa pb nil)
            (setq seglst (list (list "X" end1 end2 enam1)))))
        (T nil))
 (reverse seglst))
 ; Ŀ
 ;   Pfx end.                                                              
 ; 

 ; Ŀ
 ;   Ido.                                                                  
 ; 
 (DEFUN C:IDO (/ ang30 ang60 ang90 ang120 ang150 ang210 ang240 ang270 ang300
                  ang330 enampt entt enam pa plist dalist iangg dangl dstyl
                              dstr obliq num sub preseg nxseg pt1 pt2 dimpt)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
 ; Ŀ
 ;   Set up the dimension text styles.                                     
 ; 
  (dimsty)
 ; Ŀ
 ;   Set up the dimension styles.                                          
 ; 
  (if (and (not (tblsearch "dimstyle" "left"))
           (findfile "dimleft.lsp"))
      (progn
           (load "dimleft")
           (c:dimleft)))
  (if (and (not (tblsearch "dimstyle" "right"))
           (findfile "dimright.lsp"))
      (progn
           (load "dimright")
           (c:dimright)))
 ; Ŀ
 ;   Make some angle variables.  180 is pi.                                
 ; 
  (setq ang30  (/ pi 6))
  (setq ang60  (/ pi 3))
  (setq ang90  (/ pi 2))
  (setq ang120 (* pi 0.75))
  (setq ang150 (* pi (/ 5 6.0)))
  (setq ang210 (* pi (/ 7 6.0)))
  (setq ang240 (* pi (/ 4 3.0)))
  (setq ang270 (* pi 1.5))
  (setq ang300 (* pi (/ 5 3.0)))
  (setq ang330 (* pi (/ 11 6.0)))
 ; Ŀ
 ;   Get an iso line.                                                      
 ; 
  (if (and (setq enampt (entsel "Iso Line: "))
           (setq entt (entget (setq enam (car enampt))))
           (setq pa (cadr enampt))
 ; Ŀ
 ;   Plist makes a list of vertices, marking those with bulges and the     
 ;   one containing the pick point.                                        
 ; 
           (setq plist (pfx pa enam))
           (setq dalist (cdr (assoc "X" plist))))
      (progn
           (setq iangg (angle (car dalist) (cadr dalist)))
 ;         (print iangg)     ; isometric line angle
           (setq dangl (getangle pa "Dimension Angle: "))
 ;         (print dangl)     ; dimension side and location angle
 ; Ŀ
 ;   Should now have an isometric line angle and a desired dim angle.      
 ;   Idealize the iso line angle.                                          
 ; 
           (setq iangg (rem iangg pi))
           (cond ((and (>= iangg 0) (< iangg ang60))
                  (setq iangg ang30))
                 ((and (>= iangg ang60) (< iangg ang120))
                  (setq iangg ang90))
                 ((and (>= iangg ang120) (< iangg pi))
                  (setq iangg ang150)))
 ; Ŀ
 ;   Idealize the dimension angle.                                         
 ; 
           (cond ((and (>= dangl 0) (< dangl ang60))
                  (setq dangl ang30))
                 ((and (>= dangl ang60) (< dangl ang120))
                  (setq dangl ang90))
                 ((and (>= dangl ang120) (< dangl pi))
                  (setq dangl ang150))
                 ((and (>= dangl pi) (< dangl ang240))
                  (setq dangl ang210))
                 ((and (>= dangl ang240) (< dangl ang300))
                  (setq dangl ang270))
                 ((and (>= dangl ang300) (< dangl (* pi 2)))
                  (setq dangl ang330)))
 ; Ŀ
 ;   Decide which text angle (and thus dim style) to use.                  
 ; 
           (cond ((= iangg ang30)
                  (cond ((or (= dangl ang90) (= dangl ang270))
                         (setq dstyl "right")
                         (setq obliq 90))
                        ((or (= dangl ang330) (= dangl ang150))
                         (setq dstyl "left")
                         (setq obliq 150))))
                 ((= iangg ang150)
                  (cond ((or (= dangl ang90) (= dangl ang270))
                         (setq dstyl "left")
                         (setq obliq 90))
                        ((or (= dangl ang30) (= dangl ang210))
                         (setq dstyl "right")
                         (setq obliq 30))))
                 ((= iangg ang90)
                  (cond ((or (= dangl ang150) (= dangl ang330))
                         (setq dstyl "right")
                         (setq obliq 330))    ; or 150?
                        ((or (= dangl ang210) (= dangl ang30))
                         (setq dstyl "left")
                         (setq obliq 210))))) ; or 30?
 ; Ŀ
 ;   Extract the section containing the pick point and the one on either   
 ;   side, if any.                                                         
 ; 
           (setq num 0)
           (while (setq sub (nth num plist))
                  (if (= (car sub) "X")
                      (progn
                           (if (> num 0)
                               (setq preseg (nth (1- num) plist)))
                           (setq nxseg (nth (1+ num) plist))
                           (setq num (1+ (length plist))))
                      (setq num (1+ num))))
 ; Ŀ
 ;   Get the two dimension endpoints.                                      
 ; 
           (if (and preseg (= (type (car preseg)) 'real))
               (setq pt1 (nth 8 (blug (entget (last preseg))
                                      (entget (last dalist))
                                      ())))
               (setq pt1 (car dalist)))
           (if (and nxseg (= (type (car nxseg)) 'real))
               (setq pt2 (nth 8 (blug (entget (setq elast (last nxseg)))
                                      (entget (entnext elast))
                                      ())))
               (setq pt2 (cadr dalist)))
 ;          (goto pt2 pt1 2 0)
 ;          (goto pt1 pt2 2 0)
 ; Ŀ
 ;   If the lines are vertical but not exactly, then adjust the points.    
 ;   This means that the dimension geometry won't lie exactly on the       
 ;   line, and won't match perfectly with the next one, so be careful.     
 ;   Actually this is a bad idea, even if it looks trivially neater.       
 ;   Later: Kent proposes a more rational solution.  Maybe.                
 ; 
 ;          (if (and (= iangg ang90)
 ;                   (not (equal (rem (angle pt1 pt2) ang90) 0 0.01)))
 ;              (progn
 ;                   (setq vdist (- (cadr pt1) (cadr pt2)))
 ;                   (setq pt2 (polar pt1 ang90 (- vdist)))))
 ; Ŀ
 ;   Get the dimension location.  Use an offset from an endpoint.          
 ; 
           (setq dimpt (polar pt1 dangl 10))
 ; Ŀ
 ;   Get a measurement.                                                    
 ; 
           (if (= "" (setq dstr (getstring t "Length <XXX>: ")))
               (setq dstr "XXX"))
 ; Ŀ
 ;   Draw the dimension.                                                   
 ;   If the iso line is vertical and the ends curve then the dim points    
 ;   may not be in a vertical line and the text may be upside down.        
 ;   This is okay if it leans to the right, but not to the left.           
 ;   This can be avoided by specifying a text angle in the dim aligned     
 ;   command if the dimension is vertical.                                 
 ; 
           (command "dimstyle" "restore" dstyl)
           (if (member obliq '(210 330))
               (command "dim" "aligned" pt1 pt2 "a" 90 dimpt dstr "exit")
               (command "dim" "aligned" pt1 pt2 dimpt dstr "exit"))
 ; Ŀ
 ;   Reprocess it.                                                         
 ; 
           (command ".dimedit" "o" (entlast) "" obliq)))
  (command ".undo" "end")
 (princ))