 ; Ŀ
 ;   Lager - subroutines to display a list of text strings in a dialog     
 ;   box and return a selected one.                                        
 ;   Also resizes the dialog on the fly.                                   
 ;   Copyright 1991, 2006, 2007, 2018 by Rocket Software Ltd.              
 ;   It's a good thing managers don't know that programming is fun.        
 ; 

 ; Ŀ
 ;   Drosop - rewrite a dcl file on the fly.                               
 ;   Arguments: Filnam, a dcl file name.                                   
 ;              Widnum, the desired dialog box width.                      
 ;              Htnum, the desired dialog box height.                      
 ;   Calls Yout, Word1, and Pout.  Returns nothing.                        
 ; 
 (DEFUN DROSOP (filnam widnum htnum / comlst num sub gnulst)
 ; Ŀ
 ;   The dialog box height is greater than the list length.                
 ; 
  (setq htnum (+ htnum 3)) ; for bottom lines plus one space
 ; Ŀ
 ;   The dialog box can't be bigger than the screen.                       
 ;   This should probably be tied to the actual screen size.               
 ; 
  (cond ((< widnum 32)
         (setq widnum 32))
        ((> widnum 165)
         (setq widnum 165)))   ; maximum width is 165 (on 1024 x 768)
  (cond ((< htnum 10)
         (setq htnum 10))
        ((> htnum 50)
         (setq htnum 50)))     ; maximum height is 50 (on 1024 x 768)
 ; Ŀ
 ;   Open, read, modify and rewrite the file.                              
 ; 
  (setq filnam (findfile "Fibl.dcl"))
  (setq comlst (yout filnam))
  (setq num 0)
  (while (setq sub (nth num comlst))
         (setq num (1+ num))
         (cond ((= (word1 sub) "width")
                (setq sub (strcat "                 width = "
                                  (itoa widnum) ";")))
               ((= (word1 sub) "height")
                (setq sub (strcat "                 height = "
                                  (itoa htnum) ";}"))))
         (setq gnulst (append gnulst (list sub))))
  (pout gnulst filnam)
 (princ))
 ; Ŀ
 ;   Drosop end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Lbox - display a list of strings in a dialog box.          
 ;   Arguments: Styldt, the list of strings to display.                    
 ;              Dclfil, the dcl file name.                                 
 ;              Dclnam, the dialog box name in the dcl file.               
 ;              Prom, the type for the number of things found prompt.      
 ;              Dianam, the dialog box title.                              
 ;   Returns a text string or nil.                                         
 ; 
 (DEFUN LBOX (styldt dclfil dclnam prom dianam / fpath dcl_id num numf filnam
                                                       fnam malist findx ret)
  (setq dcl_id (load_dialog dclfil))
  (new_dialog dclnam dcl_id)      ; must come before data for list box
  (set_tile "diabox" dianam)
 ; Ŀ
 ;   Make the Style list for the list box.                                 
 ; 
  (start_list "the_list")         ; read ltype data list into list box
  (setq num 0)
  (while (setq stylnm (nth num styldt))
         (add_list stylnm)
         (setq malist (cons stylnm malist))
         (setq num (1+ num)))
  (end_list)
  (setq malist (reverse malist))
  (set_tile "babtext" (strcat (itoa num) " " prom))
 ; Ŀ
 ;   Actions for given buttons/selections.  Must come after New_dialog     
 ;   call and before Start_dialog.                                         
 ; 
  (action_tile "select_ok" "(setq findx (selok $reason))")
  (action_tile "the_list" "(setq findx (lisok $reason))")
  (action_tile "fcancel" "(setq findx ())")
 ; Ŀ
 ;   Run it.                                                               
 ; 
  (setq ret (start_dialog))
  (unload_dialog dcl_id)
 ; Ŀ
 ;   Return a text string or nil.                                          
 ; 
 (if (and findx (/= findx ""))
     (nth (read findx) malist) nil))
 ; Ŀ
 ;   Lbox end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Lisok - if the list box generated a callback, see if it    
 ;   was a double click or an Enter, in which case return the value of     
 ;   the tile and close the dialog box.                                    
 ; 
 (DEFUN LISOK (reason / lisval)
  (setq lisval (get_tile "the_list"))
  (if (= reason 4)
      (done_dialog)
      (set_tile "babtext" ""))
 lisval)
 ; Ŀ
 ;   Lisok end.                                                            
 ; 

 ; Ŀ
 ;   Lsdata - get the maximum string length in a list of strings.          
 ;   Arguments: Lista, a list.                                             
 ;   Calls nothing, returns a number (0 if there were no strings.)         
 ; 
 (DEFUN LSDATA (lista / num maxa sub len)
  (setq num 0)
  (setq maxa 0)
  (while (setq sub (nth num lista))
         (setq num (1+ num))
         (if (and (= (type sub) 'STR)
                  (> (setq len (strlen sub)) maxa))
             (setq maxa len)))
 maxa)
 ; Ŀ
 ;   Lsdata end.                                                           
 ; 
         
 ; Ŀ
 ;   OneUp - Upper case the first character in a string.                   
 ;   Takes one argument, a string, returns a modified string.              
 ; 
 (DEFUN ONEUP (str)
  (strcat (strcase (substr str 1 1)) (substr str 2)))
 ; Ŀ
 ;   Oneup end.                                                            
 ; 

 ; Ŀ
 ;   Pout - write a list of strings to a file.                             
 ;   Arguments: Lista, a list.                                             
 ;              Filnam, a filename.                                        
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN POUT (lista filnam / fn sub)
  (setq fn (open filnam "w"))
  (while (setq sub (car lista))
         (setq lista (cdr lista))
         (write-line sub fn))
  (close fn)
 (princ))
 ; Ŀ
 ;   Pout end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Selok - if OK was pressed, see if a file name was          
 ;   selected, if so exit the dialog box and return the zero based index   
 ;   of that name.  Otherwise show an error.                               
 ; 
 (DEFUN SELOK (reason / lisval)
  (setq lisval (get_tile "the_list"))
  (if (and lisval (/= lisval ""))
      (done_dialog)
      (set_tile "babtext" "You must select a name."))
 lisval)
 ; Ŀ
 ;   Selok end.                                                            
 ; 

 ; Ŀ
 ;   Word1 - get the first word in a string.                               
 ;   Arguments: Str, a string.                                             
 ;   Calls nothing, returns a word or "".                                  
 ; 
 (DEFUN WORD1 (str / pos chra)
  (while (= (substr str 1 1) " ")
         (setq str (substr str 2)))
  (setq pos 1)
  (while (and (/= (setq chra (substr str pos 1)) "")
              (/= chra " "))
         (setq pos (1+ pos)))
  (cond ((= (substr str pos 1) " ")
         (setq str (substr str 1 (1- pos))))
        (t
         (setq str (substr str 1 pos)))))
 ; Ŀ
 ;   Word1 end.                                                            
 ; 

 ; Ŀ
 ;   Yout - suck a text file into a list.                                  
 ;   Arguments: filnam, a filename.                                        
 ;   Returns a list of strings.                                            
 ; 
 (DEFUN YOUT (filnam / fn linn malist)
  (if (setq fn (open filnam "r"))
      (progn
           (while (setq linn (read-line fn))
                  (setq malist (append malist (list linn))))
           (close fn)))
 malist)
 ; Ŀ
 ;   Yout end.                                                             
 ; 

 ; Ŀ
 ;   Lager: use a dialog box to select a string from a list.               
 ;   Arguments: Malist, a list of strings.                                 
 ;   Calls Drosop, Lbox, Lisok, Lsdata, Oneup, Pout, Selok, Word1,         
 ;   and Yout.  Returns a string or nil.                                   
 ; 
 (DEFUN LAGER (malist / ss htnum widnum)
 ; Ŀ
 ;   Get a list of all layers.                                             
 ; 
  (setq malist (blayo))
 ; Ŀ
 ;   Put the list in order.                                                
 ; 
  (if (null tize) (load "tize" ()))
  (setq malist (tize malist 1 nil))
 ; Ŀ
 ;   Get the list length and maximum string length, call Drosop to         
 ;   rewrite the .dcl file to reflect these numbers.                       
 ; 
  (setq htnum (length malist))
  (setq widnum (lsdata malist))
  (drosop "fibl.dcl" widnum htnum)
 ; Ŀ
 ;   And call the dialog box to get a layer name.                          
 ; 
  (lbox malist "fibl.dcl" "fibl" "Layers." "Layer to Find"))
 ; Ŀ
 ;   Lager end.  Sad.                                                      
 ; 
