 ; Ŀ
 ;   Lisht - list all text heights in a drawing.                           
 ;   Copyright 2011, 2014 by Rocket Software Ltd.                          
 ;                                                                         
 ; 

 ; Ŀ
 ;   Subroutine Horiz - put a list in order by the first number in each    
 ;   sublist.  Takes one argument, a list, which it returns in order       
 ;   from smallest to largest first element.                               
 ;   Rehashed 2007.10.02: the first number may not be a string.            
 ; 
 (DEFUN HORIZ (nexlst / low lowp nxtsub hrzlst newlst orderd)
  (while nexlst
        (setq low (lowest nexlst))                 ; lowest leading number
        (while (and nexlst (setq nxtsub (nth 0 nexlst)))
               (setq lowp (car nxtsub))
               (if (= 'str (type lowp)) (setq lowp (read lowp)))
               (if (equal low lowp)
                   (setq hrzlst (append hrzlst (list nxtsub)))
                   (setq newlst (append newlst (list nxtsub))))
               (setq nexlst (cdr nexlst)))          ; remove 1st ent from list
        (setq orderd (append orderd hrzlst))        ; add lev sublst to levels
        (setq hrzlst ())                            ; set to () for next loop
        (setq nexlst newlst)                        ; nexlst reconstituted
        (setq newlst ()))                           ; empty new list & reuse
  orderd)
 ; Ŀ
 ;   Horiz end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Lowest - find the smallest leading number in a sublist     
 ;   of the list Nexlst which is the sole argument.                        
 ;   Revised 2007.10.02 - leading numbers can be ints, reals, or number    
 ;   strings.                                                              
 ; 
 (DEFUN LOWEST (nexlst / num minlst neth carna)
  (setq num 0)
  (setq minlst (list min))
  (while (setq neth (nth num nexlst))
         (setq carna (car neth))
         (if (= 'str (type carna)) (setq carna (read carna)))
         (if neth (setq minlst (append minlst (list carna))))
         (setq num (1+ num)))
  (eval minlst))
 ; Ŀ
 ;   Lowest end.                                                           
 ; 

 ; Ŀ
 ;   Lisht.                                                                
 ; 
 (DEFUN C:LISHT (/ ss num enam entt hite sub suba totls headls endlis totlis
                                   total linn htmls2 htmlis prefa fn ff stra)
 ; Ŀ
 ;   Get an ss of all text in the drawing.                                 
 ; 
  (if (setq ss (ssget "x" (list (cons 0 "text"))))
      (progn
           (setq num 0)
           (setq total (sslength ss))
 ; Ŀ
 ;   Extract the data for each entity.                                     
 ; 
           (while (setq enam (ssname ss num))
                  (setq num (1+ num))
                  (setq entt (entget enam))
                  (setq hite (cdr (assoc 40 entt)))
 ; Ŀ
 ;   Save height counts.                                                   
 ; 
                  (if (setq sub (assoc hite totls))
                      (progn
                           (setq suba (cons hite (1+ (cdr sub))))
                           (setq totls (subst suba sub totls)))
                      (setq totls (cons (cons hite 1) totls)))
 ; Some drawings have text heights differing in the eighth decimal.
 ;(print (rtos (* hite 100000000) 2 8)))
 ; Ŀ
 ;   Re-order the list.                                                    
 ; 
           (setq totls (horiz totls))
           (setq hites (length totls))
 ; Ŀ
 ;   Make the html header and end lists.                                   
 ; 
  (setq headls (list "<body bgcolor=\"#336699\" text=\"#FFFFCC\">"
                     "<center><br>"))
  (setq endlis (list "</center>"
                     "<br><br>"))
 ; Ŀ
 ;   Make the Total table header list.                                     
 ; 
  (setq totlis (list "<font size=\"6\"><b>Text Height Totals</b></font>"
                     "<br><br><font size=\"4\">"
                     (strcat "Text Entities: " (itoa total))
                     "<br>"
                     (strcat "Different Heights: " (itoa hites))
                     "</font><br>"
                     "<table border=\"1\">"
                     "<colgroup align=\"center\" width=\"125\"></colgroup>"
                     "<tr><td>Text Height</td><td>Count </td></tr>"))
 ; Ŀ
 ;   Make the second table line list.                                      
 ; 
           (while (setq sub (car totls))
                  (setq totls (cdr totls))
                  (setq linn (strcat "<tr><td>" (rtos (car sub)) "</td>"
                                     "<td>" (itoa (cdr sub))
                                     "</td></tr>"))
                  (setq htmls2 (append htmls2 (list linn))))
 ; Ŀ
 ;   Add it the total table header and close lists.                        
 ; 
           (setq totlis (append totlis htmls2 clos))
 ; Ŀ
 ;   Make the entire html file list set: header, table, close.             
 ; 
           (setq htmlis (append headls totlis endlis))
 ; Ŀ
 ;   Open the file to write to.                                            
 ; 
           (setq prefa (getvar "dwgprefix"))
           (setq fn (strcat prefa "Texthts.htm"))
           (setq ff (open fn "w"))
 ; Ŀ
 ;   Write the html lines.                                                 
 ; 
           (setq num 0)
           (while (setq stra (nth num htmlis))
                  (setq num (1+ num))
                  (write-line stra ff))
 ; Ŀ
 ;   Close the file, end.                                                  
 ; 
           (if ff (close ff))
           (command "start" (strcat "firefox.exe \"" fn "\""))))
 (princ))