 ; Ŀ
 ;   Logox - Put a logo into a drawing.                                    
 ;   Copyright 1994, 2001 - 2014 by Rocket Software Ltd.                   
 ;   Only does the first type of title block it finds, but does all of     
 ;   those, because I can't imagine why you'd have two different tbs in    
 ;   a drawing.                                                            
 ; 

 ; Ŀ
 ;   Lath - find the first layer which isn't off, locked, or frozen.       
 ;   Takes no arguments.                                                   
 ;   Returns a layer name.                                                 
 ; 
 (DEFUN LATH (/ rew oklay lanam)
  (setq rew t)
  (while (and (null oklay) (setq lanam (tblnext "layer" rew)))
         (if (layp lanam)
             (setq rew ())
             (setq oklay lanam)))
 oklay)
 ; Ŀ
 ;   Lath end.                                                             
 ; 

 ; Ŀ
 ;   Layp - see if a layer is off, locked, or frozen.                      
 ;   Takes one argument, a layer name.                                     
 ;   Returns a list of conditions or nil - It's ok or doesn't exist.       
 ; 
 (DEFUN LAYP (lanam / llist sev col stalst)
  (if (setq llist (tblsearch "layer" lanam))
      (progn
           (setq sev (cdr (assoc 70 llist)))
           (setq col (cdr (assoc 62 llist)))
           (if (= (logand sev 1) 1) (setq stalst (list "frozen")))
           (if (= (logand sev 4) 4) (setq stalst (cons "locked" stalst)))
           (if (minusp col) (setq stalst (cons "off" stalst)))))
 stalst)
 ; Ŀ
 ;   Layp end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Loog - install a block in a position expressed as an       
 ;   offset from the insertion point of another block and at a given       
 ;   size.                                                                 
 ;   Arguments: Blnam, the name of the block to insert.                    
 ;              Pa, a point.                                               
 ;              Xoff, the X offset from the block insertion point.         
 ;              Yoff, the Y offset.                                        
 ;              Bsiz, the block size for a tb scale of 1.                  
 ;   Does nothing if the block name is nil.                                
 ;   Calls Nothing.                                                        
 ;   Returns nothing.                                                      
 ; 
 (DEFUN LOOG (blnam pa xoff yoff bsiz / osna bsiz ds dsxp dsyp pa)
  (setq osna (getvar "osmode"))
  (setvar "osmode" 0) 
  (setq bsiz (* bsiz (setq ds (cdr (assoc 41 entt)))))
  (setq dsxp (+ (car pa) (* xoff ds)))
  (setq dsyp (+ (cadr pa) (* yoff ds)))
  (setq pa (list dsxp dsyp))
  (cond ((findfile (strcat blnam ".dwg"))
         (command "insert" (strcat blnam "=") pa bsiz "" "0")
         (while (= 1 (getvar "cmdactive")) (command " ")))
        ((tblsearch "block" blnam)
         (prompt (strcat "\nExternal file for " blnam " not found.  "
                         "Using the definition in the drawing."))
         (command "insert" blnam pa bsiz "" "")
         (while (= 1 (getvar "cmdactive")) (command " ")))
        (t
         (prompt (strcat "\n** Block " blnam " isn't available. **"))))
  (setvar "osmode" osna) 
 (princ))
 ; Ŀ
 ;   Subroutine Loog end.                                                  
 ; 

 ; Ŀ
 ;   Schlog - insert a block near each block in an ss.                     
 ;   Arguments: ss, a selection set of block inserts.                      
 ;              Lognam, another block name (the one to insert).            
 ;              Ofx, an X offset.                                          
 ;              Ofy, a Y offset.                                           
 ;              Bsiz, the block scale as a fraction of main block scale.   
 ;   Calls Loog, Returns nothing.                                          
 ; 
 (DEFUN SCHLOG (ss lognam ofx ofy bsiz / num enam entt laya pa layy ctab sso)
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (setq entt (entget enam))
         (setq laya (cdr (assoc 8 entt)))
         (setq pa (cdr (assoc 10 entt)))
 ; Ŀ
 ;   Use layer 0 unless it's frozen or off in which case use the tb layer. 
 ;   If that's unusable then we're toast.                                  
 ; 
         (cond ((null (layp "0"))
                (setq layy "0"))
               ((null (layp laya))
                (setq layy laya))
               (t (setq layy (lath))))
         (setvar "clayer" layy)
 ; Ŀ
 ;   Make sure we're in the right tab.                                     
 ; 
         (setvar "ctab" (setq ctab (cdr (assoc 410 entt))))
         (if (/= (strcase ctab t) "model")
             (command ".pspace"))
 ; Ŀ
 ;   Erase any existing inserts.                                           
 ; 
         (if (setq sso (ssget "x" (list (cons 2 lognam))))
             (command ".erase" sso ""))
         (if (setq sso (ssget "x" (list (cons 2 "catch2"))))
             (command ".erase" sso ""))
 ; Ŀ
 ;   Call Loog to insert the block.                                        
 ; 
         (loog lognam pa ofx ofy bsiz))
 (princ))
 ; Ŀ
 ;   Schlog end.                                                           
 ; 

 ; Ŀ
 ;   Logox.                                                                
 ;   Note that there are lots of title blocks called Border, Tb, Title,    
 ;   and Title_Block.                                                      
 ; 
 (DEFUN C:LOGOX (/ lognam nama ss)
  (setvar "cmdecho" 0)
  (setq lognam "catch logo")
 ; Ŀ
 ;   Schlog arguments:                                                     
 ;     Ss of title blocks.                                                 
 ;     Logo block name.                                                    
 ;     X offset from insertion of the title block.                         
 ;     Y offset.                                                           
 ;     Block scale as a fraction of main block scale.                      
 ; 
 ; Ŀ
 ;   DRAWING_DESC. - appears to be Apache.                                 
 ; 
  (cond ((and (setq nama "DRAWING_DESC.")
              (tblsearch "block" nama)
              (setq ss (ssget "x" (list (cons 2 nama)))))
         (schlog ss lognam 693 47 0.15)
         (if (setq ss (ssget "x" (list (cons 2 "A$C04A71BEC"))))
             (command "erase" ss "")))
 ; Ŀ
 ;   HUSKY ANSI D Tblock Attributes.                                       
 ; 
  ((and (setq nama "HUSKY ANSI D Tblock Attributes")
              (tblsearch "block" nama)
              (setq ss (ssget "x" (list (cons 2 nama)))))
         (schlog ss lognam 782 334 0.2))
 ; Ŀ
 ;   HUSKY HOOL ANSI D Tblock Attributes.                                  
 ;   What is wrong with you people?                                        
 ; 
        ((and (setq nama "HUSKY HOOL ANSI D Tblock Attributes")
              (tblsearch "block" nama)
              (setq ss (ssget "x" (list (cons 2 nama)))))
         (schlog ss lognam 782 334 0.2))
 ; Ŀ
 ;   HUSKY-2X.                                                             
 ; 
        ((and (setq nama "HUSKY-2X")
              (tblsearch "block" nama)
              (setq ss (ssget "x" (list (cons 2 nama)))))
         (schlog ss lognam 725.6246 65.0203 0.2))
 ; Ŀ
 ;   Print a warning if no title block was found.                          
 ; 
        (t (write-line "\n*** No Title Block Was Found ***")))
 (princ))