 ; Ŀ
 ;   Ns: mark stuff that isn't on snap.                                    
 ;   Copyright 2011 by Rocket Software Ltd.                                
 ;                                                                         
 ; 

 ; Ŀ
 ;   Subroutine Grarc - arc maker.                                         
 ;   Based on Parc but uses points rather than angles for some arguments.  
 ;   Arguments: Pa, the arc centrepoint.                                   
 ;              Stapt, the start point.                                    
 ;              Endpt, the end point.                                      
 ;              Reps, the grdraw segments in each arc.                     
 ;              Colo, the colour.                                          
 ;   Arcs are drawn counterclockwise.                                      
 ;   Calls nothing, returns nothing, doesn't do anything very permanent... 
 ; 
 (DEFUN GRARC (pa stapt endpt reps colo / stangl endang radd incang bitang
                                                                    pa1 pa2)
  (setq radd (distance pa stapt))
  (setq stangl (angle pa stapt))
  (setq endang (angle pa endpt))
  (if (> stangl endang)
      (setq incang (- (* 2 pi) (- stangl endang)))
      (setq incang (abs (- stangl endang))))
  (setq bitang (/ incang reps))         ; individual dash chord angle
  (repeat reps
         (setq pa1 (polar pa stangl radd))
         (setq stangl (+ stangl bitang))
         (setq pa2 (polar pa stangl radd))
         (grdraw pa1 pa2 colo))
 (princ))
 ; Ŀ
 ;   Grarc end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Haht - grdraw a heart.                                     
 ;   Arguments: Pa, a heart base point.                                    
 ; 
 (DEFUN HAHT (pa / scla incr pa)
 ; Ŀ
 ;   Viewsize is screen height in drawing units.                           
 ; 
  (setq scla (/ (getvar "viewsize") 64))
  (setq incr (/ scla 12.0))
  (repeat 16
         (heart pa scla 0 16 -1 nil)
         (heart pa scla 0 16 -1 nil)
         (setq scla (+ scla incr)))
  (heart pa scla 0 16 220 nil)
  (setq scla (+ scla (* incr 3)))
  (heart pa scla 0 16 1 nil)
 (princ))
 ; Ŀ
 ;   Subroutin Haht end.                                                   
 ; 

 ; Ŀ
 ;   Heart - grdraw a heart.                                               
 ;   Arguments: Pa, the centre point.                                      
 ;              Scala, the height in draing units.                         
 ;              Rota, the rotation in degrees.                             
 ;              Segs, the number of segments in each arc.                  
 ;              Colo, the colour.                                          
 ;              Colo2, the crossing line colour, or nil for none.          
 ;   Calls Scral and Grarc.  Returns nothing.                              
 ; 
 (DEFUN HEART (pa scala rota segs colo colo2 / pr pt pl pb cur cul cll clr
                                                        pcur pcul pcll pclr)
 ; Ŀ
 ;   Make the distance and angle lists for a 0 rotation, 1 height heart.   
 ;   The four arc endpoints.  (Each is shared by two arcs.)                
 ; 
  (setq pr '(0.56388945 4.97641586))      ; right
  (setq pt '(0.34186491 90.0))            ; top
  (setq pl '(0.56388945 175.02358414))    ; left
  (setq pb '(0.50000000 270.0))           ; bottom
 ; Ŀ
 ;   The four arc centrepoints.                                            
 ; 
  (setq cur '(0.32975042 33.68896113))    ; upper right
  (setq cul '(0.32975042 146.31103887))   ; upper left
  (setq cll '(0.66091084 49.41670458))    ; lower left
  (setq clr '(0.66091084 130.58329542))   ; lower right
 ; Ŀ
 ;   Call Scral to find the actual points.                                 
 ; 
  (setq pr (scral pa pr scala rota))      ; right endpoint
  (setq pt (scral pa pt scala rota))      ; top endpoint
  (setq pl (scral pa pl scala rota))      ; lower endpoint
  (setq pb (scral pa pb scala rota))      ; bottom endpoint
  (setq pcur (scral pa cur scala rota))    ; upper right arc centre
  (setq pcul (scral pa cul scala rota))    ; upper left arc centre
  (setq pcll (scral pa cll scala rota))    ; lowerleft arc centre
  (setq pclr (scral pa clr scala rota))    ; lower right arc centre
 ; Ŀ
 ;   Draw the arcs.                                                        
 ; 
  (grarc pcur pr pt segs colo)
  (grarc pcul pt pl segs colo)
  (grarc pcll pl pb segs colo)
  (grarc pclr pb pr segs colo)
 ; Ŀ
 ;   Draw the lines.                                                       
 ; 
  (if colo2
      (progn
           (grdraw pt pb colo2)
           (grdraw pa pl colo2)
           (grdraw pa pr colo2)))
 (princ))
 ; Ŀ
 ;   Subroutine Heart end.                                                 
 ; 

 ; Ŀ
 ;   Mark - mark a point.                                                  
 ;   Arguments: Pa - the point to mark.                                    
 ;              Rad - the marker segment length.                           
 ;              Colo - the marker grdraw line colour.                      
 ; 
 (DEFUN MARK (pa rad colo /)
  (grdraw (polar pa (/ pi 4) rad) (polar pa (* 1.25 pi) rad) colo)
  (grdraw (polar pa (* pi 0.75) rad) (polar pa (* pi 1.75) rad) colo)
 (princ))
 ; Ŀ
 ;   Mark end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Nsnax - see if a point is on snap.                         
 ; 
 (DEFUN NSNAX (pa / snunit pax remx pay remy)
  (setq snunit (getvar "snapunit"))
  (if (zerop (car snunit))
      (progn
           (prompt "\nSnap Is Not Set.")
           (exit)))
  (setq pax (car pa))
  (setq remx (/ pax (car snunit)))
  (setq pay (cadr pa))
  (setq remy (/ pay (cadr snunit)))
 (and (= remx (fix remx))
      (= remy (fix remy))))
 ; Ŀ
 ;   Nsnax end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Parc - arc maker.                                          
 ;   Not currently called but goes with Grarc.                             
 ;   Arguments: Pa, the arc centrepoint.                                   
 ;              Stangl, the start angle in radians.                        
 ;              Endang, the end angle in radians.                          
 ;              Radd, the radius.                                          
 ;              Reps, the grdraw segments in each arc.                     
 ;              Colo, the colour.                                          
 ;   Arcs are drawn counterclockwise.                                      
 ;   Calls nothing, returns nothing, doesn't do anything very permanent... 
 ; 
 (DEFUN PARC (pa stangl endang radd reps colo / incang bitang stangl pa1 pa2)
  (if (> stangl endang)
      (setq incang (- (* 2 pi) (- stangl endang)))
      (setq incang (abs (- stangl endang))))
  (setq bitang (/ incang reps))         ; individual dash chord angle
  (repeat reps
         (setq pa1 (polar pa stangl radd))
         (setq stangl (+ stangl bitang))
         (setq pa2 (polar pa stangl radd))
         (grdraw pa1 pa2 colo))
 (princ))
 ; Ŀ
 ;   Parc end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Scral - scale, rotate and offset a location.               
 ;   Arguments: Pa, the base point.                                        
 ;              Liss, the base distance and angle list.                    
 ;              Scala, the scale.                                          
 ;              Rota, the rotation angle.                                  
 ;   Calls nothing, returns a point.                                       
 ; 
 (DEFUN SCRAL (pa liss scala rota / gnulen gnurot)
 ; Ŀ
 ;   The base point is moved, then offsets are scaled from there.          
 ;   That is, offset lengths are scaled and rotated from the base point.   
 ; 
  (setq gnulen (* scala (car liss)))
  (setq gnurot (+ rota (cadr liss)))
 (polar pa (* gnurot (/ pi 180)) gnulen))
 ; Ŀ
 ;   Scral end.                                                            
 ; 

 ; Ŀ
 ;   Spit - returns the insertion point of the text entity whose data was  
 ;   passed as its sole argument.  Note that this is not necessarily the   
 ;   same as the 10 association code.                                      
 ; 
 (DEFUN SPIT (entt / xjust yjust)
  (setq xjust (cdr (assoc 72 entt)))
  (setq yjust (cdr (assoc 73 entt)))
  (if (or (/= xjust 0) (/= yjust 0))
      (cdr (assoc 11 entt))
      (cdr (assoc 10 entt))))
 ; Ŀ
 ;   Spit end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Wasp - are we in the Model Space Tab, Paper Space, or      
 ;   a viewport in Paper Space?                                            
 ;   Brooks no Arguments.                                                  
 ;   Calls nothing.                                                        
 ;   Returns: 1 - Model space.                                             
 ;            2 - Paper space.                                             
 ;            3 - A Viewport in Paper Space.                               
 ; 
 (DEFUN WASP ()
  (cond ((= (getvar "tilemode") 1) 1)
        ((= (getvar "cvport") 1) 2)
        (t 3)))
 ; Ŀ
 ;   Wasp end.                                                             
 ; 

 ; Ŀ
 ;   Ns.                                                                   
 ; 
 (DEFUN C:NS (/ rad skulok ss num entt enam typ ps pa ons nums)
  (setq rad (/ (getvar "viewsize") 65))
 ; Ŀ
 ;   Try to load the grskull maker.                                        
 ; 
  (setq skulok (load "sk" ()))
 ; Ŀ
 ;   Try to load the entity measure and outliner.                          
 ; 
  (setq pusok (load "puss" ()))
 ; Ŀ
 ;   Get everything in the current space.                                  
 ;   Wasp returns: 1 - Model space.                                        
 ;                 2 - Paper space.                                        
 ;                 3 - A Viewport in Paper Space.                          
 ; 
  (setq spax (wasp))
  (setq tabb (getvar "ctab"))
  (cond ((member spax '(1 3))
         (setq ss (ssget "x" (list (cons 67 0)))))
        ((= spax 2)
         (setq ss (ssget "x" (list (cons 67 1) (cons 410 tabb))))))
  (setq num 0)
  (setq nums 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (setq entt (entget enam))
         (setq typ (cdr (assoc 0 entt)))
         (cond ((member typ '("VIEWPORT"))
                (setq pa nil))
               ((= typ "POLYLINE")
                (setq pa (cdr (assoc 10 (entget (entnext enam))))))
               ((member typ '("TEXT" "ATTDEF" "MTEXT"))
                 (setq pa (spit entt)))
               (t
                (setq pa (cdr (assoc 10 entt)))))
 ; Ŀ
 ;   See if the ten point is on snap.                                      
 ; 
         (if pa
            (if (nsnax pa)
                (haht pa)
                (progn
                     (setq nums (1+ nums))
 ; P1ssy doesn't understand leaders or mtext.  Teach it to handle or ignore
 ; these entities.  This section is currently deactivated.
 ;                   (if pusok (p1ssy enam))
                     (if skulok (sk pa 0.075))))))
 ; Ŀ
 ;   Sum up.                                                               
 ; 
  (write-line (strcat "On Snap: " (itoa (setq ons (- num nums))) "."
                      "  Not On Snap: " (itoa nums) "."
                      "  Snap Ratio: " (rtos (/ ons num 0.01) 2 2) "%."))
 (princ))
