 ; Ŀ
 ;   Nx - go to the next drawing.                                          
 ;   Xn - go to the previous drawing.                                      
 ;   Copyright 1995, 2001, 2005, 2009, 2011, 2012, 2018                    
 ;   by Rocket Software Ltd.                                               
 ;   Turn on SDI and actually get something done.  You will love this.     
 ; 

 ; Ŀ
 ;   Drexd - get the name of the file after the current one.               
 ;   Arguments: Dira, T = Forward, nil = Backwards.                        
 ;   Returns a filename with path.                                         
 ; 
 (DEFUN DREXD (dira / pref fils fnam dwgnam sub num)
  (setq pref (getvar "dwgprefix"))
  (setq fils (vl-directory-files pref "*.dwg" 1))
 ; Ŀ
 ;   Put the filename list in alphabetical order.                          
 ; 
  (setq fils (vl-sort fils '<))
  (if (not dira) (setq fils (reverse fils)))
 ; Ŀ
 ;   Find the drawing after the current one.                               
 ; 
  (setq dwgnam (getvar "dwgname"))
  (setq num 0)
  (while (and fils (setq sub (nth num fils)))
         (setq num (1+ num))
         (if (equal sub dwgnam)
             (progn
                  (setq fnam (nth num fils))
                  (setq num (1+ (length fils))))))
  (if (null fnam)
      (setq fnam (car fils)))
 (if fnam (strcat pref fnam)))
 ; Ŀ
 ;   Drexd end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Nexxt - open a drawing, optionally save the last one.      
 ;   Arguments: Fnam, the filename, ideally with a path.                   
 ;              Sava, "Y" or "N" - save changes if asked.                  
 ;   Returns T if need to restart/remake the file, nil otherwise.          
 ; 
 (DEFUN NEXXT (fnam sava / len)
  (setvar "cmdecho" 0)
  (setvar "sdi" 1)
  (if (/= (getvar "sdi") 1)
      (prompt "*Caution: SDI is not set - you are opening multiple drawings."))
 ; Ŀ
 ;   If the filename stored in fnam has the extension ".dwg", remove it.   
 ; 
  (if (and fnam
           (= (strcase (substr fnam (- (setq len (strlen fnam)) 3))) ".DWG"))
      (setq fnam (substr fnam 1 (- len 4))))
 ; Ŀ
 ;   If fnam contains a drawing filename and the drawing exists, open it.  
 ; 
  (cond ((and fnam (findfile (strcat fnam ".dwg")))
         (command "open")
 ; Ŀ
 ;   Really want to discard all changes to drawing?                        
 ;   So N = No, = save, Y= Yes = discard.                                  
 ;   A dbmod of 32 doesn't ask to save changes, anything else does.        
 ; 
         (if (> (rem (getvar "dbmod") 32) 0)
             (command sava)
             (princ))
         (command fnam))
 ; Ŀ
 ;   If the drawing doesn't exist, point this out.                         
 ; 
        (fnam
         (write-line (strcat "The File " fnam
                             " is unavailable.  Try again."))))
 (princ))
 ; Ŀ
 ;   Nexxt end.                                                            
 ; 

 ; Ŀ
 ;   Nx - go to the next file, save changes (if any).                      
 ; 
 (DEFUN C:NX (/ fnam)
  (if (setq fnam (drexd t))
      (nexxt fnam "n")
      (prompt "No Next File."))
 (princ))

 ; Ŀ
 ;   Xn - go to the previous file, save changes (if any).                  
 ; 
 (DEFUN C:XN (/ fnam)
  (if (setq fnam (drexd nil))
      (nexxt fnam "n")
      (prompt "No Previous File."))
 (princ))

(princ)