 ; Ŀ
 ;   Pann - suck data from a csv, make a distribution panel.               
 ;   Copyright 2002 by Rocket Software Ltd.                                
 ;                                                                         
 ; 

 ; Ŀ
 ;   Spad - replace empty strings in a list and sublists with dashes.      
 ;   Arguments: Thing, an atom or list.                                    
 ;   Returns the same thing, but with dashes.                              
 ;   Calls only itself.  (Recursive.)                                      
 ; 
 (DEFUN SPAD (thing / typ)
  (setq typ (type thing))
  (cond ((and (= typ 'STR)
              (member thing '("" " " "  ")))
         (setq thing "-"))
        ((= typ 'LIST)
         (setq thing (mapcar 'spad thing))))
 thing)
 ; Ŀ
 ;   Spad end.                                                             
 ; 

 ; Ŀ
 ;   Bcnofne - draw a series of vertical lines.                            
 ;   Arguments: Pa, a base point.                                          
 ;              Vlen, the common length.  (Lines are drawn down.)          
 ;              Oflst, a list of offsets right from Pa.                    
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN BCNOFNE (pa vlen oflist / pb)
  (mapcar '(lambda (offdis)
            (command ".line" (setq pb (polar pa 0 offdis))
                             (polar pb down vlen) ""))
           oflist)
 (princ))
 ; Ŀ
 ;   Bcnofne end.                                                          
 ; 

 ; Ŀ
 ;   Bl3nk - draw a three phase breaker line.                              
 ;   Argument: Pa, the start point - not the line start.                   
 ;   Calls nothing, returns nothing.                                       
 ;   Sets the global variable dpos3 which indicates the right, middle, or  
 ;   left dot position and which should be local to the calling function.  
 ; 
 (DEFUN BL3NK (pa / pb pdot pc parc1 pacen parc2)
  (setq pb (polar (polar pa down 3.75) 0 2.5))
  (cond ((or (null dpos3) (= dpos3 "A"))
         (setq dpos3 "B")
         (setq pdot (polar pb 0 15)))
        ((= dpos3 "B")
         (setq dpos3 "C")
         (setq pdot (polar pb 0 22.5)))
        ((= dpos3 "C")
         (setq dpos3 "A")
         (setq pdot (polar pb 0 30))))
  (command ".insert" "dot" pdot 1 1 0)
  (setq pc (polar pb 0 3.75))
  (command ".line" pb pc "")
  (setq parc1 (polar pc (/ pi 2) 1.25))
  (setq pacen (polar (polar pc down 3.75) 0 3.75))
  (setq parc2 (polar parc1 0 7.5))
  (command ".arc" parc2 "c" pacen parc1)
  (setq pb (polar pc 0 7.5))
  (setq pc (polar pb 0 22.5))
  (command ".line" pb pc "")
  (setq parc1 (polar pc (/ pi 2) 1.25))
  (setq pacen (polar (polar pc down 3.75) 0 3.75))
  (setq parc2 (polar parc1 0 7.5))
  (command ".arc" parc2 "c" pacen parc1)
  (setq pb (polar pc 0 7.5))
  (setq pc (polar pb 0 3.75))
  (command ".line" pb pc "")
 (princ))
 ; Ŀ
 ;   Bl3nk end.                                                            
 ; 

 ; Ŀ
 ;   Blink - draw a single phase breaker line.                             
 ;   Argument: Pa, the start point - not the line start.                   
 ;   Calls nothing, returns nothing.                                       
 ;   Sets the global variable dpos which indicates the right or left dot   
 ;   position and which should be local to the calling function.           
 ; 
 (DEFUN BLINK (pa / pb pdot pc parc1 pacen parc2)
  (setq pb (polar (polar pa down 3.75) 0 2.5))
  (if (= dpos "B")
      (progn
           (setq dpos "A")
           (setq pdot (polar pb 0 23.75)))
      (progn
           (setq dpos "B")
           (setq pdot (polar pb 0 16.25))))
  (command ".insert" "dot" pdot 1 1 0)
  (setq pc (polar pb 0 3.75))
  (command ".line" pb pc "")
  (setq parc1 (polar pc (/ pi 2) 1.25))
  (setq pacen (polar (polar pc down 3.75) 0 3.75))
  (setq parc2 (polar parc1 0 7.5))
  (command ".arc" parc2 "c" pacen parc1)
  (setq pb (polar pc 0 7.5))
  (setq pc (polar pb 0 17.5))
  (command ".line" pb pc "")
  (setq parc1 (polar pc (/ pi 2) 1.25))
  (setq pacen (polar (polar pc down 3.75) 0 3.75))
  (setq parc2 (polar parc1 0 7.5))
  (command ".arc" parc2 "c" pacen parc1)
  (setq pb (polar pc 0 7.5))
  (setq pc (polar pb 0 3.75))
  (command ".line" pb pc "")
 (princ))
 ; Ŀ
 ;   Blink end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Csplit - divide a text string at commas, make into a list  
 ;   of substrings.                                                        
 ; 
 (DEFUN CSPLIT (linn / len pos name1 strlst)
  (while (/= (strlen linn) 0)
         (while (and (= (substr linn 1 1) " ")
                     (/= (strlen linn) 0))
                (setq linn (substr linn 2)))
         (while (= (substr linn (setq len (strlen linn))) " ")
                (setq linn (substr linn 1 (1- len))))
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) ",")
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
         (setq strlst (append strlst (list name1))))
  (if (null strlst) (setq strlst (list "")))
  strlst)
 ; Ŀ
 ;   Csplit end.                                                           
 ; 

 ; Ŀ
 ;   Drog - install breaker line text and horizontal lines.                
 ;   Arguments: Pa, the initial upper left point.                          
 ;              Alist, the list of breaker lines.                          
 ;              Side, the Left or Right side indicator.                    
 ;   Calls numerous subroutines, Returns nothing.                          
 ; 
 (DEFUN DROG (pa alist side phase / num sub poles descr 2psiz 3psiz)
  (setq num 0)
  (while (setq sub (nth num alist))
         (setq num (1+ num))  
 ; Ŀ
 ;   See if this is a GFI breaker.                                         
 ; 
         (setq descr (nth 1 sub))
         (if (member "GFI" (splot (strcase (nth 3 sub))))
             (setq descr (strcat "GFI: " descr)))
 ; Ŀ
 ;   Decide how many poles we are dealing with.                            
 ; 
         (setq poles (substr (nth 3 sub) 1 2))
 ; Ŀ
 ;   Draw a 2 pole set.                                                    
 ; 
         (cond ((= poles "2P")
                (setq 2psiz (cadr (splot (nth 3 sub))))
                (if (= side "L")
                    (2pl pa descr (nth 2 sub) 2psiz (nth 0 sub)
                             (car (nth num alist)) phase)
                    (2pr pa descr (nth 2 sub) 2psiz (nth 0 sub)
                             (car (nth num alist)) phase))
                (setq num (1+ num))
                (setq pa (polar pa down 15)))
 ; Ŀ
 ;   Draw a 3 pole set.                                                    
 ; 
               ((= poles "3P")
                (setq 3psiz (cadr (splot (nth 3 sub))))
                (if (= side "L")
                    (3pl pa descr (nth 2 sub) 3psiz (nth 0 sub)
                       (car (nth num alist)) (car (nth (1+ num) alist)) phase)
                    (3pr pa descr (nth 2 sub) 3psiz (nth 0 sub)
                       (car (nth num alist)) (car (nth (1+ num) alist)) phase))
                (setq num (+ num 2))
                (setq pa (polar pa down 22.5)))
 ; Ŀ
 ;   Draw a single pole set.                                               
 ; 
               (t
                (if (= side "L")
                    (1pl pa descr (nth 2 sub)
                            (nth 3 sub) (nth 0 sub) phase)
                    (1pr pa descr (nth 2 sub)
                            (nth 3 sub) (nth 0 sub) phase))
                (setq pa (polar pa down 7.5)))))
 (princ))
 ; Ŀ
 ;   Drog end.                                                             
 ; 

 ; Ŀ
 ;   Hlinak - draw a series of horizontal lines.                           
 ;   Arguments: Pa, a base point.                                          
 ;              Hlen, the common length.  (Lines are drawn to the right.)  
 ;              Oflst, a list of offsets down from Pa.                     
 ;   Calls nothing, returns nothing.                                       
 ;                                                                         
 ; 
 (DEFUN HLINAK (pa hlen oflist / pb)
  (mapcar '(lambda (offdis)
            (command ".line" (setq pb (polar pa down offdis))
                             (polar pb 0 hlen) ""))
           oflist)
 (princ))
 ; Ŀ
 ;   Hlinak end.                                                           
 ; 

 ; Ŀ
 ;   Ma3br - draw three single phase breakers.                             
 ;   Argument: Pa, the ul corner of the surrounding box.                   
 ;   Calls nothing, returns a list of the three bus line start points.     
 ; 
 (DEFUN MA3BR (pa / pb pb1 pb2 pc pc1 pc2 parc1 pamid parc2 pamid1)
  (setq pb (polar (polar pa down 12.5) 0 17.5))
 ; Ŀ
 ;   Draw the contact circles.                                             
 ; 
  (command ".circle" pb 1.25)
  (command ".circle" (setq pb1 (polar pb 0 7.5)) 1.25)
  (command ".circle" (setq pb2 (polar pb 0 15)) 1.25)
  (command ".circle" (setq pc (polar pb down 10)) 1.25)
  (command ".circle" (setq pc1 (polar pb1 down 10)) 1.25)
  (command ".circle" (setq pc2 (polar pb2 down 10)) 1.25)
 ; Ŀ
 ;   The first arc.                                                        
 ; 
  (setq parc1 (polar pb 0 3))
  (setq pamid (polar (polar parc1 down 5) 0 2))
  (setq parc2 (polar parc1 down 10))
  (command ".arc" parc2 pamid parc1)
 ; Ŀ
 ;   The second arc.                                                       
 ; 
  (setq parc1 (polar pb1 0 3))
  (setq pamid1 (polar (polar parc1 down 5) 0 2))
  (setq parc2 (polar parc1 down 10))
  (command ".arc" parc2 pamid1 parc1)
 ; Ŀ
 ;   The third arc.                                                        
 ; 
  (setq parc1 (polar pb2 0 3))
  (setq pamid1 (polar (polar parc1 down 5) 0 2))
  (setq parc2 (polar parc1 down 10))
  (command ".arc" parc2 pamid1 parc1)
 ; Ŀ
 ;   The line between breaker arcs.                                        
 ; 
  (command ".line" pamid pamid1 "")
 ; Ŀ
 ;   Return the bus contact points.                                        
 ; 
 (list (polar pc down 1.25)
       (polar pc1 down 1.25)
       (polar pc2 down 1.25)))
 ; Ŀ
 ;   Ma3br end.                                                            
 ; 

 ; Ŀ
 ;   Maibr - draw a pair of single phase breakers.                         
 ;   Argument: Pa, the ul corner of the surrounding box.                   
 ;   Calls nothing, returns a list of the two bus line start points.       
 ; 
 (DEFUN MAIBR (pa / pb pb1 pc parc1 pamid parc2 pamid1)
  (setq pb (polar (polar pa down 12.5) 0 18.75))
  (command ".circle" pb 1.25)
  (command ".circle" (setq pb1 (polar pb 0 7.5)) 1.25)
  (command ".circle" (setq pc (polar pb down 10)) 1.25)
  (command ".circle" (polar pc 0 7.5) 1.25)
  (setq parc1 (polar pb 0 3))
  (setq pamid (polar (polar parc1 down 5) 0 2))
  (setq parc2 (polar parc1 down 10))
  (command ".arc" parc2 pamid parc1)
  (setq parc1 (polar parc1 0 7.5))
  (setq pamid1 (polar pamid 0 7.5))
  (setq parc2 (polar parc2 0 7.5))
  (command ".arc" parc2 pamid1 parc1)
  (command ".line" pamid pamid1 "")
 (list (polar pc down 1.25) (polar pb1 down 11.25)))
 ; Ŀ
 ;   Maibr end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Nphase - see if a cdf line indicates a single or three     
 ;   phase panel.                                                          
 ;   Argument: Lina, a comma delimited text string.                        
 ;   Calls csplit, returns a 1 or a 3.                                     
 ;   *** Not tested with a single phase panel yet.                         
 ; 
 (DEFUN NPHASE (lina / linn num)
  (setq linn (csplit lina))
  (setq linn (cddddr linn))
  (setq num 1)
  (while (/= (car linn) "CCT #")
         (setq num (1+ num))
         (setq linn (cdr linn)))
  (if (= num 4) 3 1))
 ; Ŀ
 ;   Subroutine Nphase end.                                                
 ; 

 ; Ŀ
 ;   1PL - make a left side single phase 1 pole breaker set.               
 ;   Arguments: Pa, the base point (upper left corner).                    
 ;              Descr, the circuit function.                               
 ;              Load, the load in Amps.                                    
 ;              Size, the breaker size (the 2P is assumed.)                
 ;              Cct, the circuit number.                                   
 ;              Phase, either 1 or 3.                                      
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN 1PL (pa descr load size cct phase / pb pc pd pe)
  (setyel)
  (setq pb (polar pa down 7.5))
  (setq pc (polar pa down 3.75))
  (if (= phase 1)
      (command "line" pb (polar pb 0 117.5) "")
      (command "line" pb (polar pb 0 115) ""))
  (malaya "text")
  (setby)
  (command ".text" "ml" (polar pc 0 2.5) 2.5 0 descr)
  (quish (entlast) 80)
  (if (= phase 3) (setq pc (polar pc pi 2.5)))
  (command ".text" "m" (polar pc 0 92.5) 2.5 0 load)
  (command ".text" "m" (polar pc 0 102.5) 2.5 0 size)
  (command ".text" "m" (polar pc 0 112.5) 2.5 0 cct)
 (princ))
 ; Ŀ
 ;   1PL end.                                                              
 ; 

 ; Ŀ
 ;   1PR - make a right side single phase 1 pole breaker set.              
 ;   Arguments: Pa, the base point (upper left corner).                    
 ;              Descr, the circuit function.                               
 ;              Load, the load in Amps.                                    
 ;              Size, the breaker size (the 2P is assumed.)                
 ;              Cct, the circuit number.                                   
 ;              Phase, either 1 or 3.                                      
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN 1PR (pa descr load size cct phase / pb pc pd pe)
  (setyel)
  (setq pb (polar pa down 7.5))
  (setq pc (polar pa down 3.75))
  (if (= phase 3)
      (command "line" (polar pb 0 2.5) (polar pb 0 117.5) "")
      (command "line" pb (polar pb 0 117.5) ""))
  (malaya "text")
  (setby)
  (if (= phase 3) (setq pc (polar pc 0 2.5)))
  (command ".text" "ml" (polar pc 0 32.5) 2.5 0 descr)
  (quish (entlast) 80)
  (command ".text" "m" (polar pc 0 25) 2.5 0 load)
  (command ".text" "m" (polar pc 0 15) 2.5 0 size)
  (command ".text" "m" (polar pc 0 5) 2.5 0 cct)
 (princ))
 ; Ŀ
 ;   1PR end.                                                              
 ; 

 ; Ŀ
 ;   2PL - make a left side single phase 2 pole breaker set.               
 ;   Arguments: Pa, the base point (upper left corner).                    
 ;              Descr, the circuit function.                               
 ;              Load, the load in Amps.                                    
 ;              Size, the breaker size (the 2P is assumed.)                
 ;              Cct1, the upper circuit number.                            
 ;              Cct2, the lower circuit number.                            
 ;              Phase, either 1 or 3.                                      
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN 2PL (pa descr load size cct1 cct2 phase / pb pc pd pe)
  (setyel)
  (setq pb (polar pa down 7.5))
  (setq pc (polar pa down 15))
  (if (= phase 3)
      (progn
           (command "line" pc (polar pc 0 115) "")
           (command "line" (polar pb 0 105) (polar pb 0 115) "")
           (command "line" (setq pf (polar (polar pa 0 125) down 1.25))
                           (polar pf down 7.5) "") ; breaker connector
           (setq pd (polar pa 0 105))
           (setq pe (polar pc 0 95)))
      (progn
           (command "line" pc (polar pc 0 117.5) "")
           (command "line" (polar pb 0 107.5) (polar pb 0 117.5) "")
           (command "line" (setq pf (polar (polar pa 0 127.5) down 1.25))
                           (polar pf down 7.5) "") ; breaker connector
           (setq pd (polar pa 0 107.5))
           (setq pe (polar pc 0 97.5))))
  (command "line" pd pe "")
  (malaya "text")
  (setby)
  (command ".text" "ml" (polar pb 0 2.5) 2.5 0 descr)
  (quish (entlast) 80)
  (if (= phase 3)
      (progn
           (setq pa (polar pa pi 2.5))
           (setq pb (polar pb pi 2.5))))
  (command ".text" "m" (polar pb 0 92.5) 2.5 0 load)
  (command ".text" "m" (polar (polar pa 0 100.75) down 3.25) 2.5 0 "2P")
  (command ".text" "m" (polar (polar pa 0 104) down 11.75) 2.5 0 size)
  (quish (entlast) 5)
  (command ".text" "m" (polar (polar pa 0 112.5) down 3.75) 2.5 0 cct1)
  (command ".text" "m" (polar (polar pa 0 112.5) down 11.25) 2.5 0 cct2)
 (princ))
 ; Ŀ
 ;   2PL end.                                                              
 ; 

 ; Ŀ
 ;   2PR - make a right side single phase 2 pole breaker set.              
 ;   Arguments: Pa, the base point (upper left corner).                    
 ;              Descr, the circuit function.                               
 ;              Load, the load in Amps.                                    
 ;              Size, the breaker size (the 2P is assumed.)                
 ;              Cct1, the upper circuit number.                            
 ;              Cct2, the lower circuit number.                            
 ;              Phase, either 1 or 3.                                      
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN 2PR (pa descr load size cct1 cct2 phase / pb pc pd pe pf)
  (setyel)
  (if (= phase 3) (setq pa (polar pa 0 2.5)))
  (setq pb (polar pa down 7.5))
  (setq pc (polar pa down 15))
  (if (= phase 3)
      (command "line" pc (polar pc 0 115) "")
      (command "line" pc (polar pc 0 117.5) ""))
 ; Ŀ
 ;   Add the breaker connector line.                                       
 ; 
  (command "line" (setq pf (polar (polar pa pi 10) down 1.25))
                           (polar pf down 7.5) "")
  (command "line" pb (polar pb 0 10) "")
  (setq pd (polar pa 0 20))
  (setq pe (polar pc 0 10))
  (command "line" pd pe "")
  (malaya "text")
  (setby)
  (command ".text" "ml" (polar pb 0 32.5) 2.5 0 descr)
  (quish (entlast) 80)
  (command ".text" "m" (polar pb 0 25) 2.5 0 load)
  (command ".text" "m" (polar (polar pa 0 13.25) down 3.25) 2.5 0 "2P")
  (command ".text" "m" (polar (polar pa 0 16.5) down 11.75) 2.5 0 size)
  (quish (entlast) 5)
  (command ".text" "m" (polar (polar pa 0 5) down 3.75) 2.5 0 cct1)
  (command ".text" "m" (polar (polar pa 0 5) down 11.25) 2.5 0 cct2)
 (princ))
 ; Ŀ
 ;   2PR end.                                                              
 ; 

 ; Ŀ
 ;   3PL - make a left side single phase 3 pole breaker set.               
 ;   Arguments: Pa, the base point (upper left corner).                    
 ;              Descr, the circuit function.                               
 ;              Load, the load in Amps.                                    
 ;              Size, the breaker size (the 3P is assumed.)                
 ;              Cct1, the upper circuit number.                            
 ;              Cct2, the lower circuit number.                            
 ;              Phase, either 1 or 3.                                      
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN 3PL (pa descr load size cct1 cct2 cct3 phase / pb pc pd pe pf)
  (setyel)
  (if (= phase 3) (setq pa (polar pa pi 2.5)))
  (setq pb (polar pa down 7.5))
  (setq pmid (polar pa down 11.25))
  (setq pc (polar pa down 15))
  (setq pc1 (polar pa down 22.5))
  (if (= phase 3)
      (command "line" (polar pc1 0 2.5) (polar pc1 0 117.5) "")
      (command "line" pc1 (polar pc1 0 117.5) ""))
 ; Ŀ
 ;   Add the breaker connector line.                                       
 ; 
  (command "line" (setq pf (polar (polar pa 0 127.5) down 1.25))
                           (polar pf down 15) "")
  (command "line" (polar pb 0 107.5) (polar pb 0 117.5) "")
  (command "line" (polar pc 0 107.5) (polar pc 0 117.5) "")
  (setq pd (polar pa 0 107.5))
  (setq pe (polar pc1 0 97.5))
  (command "line" pd pe "")
  (malaya "text")
  (setby)
  (command ".text" "ml" (polar pmid 0 5) 2.5 0 descr)
  (quish (entlast) 80)
  (command ".text" "m" (polar pmid 0 92.5) 2.5 0 load)
  (command ".text" "m" (polar (polar pa 0 100.75) down 3.25) 2.5 0 "3P")
  (command ".text" "m" (polar (polar pa 0 103.5) down 19.25) 2.5 0 size)
  (quish (entlast) 5.5)
  (command ".text" "m" (polar (polar pa 0 112.5) down 3.75) 2.5 0 cct1)
  (command ".text" "m" (polar (polar pa 0 112.5) down 11.25) 2.5 0 cct2)
  (command ".text" "m" (polar (polar pa 0 112.5) down 18.75) 2.5 0 cct3)
 (princ))
 ; Ŀ
 ;   3PL end.                                                              
 ; 

 ; Ŀ
 ;   3PR - make a right side single phase 3 pole breaker set.              
 ;   Arguments: Pa, the base point (upper left corner).                    
 ;              Descr, the circuit function.                               
 ;              Load, the load in Amps.                                    
 ;              Size, the breaker size (the 3P is assumed.)                
 ;              Cct1, the upper circuit number.                            
 ;              Cct2, the lower circuit number.                            
 ;              Phase, either 1 or 3.                                      
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN 3PR (pa descr load size cct1 cct2 cct3 phase / pb pc pd pe pf)
  (setyel)
  (if (= phase 3) (setq pa (polar pa 0 2.5)))
  (setq pb (polar pa down 7.5))
  (setq pmid (polar pa down 11.25))
  (setq pc (polar pa down 15))
  (setq pc1 (polar pa down 22.5))
  (if (= phase 3)
      (command "line" pc1 (polar pc1 0 115) "")
      (command "line" pc1 (polar pc1 0 117.5) ""))
 ; Ŀ
 ;   Add the breaker connector line.                                       
 ; 
  (command "line" (setq pf (polar (polar pa pi 10) down 1.25))
                           (polar pf down 15) "")
  (command "line" pb (polar pb 0 10) "")
  (command "line" pc (polar pc 0 10) "")
  (setq pd (polar pa 0 20))
  (setq pe (polar pc1 0 10))
  (command "line" pd pe "")
  (malaya "text")
  (setby)
  (command ".text" "ml" (polar pmid 0 32.5) 2.5 0 descr)
  (quish (entlast) 80)
  (command ".text" "m" (polar pmid 0 25) 2.5 0 load)
  (command ".text" "m" (polar (polar pa 0 13.25) down 3.25) 2.5 0 "3P")
  (command ".text" "m" (polar (polar pa 0 16) down 19.25) 2.5 0 size)
  (quish (entlast) 5.5)
  (command ".text" "m" (polar (polar pa 0 5) down 3.75) 2.5 0 cct1)
  (command ".text" "m" (polar (polar pa 0 5) down 11.25) 2.5 0 cct2)
  (command ".text" "m" (polar (polar pa 0 5) down 18.75) 2.5 0 cct3)
 (princ))
 ; Ŀ
 ;   3PR end.                                                              
 ; 

 ; Ŀ
 ;   Panic - draw everything except the individual circuit lines and text. 
 ;   Arguments: Pa, the panel upper left point.                            
 ;              Phase, one or three phase.                                 
 ;              Rows, the number of rows: each = L and R circuits.         
 ;              Title, the panel header title.                             
 ;              Loca, the location string.                                 
 ;   Calls various things, but Returns zilch.                              
 ; 
 (DEFUN PANIC (pa phase rows title loca / rows ovrlht pur plr pa1 brkrht pb1
                                  pb2 pn pn2 ent1 ent2 pabr buslen plist pb3)
  (if (/= rows (fix rows))
      (progn
           (setq rows (fix (1+ rows)))
           (prompt "\nUneven number of circuits fixed."))
      (setq rows (fix rows)))
  (setq ovrlht (+ (* rows 7.5) 47.5 25))
 ; Ŀ
 ;   First draw the more-or-less fixed stuff.                              
 ;   Draw the panel outline.                                               
 ; 
  (setcya)
  (command "pline" pa (setq pur (polar pa 0 280))
                      (setq plr (polar pur down ovrlht))
                      (polar plr pi 280) "c")
 ; Ŀ
 ;   Header and base horizontal lines.                                     
 ; 
  (hlinak pa 280 (list 10 45 47.5 (+ 47.5 (* rows 7.5)) (+ 50 (* rows 7.5))))
 ; Ŀ
 ;   Vertical column lines.                                                
 ; 
  (setyel)
  (setq pa1 (polar pa down 10))
  (setq brkrht (+ 37.5 (* rows 7.5)))
  (if (= phase 1)
      (bcnofne pa1 brkrht (list 87.5 97.5 107.5 117.5 162.5 172.5 182.5 192.5))
      (bcnofne pa1 brkrht (list 85 95 105 115 165 175 185 195)))
 ; Ŀ
 ;   Vertical base section dividers.                                       
 ; 
  (setcya)
  (if (= phase 1)
      (setq pb1 (polar (polar pa down (+ brkrht 12.5)) 0 97.5))
      (setq pb1 (polar (polar pa down (+ brkrht 12.5)) 0 95)))
  (setq pb2 (polar pb1 down 22.5))
  (command ".line" pb1 pb2 "")
  (if (= phase 1)
      (command ".line" (polar pb1 0 85) (polar pb2 0 85) "")
      (command ".line" (polar pb1 0 90) (polar pb2 0 90) ""))
 ; Ŀ
 ;   Solid Neutral symbology.                                              
 ; 
  (setby)
  (setq pn (polar (polar pb1 down 8.5) 0 42.5))
  (setq pn2 (polar pn down 4))
  (command ".line" pn pn2 "")
  (command ".insert" "dot" pn2 1 1 0)
  (command ".line" (polar pn2 pi 10) (polar pn2 0 10) "")
  (setby)
  (malaya "text")
  (command ".text" "M" (polar pn2 down 5) 3 0 "SOLID NEUTRAL")
 ; Ŀ
 ;   Various header text.                                                  
 ; 
  (setblu)
  (setq pn (polar (polar pa down 5) 0 140))
  (command ".text" "m" pn 4 0 title)
  (setby)
  (if (= phase 1)
      (command ".text" "mr" (polar pn 0 135) 3 0 "120/208V, 1%%C, 3 WIRE")
      (command ".text" "mr" (polar pn 0 135) 3 0 "120/208V, 3%%C, 4 WIRE"))
 ; Ŀ
 ;   Connected load header.                                                
 ; 
  (if (= phase 1)
      (setq pn (polar (polar pa down 27.5) 0 90.5))
      (setq pn (polar (polar pa down 27.5) 0 88)))
  (command ".text" "m" pn 2.5 90 "CONNECTED")
  (setq ent1 (entlast))
  (command ".text" "m" (polar pn 0 4) 2.5 90 "LOAD")
  (setq ent2 (entlast))
  (if (= phase 1)
      (command ".copy" ent1 ent2 "" "95,0" "")
      (command ".copy" ent1 ent2 "" "100,0" ""))
 ; Ŀ
 ;   Breaker Size header.                                                  
 ; 
  (setq pn (polar pn 0 10))
  (command ".text" "m" pn 2.5 90 "BREAKER SIZE")
  (setq ent1 (entlast))
  (command ".text" "m" (polar pn 0 4) 2.5 90 "(AMPS)")
  (setq ent2 (entlast))
  (if (= phase 1)
      (command ".copy" ent1 ent2 "" "75,0" "")
      (command ".copy" ent1 ent2 "" "80,0" ""))
 ; Ŀ
 ;   Curcuit No. header.                                                   
 ; 
  (setq pn (polar pn 0 12))
  (command ".text" "m" pn 2.5 90 "CIRCUIT No.")
  (setq ent1 (entlast))
  (if (= phase 1)
      (command ".copy" ent1 "" "55,0" "")
      (command ".copy" ent1 "" "60,0" ""))
 ; Ŀ
 ;   Location prototype text for base area.                                
 ; 
  (setq pn (polar (polar pa down (+ 53.75 (* rows 7.5))) 0 2.5))
  (command ".text" "ml" pn 2.5 0 loca)
 ; Ŀ
 ;   Breakers, lines, and connecting dots.                                 
 ; 
  (malaya "misc")
  (if (= phase 1)
      (setq pabr (polar (polar pa down 47.5) 0 117.5))
      (setq pabr (polar (polar pa down 47.5) 0 115)))
  (repeat rows
          (if (= phase 1) (blink pabr) (bl3nk pabr))
          (setq pabr (polar pabr down 7.5)))
 ; Ŀ
 ;   Main breakers and bus lines.                                          
 ; 
  (setq buslen (+ 13.75 (- (* rows 7.5) 2.5)))
  (if (= phase 1)
      (progn
           (setq pabr (polar (polar pa down 10) 0 117.5))
           (setq plist (maibr pabr)))
      (progn
           (setq pabr (polar (polar pa down 10) 0 115))
           (setq plist (ma3br pabr))))
  (setq pb1 (car plist))
  (setq pb2 (cadr plist))
  (setq pb3 (caddr plist))
  (setcya)
  (command ".line" pb1 (polar pb1 down buslen) "")
  (command ".line" pb2 (polar pb2 down buslen) "")
  (if pb3 (command ".line" pb3 (polar pb3 down buslen) ""))
 (princ))
 ; Ŀ
 ;   Panic end.  (Finally.)                                                
 ; 

 ; Ŀ
 ;   Pann: read a .csv file into a pair of lists.                          
 ;   Takes no arguments.                                                   
 ;   Calls Nphase.                                                         
 ;                                                                         
 ;   Returns a list: the left and right side breaker data lists, the       
 ;   number of breaker rows, one or three phase, and the title and         
 ;   location strings.                                                     
 ; 
 (DEFUN PANN (/ filnam fn title loca phase linn llist malist rows sub sub1
                                             sub2 gnulst num evenlst oddlst)
 ; Ŀ
 ;   Get a data file name.                                                 
 ; 
  (if (/= (type filnam) 'STR)
      (setq filnam ""))
  (setq filnam (getfiled "Data File" filnam "" 12))
  (if (and filnam (setq fn (open filnam "r")))
      (progn
 ; Ŀ
 ;   Extract the title, location, and number of phases from the first      
 ;   three lines.  This pretty well locks us into this file format.        
 ; 
           (setq title (car (csplit (read-line fn))))
           (setq loca (car (csplit (read-line fn))))
           (setq phase (nphase (read-line fn)))
 ; Ŀ
 ;   Add any lines from the rest of the file to the list of good data.     
 ; 
           (while (setq linn (read-line fn))
                  (setq llist (csplit linn))
                  (setq llist (spad llist))      ; replace "" with "-"
 ; Ŀ
 ;   A good circuit data line: 3 phase, Nth 3 and 7 are integers           
 ;   (cct numbers), or single phase and nth 3 and 6 are integers.          
 ; 
                  (if (or (and (= phase 3)
                               (nth 3 llist) (nth 7 llist)
                               (= (type (read (nth 3 llist))) 'INT)
                               (= (type (read (nth 7 llist))) 'INT))
                          (and (= phase 1)
                               (nth 3 llist) (nth 6 llist)
                               (= (type (read (nth 3 llist))) 'INT)
                               (= (type (read (nth 6 llist))) 'INT)))
                      (setq malist (cons llist malist))))
           (close fn)))
 ; Ŀ
 ;   The list length is the number of rows.                                
 ; 
  (setq rows (length malist))
 ; Ŀ
 ;   Mash the data lines into the appropriate format: individual breaker   
 ;   lists, number first.                                                  
 ; 
  (setq malist (reverse malist))
  (while (setq sub (car malist))
         (setq malist (cdr malist))
         (setq sub1 (list (nth 3 sub) (nth 0 sub) (nth 1 sub) (nth 2 sub)))
         (if (= phase 3)
            (setq sub2 (list (nth 7 sub) (nth 10 sub) (nth 9 sub) (nth 8 sub)))
            (setq sub2 (list (nth 6 sub) (nth 9 sub) (nth 8 sub) (nth 7 sub))))
         (setq gnulst (append gnulst (list sub1) (list sub2))))
 ; Ŀ
 ;   Split the list into right and left sublists.                          
 ; 
  (while (setq sub (car gnulst))
         (setq gnulst (cdr gnulst))
         (setq num (/ (read (car sub)) 2.0))
         (if (= num (fix num))
             (setq evenlst (append evenlst (list sub)))
             (setq oddlst (append oddlst (list sub)))))
  (list oddlst evenlst phase rows title loca))
 ; Ŀ
 ;   Subroutine Pann end.                                                  
 ; 

 ; Ŀ
 ;   Quish - If text is over a certain real width, make it narrower.       
 ;   Takes two arguments: a text ename and desired width.                  
 ; 
 (DEFUN QUISH (enam width / txtscl entt tblist realwd widscl prev41 scalfc)
  (setq txtscl 1.0)   ; set the desired text width scale
 ; Ŀ
 ;   Get the text entity data.                                             
 ; 
  (setq entt (entget enam))
 ; Ŀ
 ;   Find the actual string width.                                         
 ; 
  (setq tblist (textbox entt))
  (setq realwd (- (caadr tblist) (caar tblist)))
 ; Ŀ
 ;   Find the attribute width scale factor.                                
 ; 
  (setq widscl (cdr (setq prev41 (assoc 41 entt))))
 ; Ŀ
 ;   Compare the actual and desired widths.                                
 ; 
  (if (and (> realwd width) (not (equal realwd width 0.1)))
 ; Ŀ
 ;   If the actual width is greater than the allowed width in the          
 ;   sublist, then adjust the width scale factor to make it fit.           
 ;   Wait: must also check to see if the width scale is greater than the   
 ;   ideal - if the attribute is too wide and the width scale is too       
 ;   large, then shrinking the attribute to fit may result in it just      
 ;   filling the space but still being too wide.                           
 ; 
      (progn
 ; Ŀ
 ;   Compare the actual width scale to the ideal width scale.              
 ; 
           (if (<= widscl txtscl)
 ; Ŀ
 ;   If the actual is less than or equal to the ideal, then make it fit.   
 ; 
               (progn
                    (setq scalfc (/ width realwd))
                    (setq widscl (* widscl scalfc))
                    (entmod (subst (cons 41 widscl) prev41 entt)))
 ; Ŀ
 ;   If the actual width scale is greater than the ideal, see if the       
 ;   attribute will be too wide if if set to the ideal.                    
 ;   If so then squash to fit, if not then set to the ideal.               
 ; 
               (progn
                    (if (> (* realwd (/ txtscl widscl)) width)
 ; Ŀ
 ;   Squash to fit.                                                        
 ; 
                        (progn
                             (setq scalfc (/ width realwd))
                             (setq widscl (* widscl scalfc))
                             (entmod (subst (cons 41 widscl) prev41 entt)))
 ; Ŀ
 ;   Set to the ideal width scale factor.                                  
 ; 
                        (entmod (subst (cons 41 txtscl) prev41 entt))))))
 ; Ŀ
 ;   Else the actual width is narrower than or equal to the available      
 ;   space.                                                                
 ; 
      (progn
 ; Ŀ
 ;   See if the attribute is narrower than it should be - if setting the   
 ;   width scale factor to the desired value would leave the attribute     
 ;   wider than the allowable space, then increase it to fill the space.   
 ; 
           (if (> (* realwd (/ txtscl widscl)) width)
               (progn
                    (setq scalfc (/ width realwd))
                    (setq widscl (* widscl scalfc))
                    (entmod (subst (cons 41 widscl) prev41 entt)))
 ; Ŀ
 ;   Otherwise set it to the ideal width scale value.                      
 ; 
               (entmod (subst (cons 41 txtscl) prev41 entt)))))
 (princ))
 ; Ŀ
 ;   Quish end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Splot - divide a text string at spaces.                    
 ;   Arguments: Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ; 
 (DEFUN SPLOT (linn / len pos name1 strlst)
  (while (/= (strlen linn) 0)
         (while (and (= (substr linn 1 1) " ")
                     (/= (strlen linn) 0))
                (setq linn (substr linn 2)))
         (while (= (substr linn (setq len (strlen linn))) " ")
                (setq linn (substr linn 1 (1- len))))
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) " ")
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (while (= (substr name1 (setq len (strlen name1))) " ")
                (setq name1 (substr name1 1 (1- len))))
         (setq linn (substr linn (1+ pos)))
         (setq strlst (append strlst (list name1))))
  (if (null strlst) (setq strlst (list "")))
  strlst)
 ; Ŀ
 ;   Splot end.                                                            
 ; 

 ; Ŀ
 ;   The Brain: Pann.                                                      
 ; 
 (DEFUN C:PANN (/ down evenlst oddlst phase rows pa)
  (setvar "cmdecho" 0)
  (setvar "osmode" 0)
  (command "undo" "be")
 ; Ŀ
 ;   Make a few utility routines, set the direction variable.              
 ; 
  (setq down (* pi 1.5))
  (defun setblu () (setvar "cecolor" "blue"))
  (defun setyel () (setvar "cecolor" "yellow"))
  (defun setcya () (setvar "cecolor" "cyan"))
  (defun setby () (setvar "cecolor" "bylayer"))
 ; Ŀ
 ;   Get layers ready.                                                     
 ; 
  (if (null c:malaya) (load "malaya"))
  (malaya "text")
  (malaya "misc")
 ; Ŀ
 ;   Call Pann to get circuit and other data from the .csv.                
 ; 
  (setq evenlst (pann))
  (setq oddlst (car evenlst))
  (setq phase (caddr evenlst))
  (setq rows (cadddr evenlst))
  (setq title (nth 4 evenlst))
  (setq loca (nth 5 evenlst))
  (setq evenlst (cadr evenlst))
 ; Ŀ
 ;   Get an insertion point.                                               
 ; 
  (setq pa (getpoint "Upper Left Corner of Panel: "))
 ; Ŀ
 ;   Call Panic to draw the outline and text headers.                      
 ; 
  (panic pa phase rows title loca)
 ; Ŀ
 ;   Call Drog twice to install the left and right circuit lines and text. 
 ; 
  (drog (polar pa down 47.5) oddlst "L" phase)
  (drog (polar (polar pa down 47.5) 0 162.5) evenlst "R" phase)
  (command "undo" "end")
 (princ))