 ; Ŀ
 ;   Pasta - set up a drawing.                                             
 ;   Copyright 1993-2012 by Rocket Software Ltd.                           
 ;   Rocket: Software that works, for people who'd rather not.             
 ; 

 ; Ŀ
 ;   Stan.  Update dimscale for a named style.                             
 ;   Arguments: Stnam, a dimension style name.                             
 ;              Scal, a dimscale.                                          
 ;   Calls nothing, returns nothing.                                       
 ;   Changing dimvars in a style: make the dimstyle current, using the     
 ;   -dimstyle command (as opposed to the Dimstyle command which brings    
 ;   up the Ddim dialog box, or the Dimstyle system variable.)             
 ;   Then update the required dimvars (which temporarily become overrides) 
 ;   and then save the style, which makes the overrides part of it.        
 ; 
 (DEFUN STAN (stnam scal /)
 ; Ŀ
 ;   If the style exists, make it current.  If not then make it.           
 ;   The latter action will apparently copy the current style to the new   
 ;   name, so the new style will need to be set up properly.               
 ; 
  (if (tblsearch "dimstyle" stnam)
      (command "-dimstyle" "restore" stnam)
      (command "-dimstyle" "save" stnam))
 ; Ŀ
 ;   Change dimscale.  This becomes an override to the current style.      
 ; 
  (setvar "dimscale" scal)
 ; Ŀ
 ;   Save the style: since it is current and there are overrides on it     
 ;   they are written back to it and nothing else is changed.              
 ;   Technically, saving a dimstyle makes a copy of the current style      
 ;   with any overrides and saves that to the name given to the save       
 ;   command.                                                              
 ;   So saving any style other than the current one is a dubious idea:     
 ;   if we weren't writing the current style then a copy of the current    
 ;   style with any overrides applied would overwrite Stnam.               
 ;   Making a style current (which saving it does) discards overrides      
 ;   to the previous current style.  This implies that only the current    
 ;   style can have overrides.                                             
 ;   Note that the if statement isn't really necessary since we made       
 ;   stnam earlier if it didn't exist.                                     
 ;   Also note that if Stnam was already the current dimstyle and has      
 ;   overrides then they will become part of the style.                    
 ; 
  (if (tblsearch "dimstyle" stnam)
      (command "dimstyle" "save" stnam "y")
      (command "dimstyle" "save" stnam))
 (princ))
 ; Ŀ
 ;   Stan end.                                                             
 ; 

 ; Ŀ
 ;   Pasta.                                                                
 ; 
 (DEFUN C:PASTA (/ ok dims insp dimsp schema gr maxx minn)
  (setvar "cmdecho" 0)
  (if (getvar "dogmode") (setvar "dogmode" 1))  ; use canis domesticus
  (setvar "snapmode" 1)                         ; turn snap on no matter what
  (setq ok t)
 ; Ŀ
 ;   Check to see if Dimscale is 1 or 0.                                   
 ; 
  (setq dims (getvar "dimscale"))
  (if (and (/= dims 1) (/= dims 0))
      (progn
           (setq ok ())
           (write-line (strcat "This drawing is already scaled to "
                               (rtos dims)))
           (initget 0 "Yes No")
           (setq insp (getkword "\nContinue <Y>: "))
           (if (or (null insp) (= insp "Yes"))
               (setq ok t)
               (write-line "Probably a wise decision."))))
 ; Ŀ
 ;   Get a scale factor.                                                   
 ; 
  (if ok
     (progn
          (if (zerop dims)
              (setq dims 1))
          (setq dimsp (getreal (strcat "New Scale Factor <"
                                       (itoa (fix dims)) ">: ")))
          (if dimsp (setq dims dimsp))
 ; Ŀ
 ;   Make the Standard dimstyle current, set dimscale therein.             
 ;   Stan sets dimscale in the named dimstyle, not as an override          
 ; 
          (stan "Standard" dims)
 ; Ŀ
 ;   Set various variables.                                                
 ;   Ask if we're in a schematic.                                          
 ; 
          (if (= dims 1)
              (progn
                   (initget 0 "Yes No Schematic")
                   (setq insp (getkword
                                   "\nIs this a schematic drawing? <N>: "))
                   (if (member insp '("Yes" "Schematic"))
                       (setq schema t))))
          (if schema
              (progn
                   (command "snap" (rtos (* 2.5 dims)))
                   (setvar "filletrad" (* dims 2.5)))
              (progn
                   (command "snap" (rtos dims))
                   (setvar "filletrad" (* dims 3))))
          (setvar "ltscale" (* dims 10.0))
          (setvar "celtscale" 1)
          (setvar "textsize" (* dims 2.5))
          (setq gr (* dims 10))
          (setvar "gridunit" (list gr gr))
 ; Ŀ
 ;   Reset the limits.                                                     
 ;   Knock the last number off the value in Extmin and Extmax, since       
 ;   Limmin and Limmax are 2D points.                                      
 ; 
          (setq maxx (getvar "extmax"))
          (setq minn (getvar "extmin"))
          (setvar "limmax" (list (car maxx) (cadr maxx)))
          (setvar "limmin" (list (car minn) (cadr minn)))
 ; Ŀ
 ;   Make the standard text style current and correct.                     
 ; 
          (command ".style" "standard" "simplex.shx" "0" "1.0" "0" "n" "n" "n")
          (setvar "textstyle" "standard")))
 (princ))