 ; Ŀ
 ;   Pel - replace an ellipse entity with a polyline ellipse.              
 ;   Copyright 2016 by Rocket Software Ltd.                                
 ;   Meerkats - actually watching for the approach of civilization.        
 ; 

 ; Ŀ
 ;   Subroutine Cstar - draw an individual grstar (centred).               
 ;   Takes four arguments: centre point, side length, rotation (radians),  
 ;   and colour.  Returns nothing, but draws a star.                       
 ; 
 (DEFUN CSTAR (pa sidlen rota colo / anginc angg hafang pb)
  (setq pa (polar pa (+ rota (/ pi 2)) (* sidlen 1.37638192)))
  (setq anginc (* 1.6 pi))
  (setq angg (+ rota anginc))
  (setq hafang (* 0.8 pi))
  (repeat 5
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg anginc))
         (setq pa pb)
         (setq pb (polar pa angg sidlen))
         (grdraw pa pb colo)
         (setq angg (- angg hafang))
         (setq pa pb))
 (princ))
 ; Ŀ
 ;   Subroutine Cstar end.                                                 
 ; 

 ; Ŀ
 ;   Subroutine Glemp - make an ss into a polyline.                        
 ;   Assumes that each element in the ss is a polyline.                    
 ;   So don't reuse this unless that's the case.                           
 ;   Argument: ss, an ss of polylines.                                     
 ;   Returns the new entity name.                                          
 ; 
 (DEFUN GLEMP (ss / num enam typ entt closd)
 ; Ŀ
 ;   Remove all unusable entities from the ss.                             
 ; 
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq typ (cdr (assoc 0 (setq entt (entget enam)))))
         (setq closd (cdr (assoc 70 entt)))
         (if (or (= (logand closd 1) 1)    ; closed polyline?
                 (not (member typ '("LINE" "ARC" "POLYLINE" "LWPOLYLINE"))))
             (ssdel enam ss)
             (setq num (1+ num))))
 ; Ŀ
 ;   If there are any entities left in the ss.                             
 ; 
  (if (< 0 (sslength ss))
      (progn
           (setq enam (ssname ss 0))
           (ssdel enam ss)))
  (if (< 0 (sslength ss))
      (progn
           (setq typ (cdr (assoc 0 (setq entt (entget enam)))))
           (if (member typ '("POLYLINE" "LWPOLYLINE"))
               (command "pedit" enam "j" ss "" "")
               (command "pedit" enam "y" "j" ss "" ""))))
 ; Ŀ
 ;   Return the last entity name, unless enam still exists, in which case  
 ;   return it.                                                            
 ; 
 (if (entget enam) enam (entlast)))
 ; Ŀ
 ;   Subroutine Glemp end.                                                 
 ; 

 ; Ŀ
 ;   Subroutine Grcirc - grdraw circle maker.                              
 ; 
 (DEFUN GRCIRC (pa radd colo reps / angg incr pa1 pa2)
  (setq angg 0)
  (setq incr (/ pi (/ reps 2)))
  (setq pa1 (polar pa angg radd))
  (repeat reps
          (setq angg (+ angg incr))
          (setq pa2 (polar pa angg radd))
          (grdraw pa1 pa2 colo)
          (setq pa1 pa2))
 (princ))
 ; Ŀ
 ;   Grcirc end.                                                           
 ; 

 ; Ŀ
 ;   Pelp - find the start and endpoints of an ellipse.                    
 ;   Arguments: Cen, the centre of the ellipse.                            
 ;              Stapar, the start parameter.                               
 ;              Endpar, the end parameter.                                 
 ;              Angg, the major axis angle.                                
 ;              Angg2, the minor axis angle.                               
 ;              Axlen1, the major axis half length.                        
 ;              Axlen2, the minor axis half length.                        
 ;              Drah, draw grgraphics if T.                                
 ;              Prom, print details if T.                                  
 ;   Returns a list: (endpoint1 endpoint2).                                
 ; 
 (DEFUN PELP (cen stapar endpar angg angg2 axlen1 axlen2 drah prom / linlpa
                                      linlpb linapa linbpa end1 end2 anga angb)
 ; Ŀ
 ;   Draw a line L at the start parameter angle to hit the inner circle    
 ;   (diameter = minor axis) and the outer circle (diameter = major axis). 
 ;   The start parameter angle is relative to the ellipse angle.           
 ; 
  (setq stapar (+ stapar (- angg2 (/ pi 2))))
  (setq linlpa (polar cen stapar axlen1))  ; outer circle
  (setq linlpb (polar cen stapar axlen2))  ; inner circle
  (if drah
      (progn
           (grdraw cen linlpb 2)
           (grdraw linlpb linlpa 1)))
 ; Ŀ
 ;   Draw line A = from the intersection (on the line) of the minor        
 ;   circle and line L (linlpa), and at the ellipse angle.                 
 ;   The other end will be Linapa.                                         
 ; 
  (setq linapa (polar linlpa angg2 10))
  (if drah (grdraw linapa linlpa 1))
 ; Ŀ
 ;   Draw line B = from the intersection of the major circle and line L    
 ;   and line L (linlpb), at the ellipse angle + pi/2 radians.             
 ; 
  (setq linbpa (polar linlpb angg 10))
  (if drah (grdraw linbpa linlpb 2))
 ; Ŀ
 ;   The intersection of lines A and B, even if not on either line         
 ;   (i.e. extended intersection) is the endpoint of the ellipse.          
 ; 
  (setq end1 (inters linlpa linapa linlpb linbpa nil))
  (if drah (grdraw cen end1 3))
 ; Ŀ
 ;   Do the other end.                                                     
 ; 
  (setq endpar (+ endpar (- angg2 (/ pi 2))))
  (setq linlpa (polar cen endpar axlen1))  ; outer circle
  (setq linlpb (polar cen endpar axlen2))  ; inner circle
  (if drah (grdraw cen linlpb 2))
  (if drah (grdraw linlpb linlpa 1))
  (setq linapa (polar linlpa angg2 10))
  (if drah (grdraw linapa linlpa 1))
  (setq linbpa (polar linlpb angg 10))
  (if drah (grdraw linbpa linlpb 2))
  (setq end2 (inters linlpa linapa linlpb linbpa nil))
  (if drah (grdraw end1 end2 3))
 ; Ŀ
 ;   Print the angles.                                                     
 ;   Debug note: endpoint angles are relative to the long axis of the      
 ;   ellipse, not to the ucs 0 angle.                                      
 ; 
  (setq anga (angle cen end1))
  (setq anga (- anga angg))
  (setq angb (angle cen end2))
  (setq angb (- angb angg))
  (if prom
      (progn
           (prompt (strcat "\nAngle 1: " (rtos (* anga (/ 180 pi)) 2 2)
                           " (" (rtos anga 2 2) "R)"))
           (prompt (strcat "\nAngle 2: " (rtos (* angb (/ 180 pi)) 2 2)
                           " (" (rtos angb 2 2) "R)"))))
  (if drah (grdraw cen end1 1))
  (if drah (grdraw cen end2 3))
(grdraw cen end1 3)
(grdraw cen end2 1)
 ; Ŀ
 ;   Return the two end points.                                            
 ; 
 (list end1 end2))
 ; Ŀ
 ;   Pelp end.                                                             
 ; 

 ; Ŀ
 ;   Segang - find the included angle in an ellipse.                       
 ;   Arguments: Stangl, the start angle.                                   
 ;              Endang, the end angle.                                     
 ;              Isgap, if this is t then return the gap angle instead.     
 ;   If the ellipse isn't broken 41 is 0 and there is no 42.  Apparently.  
 ; 
 (DEFUN SEGANG (stangl endang isgap / angg)
  (cond ((or (null stangl) (null endang))  ; safety
         (setq angg 0))
        ((<= stangl endang)
         (setq angg (- endang stangl)))
        ((> stangl endang)
         (setq angg (- (* 2 pi) (abs (- endang stangl)))))
        (t
         (setq angg 0)
         (prompt "\nProgram Error - Unknown Gap Angle.")))
  (if isgap
      (setq angg (- (* 2 pi) angg)))
 angg)
 ; Ŀ
 ;   Segang end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Smack - make an ss of any entities after a given one.      
 ;   Takes the marker ename as an argument, returns an ss.                 
 ;   Note: this routine looks overly baroque.                              
 ; 
 (DEFUN SMACK (aaa / ss bbb)
  (if (setq bbb (entnext aaa))       ; put next ent in new ss
      (setq ss (ssadd bbb)))         ; put next ent in new ss
 ; Ŀ
 ;   If the entity is an insert and there are attributes:                  
 ; 
  (if (and bbb
           (= (cdr (assoc 0 (entget bbb))) "INSERT")
           (= (cdr (assoc 66 (entget bbb))) 1))
 ; Ŀ
 ;   Then find the seqend before assuming entnext will give the next ent.  
 ; 
      (progn
           (while (/= (cdr (assoc 0 (entget bbb))) "SEQEND")
                  (setq bbb (entnext bbb)))))
 ; Ŀ
 ;   Find all entities after the marker point, put them in an ss.          
 ; 
  (while (and bbb (entnext bbb))        ; while there are entities
         (setq bbb (entnext bbb))       ; find the next new entity
         (ssadd bbb ss)                 ; add it to the selection set
         (if (and (= (cdr (assoc 0 (entget bbb))) "INSERT")
                  (= (cdr (assoc 66 (entget bbb))) 1))
             (progn
                  (while (/= (cdr (assoc 0 (entget bbb))) "SEQEND")
                         (setq bbb (entnext bbb))))))
  ss)
 ; Ŀ
 ;   Smack end.                                                            
 ; 

 ; Ŀ
 ;   Pel.                                                                  
 ; 
 (DEFUN C:PEL (/ snapp peli drah prom *error* enampt entt enam typ cen elv
                 ratio stapar endpar axend1 axend2 angg angg2 axlen1 axlen2
                 axend3 end1 end2 gapang aaa bbb ell angga anggb lena lenb
                                                       end1a end2a ss ellp)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq snapp (getvar "snapmode"))
  (setvar "snapmode" 0)
  (setq peli (getvar "pellipse"))
  (setvar "pellipse" 1)
 ; Ŀ
 ;   Set the grdraw and prompt flags.                                      
 ; 
 ; (setq drah t)
 ; (setq prom t)
 ; Ŀ
 ;   Make the new local error handler.                                     
 ; 
  (defun *error* (shk)
   (setvar "snapmode" snapp)
   (setvar "pellipse" peli)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Get en ellipse entity.                                                
 ; 
  (cond ((null (setq enampt (entsel)))
         (prompt "\nNothing Selected."))
        ((null (setq entt (entget (setq enam (car enampt))))))
        ((not (equal (setq typ (cdr (assoc 0 entt))) "ELLIPSE"))
         (prompt (strcat "\nThat Was a " typ ", Not An Ellipse Entity.")))
        (t
         (setq cen (cdr (assoc 10 entt)))
         (setq elv (cdr (assoc 11 entt)))
         (setq ratio (cdr (assoc 40 entt)))
 ; Ŀ
 ;   Note that the start and end parameters are not the angles             
 ;   of the start and end points.                                          
 ; 
         (setq stapar (cdr (assoc 41 entt)))   ; start parameter
         (setq endpar (cdr (assoc 42 entt)))   ; end parameter
 ; Ŀ
 ;   Get the axis lengths, endpoints and angles.                           
 ; 
         (setq axend1 (list (+ (car cen) (car elv))
                            (+ (cadr cen) (cadr elv))))
         (setq axend2 (list (- (car cen) (car elv))
                            (- (cadr cen) (cadr elv))))
         (setq angg (angle axend2 axend1))
         (setq angg2 (+ angg (/ pi 2)))
         (setq axlen1 (distance cen axend1))
         (setq axlen2 (* ratio axlen1))
         (setq axend3 (polar cen angg2 axlen2))
 ; Ŀ
 ;   Mark the top end of the major axis.                                   
 ; 
         (cstar axend1 (/ axlen1 25) angg 5)
         (if prom (prompt (strcat "Ellipse major axis angle is: "
                                  (rtos (* angg (/ 180 pi)) 2 2))))
 ; Ŀ
 ;   Draw the axes and major and minor axis circles.                       
 ; 
         (if drah
             (progn
                  (grdraw axend1 axend2 8)
                  (grdraw cen axend3 8)
                  (grcirc cen axlen1 8 45)
                  (grcirc cen axlen2 8 45)))
 ; Ŀ
 ;   You can't use the Arc version of the ellipse command to draw a        
 ;   partial ellipse when pellipse = 1 - when making a pline ellipse.      
 ;   So we have to find the endpoints and trim the ellipse there.          
 ;   (See https://forums.autodesk.com/t5/net/dxf-ellipse/td-p/2439751)     
 ; 
         (setq end1 (pelp cen stapar endpar angg angg2 axlen1 axlen2
                                                                  drah prom))
         (setq end2 (cadr end1))
         (setq end1 (car end1))
         (cstar end1 (/ axlen1 25) angg 3)
         (cstar end2 (/ axlen1 25) angg 1)
         (grcirc end1 (/ axlen1 16) 8 21)
         (grcirc end2 (/ axlen1 16) 8 21)
 ; Ŀ
 ;   Get the angle occupied by the gap.                                    
 ; 
         (setq gapang (segang (angle cen end1) (angle cen end2) t))
 ; Ŀ
 ;   Find the last entity, save its ename.                                 
 ; 
         (setq aaa (entlast))
         (while (setq bbb (entnext aaa))
                (setq aaa bbb))
 ; Ŀ
 ;   Erase the ellipse, make a new one.                                    
 ; 
         (entdel enam)
         (command ".ellipse" "c" cen axend1 axlen2)
         (setq ell (entlast))
 ; Ŀ
 ;   Circles break counterclockwise.  Ellipses break so as to leave the    
 ;   smallest break, regardless of direction.  You can't break off most    
 ;   of an ellipse.                                                        
 ;   Polylines break between the points, in the shortest direction.        
 ;   Mirror the endpoints to the opposite side, dividing the ellipse into  
 ;   four chunks.  Then break off three, starting with one next to the     
 ;   section to be left.                                                   
 ; 
         (setq angga (angle cen end1))
         (setq anggb (angle cen end2))
         (setq lena (distance cen end1))
         (setq lenb (distance cen end2))
         (setq end1a (polar cen (+ angga pi) lena))
         (setq end2a (polar cen (+ anggb pi) lenb))
         (grdraw cen end1a 9)
         (grdraw cen end2a 9)
         (cstar end1a (/ axlen1 25) angg 3)
         (cstar end2a (/ axlen1 25) angg 1)
 ; Ŀ
 ;   Break the new ellipse.                                                
 ;   This is a two-state problem: cut beween the two points (the short     
 ;   length) or outside (the long one.)                                    
 ;   If gapang is > pi (i.e. half a circle) then break outside, using      
 ;   the three section method, else just cut the ends, doing inside the    
 ;   two points.                                                           
 ;   Also (if last entity /= new last entity) then join new to prev_last   
 ;   and get last entity before the next break command.                    
 ;   1. Do the single shot (inside) break.                                 
 ; 
         (cond ((equal gapang (* pi 2) 0.0001))
               ((< gapang pi)
                (command ".break" (list ell end1) "f" end2 end1)
                (setq ss (smack aaa))
                (if (setq ellp (glemp ss))
                    (setq ell ellp)))
 ; Ŀ
 ;   Or 2. Do the triple (outside) break.                                  
 ; 
               (t
                (command ".break" (list ell end1) "f" end1a end2a)
 ; Ŀ
 ;   Get an ss of the new entities, make them into one polyline.           
 ; 
                  (setq ss (smack aaa))
                  (if (setq ellp (glemp ss))
                      (setq ell ellp))
                  (command ".break" (nentselp end2) "f" end2 end1a)
                  (setq ss (smack aaa))
                  (if (setq ellp (glemp ss))
                      (setq ell ellp))
                  (command ".break" (nentselp end1) "f" end2a end1)
                  (setq ss (smack aaa))
                  (if (setq ellp (glemp ss))
                      (setq ell ellp))))))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (*error* nil)
 (princ))
