 ; Ŀ
 ;   Prawn - Put a stamp and permit into a drawing.                        
 ;   There is no C:Prawn - this is to be called from a stamp inserter.     
 ;   This routine requires the files Puss.lsp and Misps.lsp.               
 ;   Copyright 1994, 2001, 2003, 2004, 2006 - 2016 by Rocket Software Ltd. 
 ;                                                                         
 ;   The subroutine Fishth places the block on the layer whose name is     
 ;   the last argument, if this is nil then the block is placed on the     
 ;   layer containing the block which is used to find the position.        
 ;   Currently this is set to 0.                                           
 ;                                                                         
 ;   Revised: if the variable oneupp is T then only the first tb will      
 ;   be plotted - this will be the one in the current space if there is    
 ;   one, then the one in model space, then the first one found.           
 ;   This can be set in the calling file.  Which is the idea.              
 ;   The code is in Fishth, so don't make the variable local to it.        
 ;                                                                         
 ;   This routine should not be construed as condoning prawnography.       
 ; 

 ; Ŀ
 ;   Blosc - fit and position a block within a box.                        
 ;   Arguments: Cent, the box centre point.                                
 ;              Desx, the box width.                                       
 ;              Desy, the box height.                                      
 ;              Enam, the entity name of the block.                        
 ;   Calls Puss to find the size of the block.                             
 ;   Returns nothing.                                                      
 ; 
 (DEFUN BLOSC (cent desx desy enam / mxlst sub gnulst bnam widf hite blcen
                                                                     scalfc)
  (load "puss")
  (setq mxlst (puss enam))
 ; Ŀ
 ;   Empty blocks (or all atts, etc.) return (nil nil nil nil).            
 ;   So make sure there are no ()s in the list.                            
 ; 
  (while (setq sub (car mxlst))
         (setq mxlst (cdr mxlst))
         (if (null sub) (setq sub 0))
         (setq gnulst (append gnulst (list sub))))
  (setq mxlst gnulst)
  (if (null mxlst)
      (progn
           (setq mxlst '(0 0 0 0))
           (setq bnam (cdr (assoc 2 (entget enam))))
           (prompt (strcat "\nBad scale data in block " bnam))))
 ; Ŀ
 ;   Get a width and a height, make sure that neither one is zero.         
 ; 
  (setq widf (- (car mxlst) (cadr mxlst)))
  (if (= widf 0) (setq widf 1))
  (setq hite (- (caddr mxlst) (cadddr mxlst)))
  (if (= hite 0) (setq hite 1)) ; i.e. a text string containing only a dash
 ; Ŀ
 ;   Get the block centrepoint.                                            
 ; 
  (setq blcen (list (/ (+ (car mxlst) (cadr mxlst)) 2)
                    (/ (+ (caddr mxlst) (cadddr mxlst)) 2)))
 ; Ŀ
 ;   Centre the scaled block within the box.                               
 ; 
  (command ".move" enam "" blcen cent)
 ; Ŀ
 ;   Calculate whether scaling the x up to the width or the y up to the    
 ;   height would produce the smallest scale factor, use that.             
 ; 
  (setq scalfc (min (/ desx widf) (/ desy hite)))
  (command ".scale" enam "" cent scalfc)
 (princ))
 ; Ŀ
 ;   Blosc end.                                                            
 ; 

 ; Ŀ
 ;   Coux - get hopefully unique block data for a named block definition.  
 ;   Argument: Blnam, a block name.                                        
 ;   Calls nothing.                                                        
 ;   Returns a list: (number_of_entities                                   
 ;                    number_of_attdefs                                    
 ;                    distance_from_insertion_to_closest_att               
 ;                    att_tag  att_tag ...)                                
 ; 
 (DEFUN COUX (blnam / bokhed enam pa subs atts entt pat dizz dmin malist)
 ; Ŀ
 ;   Get the block head entity.                                            
 ; 
  (setq bokhed (tblsearch "block" blnam))
 ; Ŀ
 ;   Get the first subentity name.                                         
 ; 
  (setq enam (cdr (assoc -2 bokhed)))
  (setq pa (cdr (assoc 10 bokhed)))
  (setq subs 0)
  (setq atts 0)
  (while enam
         (setq subs (1+ subs))
         (setq entt (entget enam))
         (if (= (cdr (assoc 0 entt)) "ATTDEF")
             (progn
                  (setq atts (1+ atts))
 ; Ŀ
 ;   Get the entity insertion point.  Note that this is the real           
 ;   insertion, not necessarily the ten point.  Also note that the two     
 ;   seem to be the same, apparently an attdef reads - when in a block     
 ;   definition, which is what we are reading here, not the insertion -    
 ;   as being empty, so the ten and eleven points are the same.            
 ;   The spit call is left in in case this changes.                        
 ; 
                  (setq pat (spit entt))
                  (setq dizz (distance pa pat))
                  (if (or (null dmin) (< dizz dmin))
                      (setq dmin dizz))
                  (setq malist (cons (cdr (assoc 2 entt)) malist))))
         (setq enam (entnext enam)))
 ; Ŀ
 ;   Return the subentity block name list.                                 
 ; 
 (list subs atts dmin (reverse malist)))
 ; Ŀ
 ;   Coux end.                                                             
 ; 

 ; Ŀ
 ;   Deeb - see if a D-Bord-E is ... which one it is.                      
 ;   Currently not called - see Coux.                                      
 ;   Takes no arguments, calls nothing.                                    
 ;   Returns T if the insertion is in the lower left corner, else nil.     
 ; 
 (DEFUN DEEB (/ disc blnam ss enam pblock patt)
 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))
  (setq disc (misps))
  (setq blnam "d-bord-e")
  (if (and (tblsearch "block" blnam)
           (setq ss (ssget "x" (list (cons 2 blnam)))))
      (progn
           (setq enam (ssname ss 0))
           (setq pblock (cdr (assoc 10 (entget enam))))
           (setq patt (cdr (assoc 10 (entget (entnext enam)))))))
 (if (> (distance patt pblock) (* disc 60)) T))
 ; Ŀ
 ;   Deeb end.                                                             
 ; 

 ; Ŀ
 ;   Draed - erase and purge blocks named in a list.                       
 ;   Arguments: Blist, a list of block names.                              
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN DRAED (blist / blnam ss)
  (while blist
         (setq blnam (car blist))
         (setq blist (cdr blist))
         (if (and blnam (setq ss (ssget "X" (list (cons 2 blnam)))))
             (command ".erase" ss ""))
         (if (and blnam (tblsearch "block" blnam))
             (command "-purge" "block" blnam "n")))
 (princ))
 ; Ŀ
 ;   Draed end.                                                            
 ; 

 ; Ŀ
 ;   Fishth - find a block or blocks, insert another block nearby.         
 ;   Arguments: Blnam, a block name (the one to find).                     
 ;              Othnam, another block name (the one to insert).            
 ;              Ofx, an X offset.                                          
 ;              Ofy, a Y offset.                                           
 ;              Bsiz, a desired block size in drawing units.               
 ;              Laya, a layer name, if nil use the block layer.            
 ;   Calls sculpt.                                                         
 ;                                                                         
 ;   Looks for the global variable oneupp which tells it only to do        
 ;   one block: the one in the current space, the one in model space,      
 ;   or the first block it finds.                                          
 ;                                                                         
 ;   Note: Fishth erases any existing Othnam blocks, so if (Prawn)         
 ;   finds more than one tb, only the last one will have a permit and      
 ;   beaver stamp inserted when the program ends.                          
 ;   This could be made an option.                                         
 ;                                                                         
 ;   Returns T if anything was found, else nil.                            
 ; 
 (DEFUN FISHTH (blnam othnam ofx ofy bsiz laya / ss enam entt ctab layy sso
                                                                        num)
 ; Ŀ
 ;   If the desired layer is set and is frozen or off, unset it, which     
 ;   will tell the routine to use the tb layer.                            
 ;   If that's unusable then we're toast.                                  
 ; 
  (if (and laya (layp laya))
      (setq laya nil))
 ; Ŀ
 ;   Find the base block, see if the one to insert is available,           
 ; 
  (if (and blnam othnam (tblsearch "block" blnam)
           (setq ss (ssget "x" (list (cons 2 blnam)))))
      (progn
 ; Ŀ
 ;   oneupp is set and there's a tb in the current space, so do it.        
 ; 
           (cond ((and oneupp
                       (setq ss (ssget "x" (list (cons 2 blnam)
                                                 (cons 410 (getvar "ctab"))))))
                  (setq enam (ssname ss 0))
                  (setq entt (entget enam))
                  (setvar "ctab" (setq ctab (cdr (assoc 410 entt))))
                  (if (/= (strcase ctab t) "model")
                      (command ".pspace"))
                  (if laya
                      (setq layy laya)
                      (setq layy (cdr (assoc 8 entt))))
                  (if (setq sso (ssget "x" (list (cons 2 othnam))))
                      (command ".erase" sso ""))
                  (setvar "clayer" layy)
                  (sculpt enam othnam ofx ofy bsiz))
 ; Ŀ
 ;   oneupp is set and there's a tb in model space, so do it.              
 ; 
                 ((and oneupp
                       (setq ss (ssget "x" (list (cons 2 blnam)
                                                 (cons 410 "Model")))))
                  (setq enam (ssname ss 0))
                  (setq entt (entget enam))
                  (setvar "ctab" (setq ctab (cdr (assoc 410 entt))))
                  (if laya
                      (setq layy laya)
                      (setq layy (cdr (assoc 8 entt))))
                  (if (setq sso (ssget "x" (list (cons 2 othnam))))
                      (command ".erase" sso ""))
                  (setvar "clayer" layy)
                  (sculpt enam othnam ofx ofy bsiz))
 ; Ŀ
 ;   oneupp is set, so do the first tb we can find.                        
 ; 
                 ((and oneupp
                       (setq ss (ssget "x" (list (cons 2 blnam)))))
                  (setq enam (ssname ss 0))
                  (setq entt (entget enam))
                  (setvar "ctab" (setq ctab (cdr (assoc 410 entt))))
                  (if (/= (strcase ctab t) "model")
                      (command ".pspace"))
                  (if laya
                      (setq layy laya)
                      (setq layy (cdr (assoc 8 entt))))
                  (if (setq sso (ssget "x" (list (cons 2 othnam))))
                      (command ".erase" sso ""))
                  (setvar "clayer" layy)
                  (sculpt enam othnam ofx ofy bsiz))
 ; Ŀ
 ;   oneupp isn't set, so do all of them.                                  
 ; 
                 (t
                  (setq num 0)
                  (while (setq enam (ssname ss num))
                         (setq num (1+ num))
                         (setq entt (entget enam))
                         (setvar "ctab" (setq ctab (cdr (assoc 410 entt))))
                         (if (/= (strcase ctab t) "model")
                             (command ".pspace"))
                         (if laya
                             (setq layy laya)
                             (setq layy (cdr (assoc 8 entt))))
 ; Ŀ
 ;   See which space each title block occupies, go to that space.          
 ; 
                         (if (setq sso (ssget "x" (list (cons 2 othnam))))
                             (command ".erase" sso ""))
                         (setvar "clayer" layy)
                         (sculpt enam othnam ofx ofy bsiz))))))
 (if ctab t ()))
 ; Ŀ
 ;   Fishth end.                                                           
 ; 

 ; Ŀ
 ;   IDAK - kill and purge all blocks named in a list, detach xrefs.       
 ;   Takes no arguments, returns nothing.  Independent and ruthless.       
 ;   Calls Isxnam as a sop to public opinion.                              
 ;   Xrefs vs. Insertions:                                                 
 ;   If a block is inserted then can't xref it in - must detach.           
 ;   Similarly, if a block is xrefed then can't insert - must erase        
 ;   and purge.                                                            
 ; 
 (DEFUN IDAK (/ klista num blnam ss)
  (setq klista '("abstermit-ref"  "Alta_Colleaux"
                 "beaver"
                 "BC_PEng_mono"
                 "beck"      "bcbeck"    "skbeck"
                 "bozek"     "skbozek"
                 "budd"
                 "checkryn"
                 "dancy"
                 "delind"
                 "eng-rbh"
                 "ExergyPerm"
                 "fast"
                 "freemant"
                 "gemsask"
                 "gondek"
                 "hayden"
                 "henderson" "skhender"
                 "huitema"   "bchuitem"  "quhuitem"  "skhuitem"
                 "ibrahim"
                 "isbister"  "skisbstr"
                 "jajarmi"   "bcjajarmi" "skjajarmi"
                 "johnson"   "bcjohnsn"  "skjohnsn"
                 "jones"     "bcjones"   "skjones"
                 "jorgense"  "skjorgen"
                 "joy"       "bcjoy"     "skjoy"
                 "kurrant"
                 "majer"
                 "macdonel"
                 "mazursky-ab"
                 "mcdougal"
                 "mclenhan"
                 "newman"    "bcnewman"  "qunewman"  "sknewman"
                 "nguyen"
                 "permab"    "permalta"  "permabkp"  "00permit"
                 "permit"
                 "permsask"  "permsask1"
                 "pavan"     "skpavan"
                 "pickel"    "skpickel"
                 "pirouz_ab"
                 "rawlyk"
                 "redko"
                 "sawatzky"  "bcsawat"   "sksawatzky"
                 "slupsky"
                 "smerek"
                 "snowdon"   "bcsnowdn"
                 "ta"
                 "tardif"
                 "warners"
                 "zotzman"
                 "AB - Arnold"    "AB - Cramer"   "AB - Freeman"
                 "AB - Jajarmi"   "AB - Joy"      "AB - Kondics"
                 "AB - Newman"    "AB - Permit"   "AB - Quon"
                 "AB - Robillard" "AB - Velichko" "AB - Warners"
                 "AB - Watts"     "AB - Yee"      "BC - Cramer"
                 "BC - Freeman"   "BC - Kondics"
                 "CatchPermit"
                 "MB - Carlson"   "MB - Permit"
                 "APEGM STAMP (HOCS)-new stamp"
                 "APEGM STAMP (HOCS)"
                 "Apegm Stamp"
                 "EXCELSIOR_PERMIT_AB"
                 "Peng _Permit_Stamp_CNRL TB"
                 "Peng _Permit_Stamp_Encana TB"
                 "Peng _Permit_Stamp_Huskey TB"
                 "Peng _Permit_Stamp_Tridyne TB"
                 "SK - Cramer"       "SK - Freeman"   "SK - Kondics"
                 "SK - Newman"       "SK - Permit"    "SK - Velichko"
                 "AB-James Beck"     "AB-Karman Beal" "AB-Mike Hayden"
                 "AB-Permit to Practice"              "AB-Shane Newman"
                 "AB-Trevor Freeman"                  "BC-James Beck"
                 "BC-Mike Hayden"    "BC-Shane Newman"
                 "BC-Trevor Freeman" "MB-Mike Hayden"
                 "ON-James Beck"     "SK-James Beck"
                 "SK-Permission to Consult"
                 "SK-Shane Newman"   "SK-Trevor Freeman"
                 "td-permit"
                 "AB-Tyler Andreychuk"))
 ; Ŀ
 ;   Step through the block list and find a way to get rid of each one.    
 ; 
  (setq num 0)
  (while (setq blnam (nth num klista))
         (setq num (1+ num))
         (if (tblsearch "block" blnam)
             (progn
 ; Ŀ
 ;   If the block is an xref then detach it.                               
 ; 
                  (if (isxnam blnam)
                      (command "-xref" "d" blnam))
 ; Ŀ
 ;   If there are still any copies then either they were normal blocks     
 ;   or the xref was nested and can't be detached, so erase them.          
 ; 
                  (if (and (tblsearch "block" blnam)
                           (setq ss (ssget "X" (list (cons 2 blnam)))))
                      (command "erase" ss ""))
 ; Ŀ
 ;   If the block is still in the block tables and it isn't an xref then   
 ;   purge it.                                                             
 ; 
                  (if (and (tblsearch "block" blnam)
                           (not (isxnam blnam)))
                      (command "-purge" "block" blnam "n")))))
 (princ))
 ; Ŀ
 ;   Idak end.                                                             
 ; 

 ; Ŀ
 ;   Isxnam: see if a given block is an xref.                              
 ;   Argument: Blnam, the block name.                                      
 ;   Returns T: it was an xref, or nil: it wasn't, or no such block is     
 ;   is defined in the drawing.                                            
 ; 
 (DEFUN ISXNAM (blnam / dat xp isxrf)
  (if (setq dat (tblsearch "block" blnam))
      (progn
           (setq xp (cdr (assoc 70 dat)))
           (setq isxrf (logand xp 4))))
 (if (= isxrf 4) T ()))
 ; Ŀ
 ;   Isxnam end.                                                           
 ; 

 ; Ŀ
 ;   Lath - find the first layer which isn't off, locked, or frozen.       
 ;   Takes no arguments.                                                   
 ;   Returns a layer name.                                                 
 ; 
 (DEFUN LATH (/ rew oklay lanam)
  (setq rew t)
  (while (and (null oklay) (setq lanam (tblnext "layer" rew)))
         (if (layp lanam)
             (setq rew ())
             (setq oklay lanam)))
 oklay)
 ; Ŀ
 ;   Lath end.                                                             
 ; 

 ; Ŀ
 ;   Layp - see if a layer is off, locked, or frozen.                      
 ;   Takes one argument, a layer name.                                     
 ;   Returns a list of conditions or nil - It's ok or doesn't exist.       
 ; 
 (DEFUN LAYP (lanam / llist sev col stalst)
  (if (setq llist (tblsearch "layer" lanam))
      (progn
           (setq sev (cdr (assoc 70 llist)))
           (setq col (cdr (assoc 62 llist)))
           (if (= (logand sev 1) 1) (setq stalst (list "frozen")))
           (if (= (logand sev 4) 4) (setq stalst (cons "locked" stalst)))
           (if (minusp col) (setq stalst (cons "off" stalst)))))
 stalst)
 ; Ŀ
 ;   Layp end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Sculpt - install a block in a position expressed as an     
 ;   offset from the insertion point of another block and at a given       
 ;   size.                                                                 
 ;   Arguments: Enam, the main (typically tb) block ename.                 
 ;              Blnam, the name of the block to insert.                    
 ;              Xoff, the X offset from the block insertion point.         
 ;              Yoff, the Y offset.                                        
 ;              Bsiz, the block size for a tb scale of 1.                  
 ;   Does nothing if the block name is nil.                                
 ;   Returns nothing, calls your mama.  And Blosc.                         
 ; 
 (DEFUN SCULPT (enam blnam xoff yoff bsiz / osna entt pa bsiz ds dsxp dsyp)
  (setq osna (getvar "osmode"))
  (setvar "osmode" 0) 
  (setq entt (entget enam))
  (setq pa (cdr (assoc 10 entt)))
  (setq bsiz (* bsiz (setq ds (cdr (assoc 41 entt)))))
  (setq dsxp (+ (car pa) (* xoff ds)))
  (setq dsyp (+ (cadr pa) (* yoff ds)))
  (cond ((findfile (strcat blnam ".dwg"))
         (command "insert" (strcat blnam "=") pa "" "" "")
         (while (= 1 (getvar "cmdactive")) (command " "))
         (blosc (list dsxp dsyp) bsiz bsiz (entlast)))
        ((tblsearch "block" blnam)
         (prompt (strcat "\nExternal file for " blnam " not found.  "
                         "Using the definition in the drawing."))
         (command "insert" blnam pa "" "" "")
         (while (= 1 (getvar "cmdactive")) (command " "))
         (blosc (list dsxp dsyp) bsiz bsiz (entlast)))
        (t
         (prompt (strcat "\n** Block " blnam " isn't available. **"))))
  (setvar "osmode" osna) 
 (princ))
 ; Ŀ
 ;   Subroutine Sculpt end.                                                
 ; 

 ; Ŀ
 ;   Spit - returns the insertion point of the text entity whose data was  
 ;   passed as its sole argument.  Note that this is not necessarily the   
 ;   same as the 10 association code.                                      
 ; 
 (DEFUN SPIT (entt / xjust yjust)
  (setq xjust (cdr (assoc 72 entt)))
  (setq yjust (cdr (assoc 73 entt)))
  (if (or (/= xjust 0) (/= yjust 0))
      (cdr (assoc 11 entt))
      (cdr (assoc 10 entt))))
 ; Ŀ
 ;   Spit end.                                                             
 ; 

 ; Ŀ
 ;   Prawn.                                                                
 ;   Note that Since Fishth erases inserted blocks only the last one       
 ;   installed will be present when (Prawn) ends.                          
 ;   This means you may have to comment out parts of this program.         
 ;   ALso note that there are lots of title blocks called Border,          
 ;   Tb, Title, and Title_Block.                                           
 ;   (Coux) counts the number of attributes in a block and can be used     
 ;   to distinguish them, or to exclude some.  This has not been done      
 ;   for Border, but should be.                                            
 ;   Prawn.                                                                
 ; 
 (DEFUN PRAWN (beaver permit / ss blizt atnum lenah trisub num blsub blnam
                                                stanam ofx ofy bsiz folio)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Kill any existing stamps.  Idak is fast but relies on an internal     
 ;   list, which hangman doesn't.  Idak is more certain for blocks it      
 ;   knows, Hangman is more universal but may miss oddball blocks.         
 ;   Hang would be faster than Hangman and also purges block definitions.  
 ; 
  (idak)
;  (c:hangman)
 ; Ŀ
 ;   Make the list of lists of title block and insertion data.             
 ;   The sublist format is:                                                
 ;     Main block name.                                                    
 ;     Block to insert name.                                               
 ;     X offset from insertion of the title block.                         
 ;     Y offset.  (The offsets are for the centre of the block rather      
 ;                 than the insertion point.)                              
 ;     Desired inserted block size in drawing units.                       
 ;   There will typically be two sublists for each title block, but this   
 ;   doesn't have to be so.                                                
 ;   All sizes and distances will be adjusted automatically block scale.   
 ;   Special case blocks (different blocks with the same name) are added   
 ;   later.  Unique blocks should be added here.                           
 ; 
 ; Ŀ
 ;   There is someone in the world stupid enough to call their             
 ;   title block "A$C3B3322C8.                                             
 ;   Also to steal an Encana TB and not even change the attribute names.   
 ; 
  (if (and (tblsearch "block" "A$C3B3322C8")
           (setq ss (ssget "x" (list (cons 2 "A$C3B3322C8")))))
      (setq blizt (append blizt
                    (list (list "A$C3B3322C8" beaver 603.5 50.75 36)
                          (list "A$C3B3322C8" permit 540 50.75 62)))))
 ; Ŀ
 ;   There is someone in the world stupid enough to call their             
 ;   title block "1".                                                      
 ; 
  (if (and (tblsearch "block" "1")
           (setq ss (ssget "x" (list (cons 2 "1"))))
           (setq lenah (cadr (coux "1")))
           (equal lenah 38 1))
      (setq blizt (append blizt
                    (list (list "1" beaver 664 138 50)
                          (list "1" permit 664 90 78)))))
 ; Ŀ
 ;   Agrium.  So far there appear to be many title blocks.                 
 ; 
  (if (and (tblsearch "block" "OVERLAYD")
           (setq ss (ssget "x" (list (cons 2 "OVERLAYD")))))
      (setq blizt (append blizt
                    (list (list "OVERLAYD" beaver 20.45 1.25 1.25)
                          (list "OVERLAYD" permit 22.8242 1.25 3.2)))))
 ; Ŀ
 ;   The main Aker TB is called Border.  This is run off the separate      
 ;   attribute block which is called APS-TB-TEXT-D.                        
 ; 
  (if (and (tblsearch "block" "APS-TB-TEXT-D")
           (setq ss (ssget "x" (list (cons 2 "APS-TB-TEXT-D")))))
      (setq blizt (append blizt
                    (list (list "APS-TB-TEXT-D" beaver -748 40.5 60)
                          (list "APS-TB-TEXT-D" permit -643 39 99)))))
 ; Ŀ
 ;   We now use Aker Proto.  But there are others.                         
 ; 
  (if (and (tblsearch "block" "Aker Proto")
           (setq ss (ssget "x" (list (cons 2 "Aker Proto")))))
      (setq blizt (append blizt
                    (list (list "Aker Proto" beaver -748 40.5 60)
                          (list "Aker Proto" permit 319 48 78)))))
 ; Ŀ
 ;   But in some Aker drawings the TB is A$C595034CC.                      
 ; 
  (if (and (tblsearch "block" "A$C595034CC")
           (setq ss (ssget "x" (list (cons 2 "A$C595034CC")))))
      (setq blizt (append blizt
                    (list (list "A$C595034CC" beaver 492.82 54.32 54)
                          (list "A$C595034CC" permit 560.02 54.32 70)))))
 ; Ŀ
 ;   BG-D.  Apparently IMV.                                                
 ; 
  (if (and (tblsearch "block" "BG-D")
           (setq ss (ssget "x" (list (cons 2 "BG-D")))))
      (setq blizt (append blizt
                    (list (list "BG-D" beaver 263 48 40)
                          (list "BG-D" permit 318 48 62)))))
 ; Ŀ
 ;   Amec Tbd-Tb-e.                                                        
 ; 
  (if (and (tblsearch "block" "tbd-tb-e")
           (setq ss (ssget "x" (list (cons 2 "tbd-tb-e")))))
      (setq blizt (append blizt
                    (list (list "tbd-tb-e" beaver 776 55 57)
                          (list "tbd-tb-e" permit 287 24 70)))))
 ; Ŀ
 ;   Tnpi.                                                                 
 ; 
  (if (and (tblsearch "block" "AMW_TITLE")
           (setq ss (ssget "x" (list (cons 2 "AMW_TITLE")))))
      (setq blizt (append blizt
                    (list (list "AMW_TITLE" beaver 465.75 26 36)
                          (list "AMW_TITLE" permit 526.5 24 72)))))
 ; Ŀ
 ;   Allow for the two different D-Bord-E blocks.                          
 ;   No: Coux should let one allow for a variety.  Unfortunately this      
 ;   means doing an ssget to make the list, then another later ... can     
 ;   this be avoided?  Without too much complexity or difficult updating?  
 ; 
  (if (and (tblsearch "block" "d-bord-e")
           (setq ss (ssget "x" (list (cons 2 "d-bord-e")))))
      (progn
           (setq lenah (caddr (coux "d-bord-e")))
           (cond ((equal lenah 422 1)
                  (setq trisub (list (list "D-Bord-E" beaver 335 38 42.5)
                                     (list "D-Bord-E" permit 390 38 51))))
                 ((equal lenah 242.7 1)
                  (setq trisub (list (list "D-Bord-E" beaver 133 36.5 44)
                                     (list "D-Bord-E" permit 197 36.5 63.5))))
                 ((equal lenah 434.6 1)
                  (setq trisub (list (list "D-Bord-E" beaver 173 36.5 44)
                                     (list "D-Bord-E" permit 237 36.5 63.5))))
                 (t
                  (setq trisub (list (list "D-Bord-E" beaver 133 36 44)
                                     (list "D-Bord-E" permit 197 36.5 63.5)))))
           (setq blizt (append blizt trisub))))
 ; Ŀ
 ;   Border.  Which imbecile did this?                                     
 ; 
;  (if (and (tblsearch "block" "border")
;           (setq ss (ssget "x" (list (cons 2 "border")))))
;      (setq blizt (append blizt
;                    (list (list "border" beaver 706.95 49 45)
;                          (list "border" permit 644.05 49.3 64)))))
 ; Ŀ
 ;   CatchTB or Catchrefs.                                                 
 ;   Check for a reference block first, then a tb.                         
 ; 
  (cond ((and (tblsearch "block" "catchrefs")
              (setq ss (ssget "x" (list (cons 2 "catchrefs")))))
         (setq blizt (append blizt
                       (list (list "catchrefs" beaver 190 25 40)
                             (list "catchrefs" permit 235 25 90)))))
        ((and (tblsearch "block" "catchtb")
              (setq ss (ssget "x" (list (cons 2 "catchtb")))))
         (setq blizt (append blizt
                       (list (list "catchtb" beaver 376 25 40)
                             (list "catchtb" permit 455 25 90))))))
 ; Ŀ
 ;   BeckTB.                                                               
 ; 
  (if (and (tblsearch "block" "becktb")
           (setq ss (ssget "x" (list (cons 2 "becktb")))))
      (setq blizt (append blizt
                    (list (list "becktb" beaver 393 28 45)
                          (list "becktb" permit 458 29 69)))))
 ; Ŀ
 ;   Bord_A1.                                                              
 ;   Several companies use this name.  The blocks are different.           
 ; 
  (if (and (tblsearch "block" "bord_a1")
           (setq ss (ssget "x" (list (cons 2 "bord_a1")))))
      (progn
           (setq lenah (cadr (coux "bord_a1")))
           (if (zerop lenah)
               (setq blizt (append blizt
                             (list (list "bord_a1" beaver 611 38 36)
                                   (list "bord_a1" permit 665 38 59))))
               (setq blizt (append blizt
                             (list (list "bord_a1" beaver 647.5 45.1 42)
                                   (list "bord_a1" permit 590.5 45.25 59)))))))
 ; Ŀ
 ;   Connacher-tb.                                                         
 ; 
  (if (and (tblsearch "block" "connacher-tb")
           (setq ss (ssget "x" (list (cons 2 "connacher-tb")))))
      (setq blizt (append blizt
                    (list (list "connacher-tb" beaver 600 27 42)
                          (list "connacher-tb" permit 536.5 27 72)))))
 ; Ŀ
 ;   CNRL-D-TBLK.                                                          
 ; 
  (if (and (tblsearch "block" "CNRL-D-TBLK")
           (setq ss (ssget "x" (list (cons 2 "CNRL-D-TBLK")))))
      (setq blizt (append blizt
                    (list (list "CNRL-D-TBLK" beaver 327 48 50)
                          (list "CNRL-D-TBLK" permit 273 48 55)))))
 ; Ŀ
 ;   Devon-D.  But there are two with the same name.                       
 ; 
  (if (and (tblsearch "block" "DEVON-D")
           (setq ss (ssget "x" (list (cons 2 "DEVON-D")))))
      (progn
           (setq lenah (cadr (coux "devon-d")))
           (cond ((equal lenah 84 1)
                  (setq blizt (append blizt
                                      (list (list "DEVON-D" beaver 467 25 44)
                                            (list "DEVON-D" permit 555 25 78)))))
                 ((equal lenah 68 1)
                  (setq blizt (append blizt
                                      (list (list "DEVON-D" beaver 480 37.5 44)
                                            (list "DEVON-D" permit 568 37.5 78))))))))
 ; Ŀ
 ;   Dover.                                                                
 ; 
  (if (and (tblsearch "block" "DOVER-ANSI_D")
           (setq ss (ssget "x" (list (cons 2 "DOVER-ANSI_D")))))
      (setq blizt (append blizt
                    (list (list "DOVER-ANSI_D" beaver 348 50 50)
                          (list "DOVER-ANSI_D" permit 274 50 78)))))
 ; Ŀ
 ;   DRAWING_DESC. - apparently Apache.  One could use the frame block     
 ;   but it's called "tblock."                                             
 ; 
  (if (and (tblsearch "block" "DRAWING_DESC.")
           (setq ss (ssget "x" (list (cons 2 "DRAWING_DESC.")))))
      (setq blizt (append blizt
                    (list (list "DRAWING_DESC." beaver 472.5 45 32.5)
                          (list "DRAWING_DESC." permit 520 14 60)))))
 ; Ŀ
 ;   Encana - T2A-7, and many others.                                      
 ; 
  (if (and (tblsearch "block" "t2a-7")
           (setq ss (ssget "x" (list (cons 2 "t2a-7")))))
      (setq blizt (append blizt
                    (list (list "t2a-7" beaver 603.5 42.56 36)
                          (list "t2a-7" permit 543 42.75 62)))))
 ; Ŀ
 ;   Encana - t-a002a.                                                     
 ; 
  (if (and (tblsearch "block" "t-a002a")
           (setq ss (ssget "x" (list (cons 2 "t-a002a")))))
      (setq blizt (append blizt
                    (list (list "t-a002a" beaver 603.5 42.56 36)
                          (list "t-a002a" permit 543 42.75 62)))))
 ; Ŀ
 ;   Encana - T2A-8.  FFS.                                                 
 ; 
  (if (and (tblsearch "block" "t2a-8")
           (setq ss (ssget "x" (list (cons 2 "t2a-8")))))
      (setq blizt (append blizt
                    (list (list "t2a-8" beaver 603.5 42.56 36)
                          (list "t2a-8" permit 543 42.75 62)))))
 ; Ŀ
 ;   Enerplus: TitleA1a.                                                   
 ; 
  (if (and (tblsearch "block" "titlea1a")
           (setq ss (ssget "x" (list (cons 2 "titlea1a")))))
      (setq blizt (append blizt
                    (list (list "titlea1a" beaver 279 43 46)
                          (list "titlea1a" permit 216 43 64)))))
 ; Ŀ
 ;   Excelsior.                                                            
 ; 
  (if (and (tblsearch "block" "EXC-805-DWG-0001-A1")
           (setq ss (ssget "x" (list (cons 2 "EXC-805-DWG-0001-A1")))))
      (setq blizt (append blizt
                    (list (list "EXC-805-DWG-0001-A1" beaver 467.5 40 38)
                          (list "EXC-805-DWG-0001-A1" permit 399 40 60)))))
 ; Ŀ
 ;   Exergy.  Probably.  Or Williams?                                      
 ; 
  (if (and (tblsearch "block" "RWTitleBlock")
           (setq ss (ssget "x" (list (cons 2 "RWTitleBlock")))))
      (setq blizt (append blizt
                    (list (list "RWTitleBlock" beaver 643.6 44.6 41)
                          (list "RWTitleBlock" permit 584.5 45.25 56)))))
 ; Ŀ
 ;   Gemini - GC_A1_Titleblock.                                            
 ; 
  (if (and (tblsearch "block" "GC_A1_Titleblock")
           (setq ss (ssget "x" (list (cons 2 "GC_A1_Titleblock")))))
      (setq blizt (append blizt
                    (list (list "GC_A1_Titleblock" beaver 579.15 30.0 45)
                          (list "GC_A1_Titleblock" permit 645.15 30.375 55)))))
 ; Ŀ
 ;   Grizzly, as done by Lavalin - Title_Block.                            
 ; 
  (if (and (tblsearch "block" "title_block")
           (setq ss (ssget "x" (list (cons 2 "title_block")))))
      (setq blizt (append blizt
                    (list (list "title_block" beaver 46.8 27 30)
                          (list "title_block" permit 120.5 29 70)))))
 ; Ŀ
 ;   Grizzly, another.                                                     
 ; 
  (if (and (tblsearch "block" "d_grizzly")
           (setq ss (ssget "x" (list (cons 2 "d_grizzly")))))
      (setq blizt (append blizt
                    (list (list "d_grizzly" beaver 24 60.27 30)
                          (list "d_grizzly" permit 100 60 68)))))
 ; Ŀ
 ;   Grizzly - Grizzly-tb.  (Actually a modified Catch tb.)                
 ; 
  (if (and (tblsearch "block" "Grizzly-tb")
           (setq ss (ssget "x" (list (cons 2 "Grizzly-tb")))))
      (setq blizt (append blizt
                    (list (list "Grizzly-tb" beaver 250 24 42.5)
                          (list "Grizzly-tb" permit 190 23.637 56)))))
 ; Ŀ
 ;   Hocs/PennWest.                                                        
 ; 
  (if (and (tblsearch "block" "new blockNEW")  ; hlf
           (setq ss (ssget "x" (list (cons 2 "new blockNEW")))))
      (setq blizt (append blizt
                    (list (list "new blockNEW" beaver 508 50 54)
                          (list "new blockNEW" permit 607 50 78)))))
 ; Ŀ
 ;   Another Hocs/PennWest.  Staggeringly.                                 
 ; 
  (if (and (tblsearch "block" "PWT TB_D Size_Template rev2")
           (setq ss (ssget "x" (list (cons 2 "PWT TB_D Size_Template rev2")))))
      (setq blizt (append blizt
                    (list (list "PWT TB_D Size_Template rev2" beaver 236 22 20)
                          (list "PWT TB_D Size_Template rev2" permit 276.5 22 35)))))
 ; Ŀ
 ;   Hocs - they have something for everyone.                              
 ; 
  (if (and (tblsearch "block" "GC_A3_ALT-TITLEBLOCK")
           (setq ss (ssget "x" (list (cons 2 "GC_A3_ALT-TITLEBLOCK")))))
      (setq blizt (append blizt
                    (list (list "GC_A3_ALT-TITLEBLOCK" beaver 181 18.8 28)
                          (list "GC_A3_ALT-TITLEBLOCK" permit 214.25 17.6 31.5)))))
 ; Ŀ
 ;   Hocs.  Iso tb - a variant I made, against my better judgement.        
 ; 
  (if (and (tblsearch "block" "GC_A3_ALT2-TITLEBLOCK")
           (setq ss (ssget "x" (list (cons 2 "GC_A3_ALT2-TITLEBLOCK")))))
      (setq blizt (append blizt
                    (list (list "GC_A3_ALT2-TITLEBLOCK" beaver 221 18.8 28)
                          (list "GC_A3_ALT2-TITLEBLOCK" permit 253.8 17.6 31.5)))))
 ; Ŀ
 ;   HORZ_2DbordA1.                                                        
 ;   Note that the distances and scales are divided by 1000, the block     
 ;   being inserted at 1000 x Dimscale.                                    
 ; 
  (if (and (tblsearch "block" "horz_2dborda1")
           (setq ss (ssget "x" (list (cons 2 "horz_2dborda1")))))
      (setq blizt (append blizt
                    (list (list "horz_2dborda1" beaver 0.751 0.1747 0.042)
                          (list "horz_2dborda1" permit 0.8175 0.174 0.065)))))
 ; Ŀ
 ;   Husky - Husky-2x.                                                     
 ; 
  (if (and (tblsearch "block" "Husky-2x")
           (setq ss (ssget "x" (list (cons 2 "Husky-2x")))))
      (setq blizt (append blizt
                    (list (list "Husky-2x" beaver 50 30 48)
                          (list "Husky-2x" permit 140 30 78)))))
 ; Ŀ
 ;   HUSKY ANSI D Tblock Attributes.  FFS.                                 
 ; 
  (if (and (tblsearch "block" "HUSKY ANSI D Tblock Attributes")
           (setq ss (ssget "x" (list (cons 2 "HUSKY ANSI D Tblock Attributes")))))
      (setq blizt (append blizt
                    (list (list "HUSKY ANSI D Tblock Attributes" beaver 801 294 48)
                          (list "HUSKY ANSI D Tblock Attributes" permit 716.5 295 78)))))
 ; Ŀ
 ;   HUSKY HOOL ANSI D Tblock Attributes.                                  
 ;   What is wrong with you people?                                        
 ; 
  (if (and (tblsearch "block" "HUSKY HOOL ANSI D Tblock Attributes")
           (setq ss (ssget "x" (list (cons 2 "HUSKY HOOL ANSI D Tblock Attributes")))))
      (setq blizt (append blizt
                    (list (list "HUSKY HOOL ANSI D Tblock Attributes" beaver 801 294 48)
                          (list "HUSKY HOOL ANSI D Tblock Attributes" permit 716.5 295 78)))))
 ; Ŀ
 ;   HUSKY A1tblk.                                                         
 ;   Simon, this was added by Danielle C out of desperation.               
 ;   Not bad for a first attempt.                                          
 ; 
  (if (and (tblsearch "block" "a1tblk")
           (setq ss (ssget "x" (list (cons 2 "a1tblk")))))
      (setq blizt (append blizt
                    (list (list "a1tblk" beaver 801 294 48)
                          (list "a1tblk" permit 140 31.5 78)))))
 ; Ŀ
 ;   "Pel D Size" - similar to Bord_A1.                                    
 ;   Hint to people making title blocks - steal one and change the name.   
 ;   Don't mess with it because you don't know what you're doing.          
 ; 
  (if (and (tblsearch "block" "pel d size")
           (setq ss (ssget "x" (list (cons 2 "pel d size")))))
      (setq blizt (append blizt
                    (list (list "pel d size" beaver 643.5 45 42)
                          (list "pel d size" permit 584.5 45.25 54)))))
 ; Ŀ
 ;   "Pel D Size(Color)" - similar to Pel D Size.  Amazingly.              
 ; 
  (if (and (tblsearch "block" "PEL D Size(Color)")
           (setq ss (ssget "x" (list (cons 2 "PEL D Size(Color)")))))
      (setq blizt (append blizt
                    (list (list "PEL D Size(Color)" beaver 643.5 45 42)
                          (list "PEL D Size(Color)" permit 584.5 45.25 54)))))
 ; Ŀ
 ;   "New Pel D Size" - ...                                                
 ; 
  (if (and (tblsearch "block" "New PEL D Size")
           (setq ss (ssget "x" (list (cons 2 "New PEL D Size")))))
      (setq blizt (append blizt
                    (list (list "New PEL D Size" beaver 643.5 45 42)
                          (list "New PEL D Size" permit 584.5 45.25 54)))))
 ; Ŀ
 ;   Shawn - this seems to be mostly a copy of Bord_A1.                    
 ;   So this is imbecilic on numerous counts.                              
 ; 
  (if (and (tblsearch "block" "shawn")
           (setq ss (ssget "x" (list (cons 2 "shawn")))))
      (setq blizt (append blizt
                    (list (list "shawn" beaver 647.5 45.1 42)
                          (list "shawn" permit 590.5 45.25 59)))))
 ; Ŀ
 ;   "8.5x11 Title Block" - descriptive.  Exergy again?                    
 ; 
  (if (and (tblsearch "block" "8.5x11 Title Block")
           (setq ss (ssget "x" (list (cons 2 "8.5x11 Title Block")))))
      (setq blizt (append blizt
                    (list (list "8.5x11 Title Block" beaver 288 151.5 42)
                          (list "8.5x11 Title Block" permit 357.5 152.5 80)))))
 ; Ŀ
 ;   Meg.  Obviously named by someone with a firm grip on reality.         
 ; 
  (if (and (tblsearch "block" "ENG-BAS-SD-0001_R0 - MEG Energy Autocad Titleblock Drawing")
           (setq ss (ssget "x" (list (cons 2 "ENG-BAS-SD-0001_R0 - MEG Energy Autocad Titleblock Drawing")))))
      (setq blizt (append blizt
                    (list (list "ENG-BAS-SD-0001_R0 - MEG Energy Autocad Titleblock Drawing"
                                beaver 424 51 44)
                          (list "ENG-BAS-SD-0001_R0 - MEG Energy Autocad Titleblock Drawing"
                                permit 516 50 78)))))
 ; Ŀ
 ;   Meg.  Another nearly identical block.  But with a good name.          
 ; 
  (if (and (tblsearch "block" "MEG_Energy_A1")
           (setq ss (ssget "x" (list (cons 2 "MEG_Energy_A1")))))
      (setq blizt (append blizt
                    (list (list "MEG_Energy_A1" beaver 493 25 45)
                          (list "MEG_Energy_A1" permit 438 25 58)))))
 ; Ŀ
 ;   new blockCIV - From MEG?  A local asylum?  A kindergarten?            
 ; 
  (if (and (tblsearch "block" "new blockCIV")
           (setq ss (ssget "x" (list (cons 2 "new blockCIV")))))
      (setq blizt (append blizt
                    (list (list "new blockCIV" beaver 493 25 45)
                          (list "new blockCIV" permit 438 25 58)))))
 ; Ŀ
 ;   Novagas - NCLTTLD.                                                    
 ; 
  (if (and (tblsearch "block" "nclttld")
           (setq ss (ssget "x" (list (cons 2 "nclttld")))))
      (setq blizt (append blizt
                    (list (list "nclttld" beaver 501.75 40.75 42)
                          (list "nclttld" permit 556.8 41.25 60)))))
 ; Ŀ
 ;   Novagas - NCLTTLD1.  I'm sure this was necessary.                     
 ; 
  (if (and (tblsearch "block" "nclttld1")
           (setq ss (ssget "x" (list (cons 2 "nclttld1")))))
      (setq blizt (append blizt
                    (list (list "nclttld1" beaver 501.75 40.75 42)
                          (list "nclttld1" permit 556.8 41.25 60)))))
 ; Ŀ
 ;   Paramount: Paramount_ENG3_tb.                                         
 ; 
  (if (and (tblsearch "block" "Paramount_ENG3_tb")
           (setq ss (ssget "x" (list (cons 2 "Paramount_ENG3_tb")))))
      (setq blizt (append blizt
                    (list (list "Paramount_ENG3_tb" beaver 420.74 43.93 56)
                          (list "Paramount_ENG3_tb" permit 500.24 44.74 80)))))
 ; Ŀ
 ;   Paramount: PARAMOUNT_D(metric).                                       
 ; 
  (if (and (tblsearch "block" "PARAMOUNT_D(metric)")
           (setq ss (ssget "x" (list (cons 2 "PARAMOUNT_D(metric)")))))
      (setq blizt (append blizt
                    (list (list "PARAMOUNT_D(metric)" beaver 225 45 60)
                          (list "PARAMOUNT_D(metric)" permit 308 45 78)))))
 ; Ŀ
 ;   Penn West.                                                            
 ; 
  (if (and (tblsearch "block" "PennWest TB_D Size-MS")
           (setq ss (ssget "x" (list (cons 2 "PennWest TB_D Size-MS")))))
      (setq blizt (append blizt
                    (list (list "PennWest TB_D Size-MS" beaver 488 41 42)
                          (list "PennWest TB_D Size-MS" permit 564 42 64)))))
 ; Ŀ
 ;   PERPETUAL_D BORDER.                                                   
 ; 
  (if (and (tblsearch "block" "PERPETUAL_D BORDER")
           (setq ss (ssget "x" (list (cons 2 "PERPETUAL_D BORDER")))))
      (setq blizt (append blizt
                    (list (list "PERPETUAL_D BORDER" beaver 494.6 38.5 50)
                          (list "PERPETUAL_D BORDER" permit 566 37 78)))))
 ; Ŀ
 ;   PPC-Dsize.  Pembina.                                                  
 ; 
  (if (and (tblsearch "block" "PPC-Dsize")
           (setq ss (ssget "x" (list (cons 2 "PPC-Dsize")))))
      (setq blizt (append blizt
                    (list (list "PPC-Dsize" beaver 141 26 35)
                          (list "PPC-Dsize" permit 202.5 26 78)))))
 ; Ŀ
 ;   Provident.                                                            
 ; 
  (if (and (tblsearch "block" "Provident TB D Size")
           (setq ss (ssget "x" (list (cons 2 "Provident TB D Size")))))
      (setq blizt (append blizt
                    (list (list "Provident TB D Size" beaver 602.5 42.5 45)
                          (list "Provident TB D Size" permit 584.5 45.5 54)))))
 ; Ŀ
 ;   Secure.                                                               
 ; 
  (if (and (tblsearch "block" "Secure_A1_Titleblock")
           (setq ss (ssget "x" (list (cons 2 "Secure_A1_Titleblock")))))
      (setq blizt (append blizt
                    (list (list "Secure_A1_Titleblock" beaver 642.5 45.4 45)
                          (list "Secure_A1_Titleblock" permit 537.5 45.5 70)))))
 ; Ŀ
 ;   Secure ... another.                                                   
 ; 
  (if (and (tblsearch "block" "Secure_A1a_Titleblock")
           (setq ss (ssget "x" (list (cons 2 "Secure_A1a_Titleblock")))))
      (setq blizt (append blizt
                    (list (list "Secure_A1a_Titleblock" beaver 630 33 45)
                          (list "Secure_A1a_Titleblock" permit 525 33 70)))))
 ; Ŀ
 ;   Tcmtb.  And we're not talking about where this came from.             
 ; 
  (if (and (tblsearch "block" "tcmtb")
           (setq ss (ssget "x" (list (cons 2 "tcmtb")))))
      (setq blizt (append blizt
                    (list (list "tcmtb" beaver 642.5 45.6 42)
                          (list "tcmtb" permit 584.5 45.5 54)))))
 ; Ŀ
 ;   Tiwwtb.                                                               
 ; 
  (if (and (tblsearch "block" "tiwwtb")
           (setq ss (ssget "x" (list (cons 2 "tiwwtb")))))
      (setq blizt (append blizt
                    (list (list "tiwwtb" beaver 12.671875 0.921875 1.5)
                          (list "tiwwtb" permit 15.640625 0.859375 4)))))
 ; Ŀ
 ;   Tridyne.                                                              
 ; 
  (if (and (tblsearch "block" "Tridyne_tb_d")
           (setq ss (ssget "x" (list (cons 2 "Tridyne_tb_d")))))
      (progn
           (setq atnum (cadr (coux "Tridyne_tb_d")))
           (cond ((equal atnum 81)
                  (setq blizt (list (list "Tridyne_tb_d"
                                           beaver 236.75 36.0223 42.5)
                                    (list "Tridyne_tb_d"
                                           permit 316.7500 36.1741 63.5))))
                 (t      ; 95 attributes
                  (setq blizt (list (list "Tridyne_tb_d" beaver 189 31 42.5)
                              (list "Tridyne_tb_d" permit 254 33 63.5)))))))
 ; Ŀ
 ;   "Wec Dsize Metric".                                                   
 ; 
  (if (and (tblsearch "block" "wec dsize metric")
           (setq ss (ssget "x" (list (cons 2 "wec dsize metric")))))
      (setq blizt (append blizt
                    (list (list "wec dsize metric" beaver 337 32 50)
                          (list "wec dsize metric" permit 259 31 80)))))
 ; Ŀ
 ;   See if a title block can be found.                                    
 ; 
  (setq num 0)
  (while (and blizt (setq blsub (nth num blizt)))
         (setq num (1+ num))
         (setq blnam (car blsub))
         (if (tblsearch "block" blnam)
             (progn
                  (setq stanam (cadr blsub))
                  (setq ofx (caddr blsub))
                  (setq ofy (cadddr blsub))
                  (setq bsiz (nth 4 blsub))
                  (if (fishth blnam stanam ofx ofy bsiz "0")
                      (setq folio (cons stanam folio))))))
 ; Ŀ
 ;   Print a warning if no title block was found, mostly for debugging.    
 ;   This isn't conclusive: a title block may be defined but not inserted. 
 ; 
  (if (null folio) (write-line "\nNo Title Block Found."))
 ; Ŀ
 ;   Erase and purge any blocks which were inserted.                       
 ;   This isn't a good idea unless the plot command was called before now. 
 ;   Note that Draed doesn't have an internal list and is thus faster      
 ;   than Idak, it is included so that the calling program can use it to   
 ;   erase and purge the stamp blocks after plotting, if desired.          
 ; 
 ; (draed folio)  ; this might be dumb since they were just inserted.
 (princ))
 ; Ŀ
 ;   Prawn End.                                                            
 ; 

 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))


