 ; Ŀ
 ;   Python - convert numerals in a text string to words.                  
 ;   Copyright 1994, 2012 by Rocket Software Ltd.                          
 ;   Snakes would make better pets if they had ears.  And fur.             
 ; 

 ; Ŀ
 ;   Pontoon - split a text string at the decimal point (if any).          
 ;   Returns a list of two strings.                                        
 ; 
 (DEFUN PONTOON (aa / pos len bb cc)
  (setq pos 1)
  (setq len (strlen aa))
  (while (>= len pos)
         (setq bb (substr aa pos 1))
         (if (= bb ".")
             (progn
                  (setq cc (substr aa (1+ pos)))
                  (setq aa (substr aa 1 (1- pos)))))
         (setq pos (1+ pos)))
  (list aa cc))
 ; Ŀ
 ;   Pontoon end.                                                          
 ; 

 ; Ŀ
 ;   Apt - transliterate numerals to text.                                 
 ; 
 (DEFUN APT (str / whole nos pos aaa)
  (setq whole " POINT")
  (setq nos (list "ZERO" "ONE" "TWO" "THREE" "FOUR"
                   "FIVE" "SIX" "SEVEN" "EIGHT" "NINE"))
  (setq pos 1)
  (while (/= (setq aaa (substr str pos 1)) "")
         (if (= aaa ".")
             (setq whole (strcat whole "."))
             (setq whole (strcat whole " " (nth (read aaa) nos))))
         (setq pos (1+ pos)))
 ; Ŀ
 ;   If the string consists solely of the word "point" then it should      
 ;   remain a period.                                                      
 ; 
  (if (= whole " POINT") (setq whole "."))
 whole)
 ; Ŀ
 ;   Apt end.                                                              
 ; 
  
 ; Ŀ
 ;   Monster - returns a list of strings, alternating between numbers and  
 ;   letters. The first string is always a character string - if the       
 ;   first character is a number then the first string will be empty (""). 
 ;   A period is a decimal point if preceded by a number.                  
 ; 
 (DEFUN MONSTER (string / chastr pre ascnum posi this asc last strlist)
  (setq chastr "")
  (setq pre "c")
  (setq ascnum (list 48 49 50 51 52 53 54 55 56 57))
  (setq posi 1)
  (while (/= "" (setq this (substr string posi 1)))
       (setq asc (ascii this))
       (cond ((member asc ascnum)         ; if char is a number
              (setq last "n"))
             ((= asc 46)                  ; if char is a .
              (if (= last "c")
                  (setq last "c")
                  (setq last "n")))
             (T                           ; otherwise it must be a letter
               (setq last "c")))
 ; Ŀ
 ;   If pre = last then strcat This onto chastr.                           
 ;   If not then append chastr onto strlist and set chastr to this.        
 ; 
       (if (equal pre last)
           (setq chastr (strcat chastr this))
           (progn
                (if strlist
                   (setq strlist (append strlist (list chastr)))
                   (setq strlist (list chastr)))
                (setq chastr this)))
       (setq pre last)
       (setq posi (1+ posi)))
  (if chastr (setq strlist (append strlist (list chastr))))
  strlist)
 ; Ŀ
 ;   Monster end.                                                          
 ; 

 ; Ŀ
 ;   One: convert a numeral to its English equivalent.                     
 ;   Useful for convincing alien life forms that you are not edible.       
 ; 
 (DEFUN ONE (str / cycle whole nos set2 set3 set4
                                            last3 strn one two thr lne num ok)
  (setvar "cmdecho" 0)
  (setq cycle 0)
  (setq whole "")
  (setq nos (list "" "one" "two" "three" "four"
                   "five" "six" "seven" "eight" "nine"))
  (setq set2 (list "ten" "eleven" "twelve" "thirteen" "fourteen"
                   "fifteen" "sixteen" "seventeen" "eighteen" "nineteen"))
  (setq set3 (list "" "" "twenty" "thirty" "forty" "fifty" "sixty"
                   "seventy" "eighty" "ninety"))
  (setq set4 (list "" "thousand" "million" "billion" "trillion" "quadrillion"
                   "pentillion" "hexillion" "heptillion" "octillion"
                   "nonillion" "decillion" "zillion" "bazillion"))
  (while (> (strlen str) 0)
  (while (< (strlen str) 3) (setq str (strcat "0" str)))
  (setq last3 (substr str (- (strlen str) 2)))
  (if (> (strlen str) 3) (setq str (substr str 1 (- (strlen str) 3)))
                         (setq str ""))
 ; Ŀ
 ;   Process next three.                                                   
 ; 
  (setq strn "")
  (setq one (substr last3 1 1))
  (setq two (substr last3 2 1))
  (setq thr (substr last3 3 1))
  (if (/= one "0") (setq strn (strcat (nth (read one) nos) " hundred")))
  (cond ((= two "1") (setq strn (strcat strn 
                                        (if (/= strn "") " and " "")
                                        (nth (read thr) set2))))
        ((= two "0"))
        (t (setq strn (strcat strn 
                              (if (= strn "") "" " and ")
                              (nth (read two) set3)))))
  (if (/= two "1")
      (setq strn (strcat strn
                   (if (and (/= strn "") (= two "0") (/= thr "0")) " and " "")
                   (if (and (/= strn "") (/= two "0")
                            (/= two "1") (/= thr "0")) " " "")
                   (nth (read thr) nos))))
 ; Ŀ
 ;   Add "and" for example in one thousand and sixty two but not in 0062.  
 ;   (And only do this for the last three digits, that is if cycle = 0)    
 ; 
  (if (and (/= str "") (= one "0") (/= strn "") (= cycle 0))
      (progn
            (setq lne (strlen str))
            (setq num 1)
            (setq ok ())
            (while (<= num lne)
                   (if (/= (substr str num 1) "0") (setq ok t))
                   (setq num (1+ num)))
            (if ok (setq strn (strcat "and " strn)))))
 ; Ŀ
 ;   Process three End.                                                    
 ; 
  (setq whole (strcat strn 
                      (if (= strn "")
                          "" 
                          (strcat " " (nth cycle set4) " "))
                      whole))
  (setq cycle (1+ cycle)))
 whole)
 ; Ŀ
 ;   One end.                                                              
 ; 

 ; Ŀ
 ;   Subroutine Python - convert a single string.                          
 ;   Argument: Str, a string.                                              
 ;   Returns the changed version.                                          
 ; 
 (DEFUN PYTHON (str / seplst num finster ptlist aaa bbb inter)
  (setq seplst (monster str))
  (setq num 0)
  (setq finster "")                      ; initialise main string
  (while (setq aaa (nth num seplst))     ; get next string from list
         (if (= (rem num 2) 1)           ; see if a char or numeral string
             (progn                      ; if a numeral string
 ; Ŀ
 ;   Check aaa for leading Zeros (or all zeros).                           
 ; 
                  (while (= (substr aaa 1 1) "0")
                         (setq aaa (substr aaa 2))
                         (setq inter (if (= finster "") "" " "))
                         (setq finster (strcat finster inter "ZERO")))
 ; Ŀ
 ;   Now use Apt to see if there is a decimal point in aaa.  If so feed    
 ;   everything up to the point to One and everything after it to Pontoon. 
 ;   Then put the two resulting strings back together.                     
 ; 
                  (setq ptlist (pontoon aaa))
 ; Ŀ
 ;   Call One to convert the first part of ptlist to English.              
 ; 
                  (setq aaa (one (car ptlist)))
 ; Ŀ
 ;   And call Pontoon to do the last part (if any).                        
 ; 
                  (if (cadr ptlist)
                      (setq bbb (apt (cadr ptlist))))
 ; Ŀ
 ;   Lose any trailing zeros from aaa (probably two if aaa /= "").         
 ; 
                  (while (and (/= aaa "")
                              (= (substr aaa (strlen aaa)) " "))
                         (setq aaa (substr aaa 1 (1- (strlen aaa)))))
                  (setq aaa (strcase aaa))))
 ; Ŀ
 ;   Now put aaa and bbb back together.                                    
 ; 
                  (if bbb (setq aaa (strcat aaa bbb)))
 ; Ŀ
 ;   See if a space is required before aaa (i.e. if Finster /= "").        
 ; 
         (setq inter (if (= finster "") "" " "))
 ; Ŀ
 ;   And add the new string to the main string.                            
 ; 
         (setq finster (strcat finster inter aaa))
         (setq num (1+ num)))
 finster)
 ; Ŀ
 ;   Subroutine Python end.                                                
 ; 

 ; Ŀ
 ;   Python - the big reptile.                                             
 ; 
 (DEFUN C:PYTHON (/ enam str)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq enam (car (entsel "Select text:\n")))
  (setq str (cdr (assoc 1 (setq entt (entget enam)))))
  (setq str (python str))
  (entmod (subst (cons 1 str) (assoc 1 entt) entt))
  (command "undo" "end")
 (princ))