 ; Ŀ
 ;   Repth: give xrefs relative paths.                                     
 ;   Copyright 2010, 2011, 2012 by Rocket Software Ltd.                    
 ;                                                                         
 ; 

 ; Ŀ
 ;   Contx - Find all referenced xref definitions in the block tables.     
 ;   Takes no arguments.                                                   
 ;   Returns a list of lists: ((Blockname Filename)...)                    
 ; 
 (DEFUN CONTX (/ rew bldat sevnt namlst)
  (setq rew t)
  (while (setq bldat (tblnext "block" rew))
         (setq rew ())
         (setq sevnt (cdr (assoc 70 bldat)))
         (if (or (= 4 (logand 4 sevnt)) (= 8 (logand 8 sevnt)))
             (setq namlst (append namlst (list (list (cdr (assoc 2 bldat))
                                                   (cdr (assoc 1 bldat))))))))
 namlst)
 ; Ŀ
 ;   Contx end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Knost - Remove the last step from a path string.           
 ;   Arguments: Patha, the path string.                                    
 ;   Returns a string which will be empty if there is no "\\".             
 ; 
 (DEFUN KNOST (patha / len)
 ; Ŀ
 ;   If there is a trailing backslash then remove it.                      
 ; 
  (if (and (/= (strlen patha) 0)
           (= (substr patha (setq len (strlen patha))) "\\"))
      (setq patha (substr patha 1 (1- len))))
 ; Ŀ
 ;   Remove everything up to the next one.                                 
 ; 
  (while (and (/= (strlen patha) 0)
              (/= (substr patha (setq len (strlen patha))) "\\"))
         (setq patha (substr patha 1 (1- len))))
 patha)
 ; Ŀ
 ;   Knost end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Mchop - chop the length of a string off the front of       
 ;   another.                                                              
 ;   Arguments: Stra, the string to chop.                                  
 ;              Stch, the string to indicate the length to chop.           
 ;   Returns a chopped string which may be empty if the cut length was >=  
 ;   the base string length.                                               
 ; 
 (DEFUN MCHOP (stra stch / len)
  (setq len (strlen stch))
 (substr stra (1+ len)))
 ; Ŀ
 ;   Mchop end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Pama - see if two strings match along the shortest         
 ;   shared length.                                                        
 ;   Arguments: Stra, a string.                                            
 ;              Strb, another string.                                      
 ;   Returns t or nil.                                                     
 ; 
 (DEFUN PAMA (stra strb / len)
  (setq len (min (strlen stra) (strlen strb)))
  (= (strcase (substr stra 1 len))
     (strcase (substr strb 1 len))))
 ; Ŀ
 ;   Pama end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Repth - make a relative path.                              
 ;   Arguments: Xpath, the xref name and path string.                      
 ;              Cpath, the current drawing path string.                    
 ;   Calls Knost and Mchop.                                                
 ;   Returns a relative path string if one is possible.                    
 ;   Could be modified to allow for different drive letters.               
 ; 
 (DEFUN REPTH (xpath cpath / num)
 ; Ŀ
 ;   Chop steps off the current path until the remainder matches.          
 ; 
  (setq num 0)
  (while (null (pama xpath cpath))
         (setq num (1+ num))
         (setq cpath (knost cpath)))
  (setq xpath (mchop xpath cpath))
  (repeat num (setq xpath (strcat "..\\" xpath)))
 xpath)
 ; Ŀ
 ;   Subroutine Repth end.                                                 
 ; 

 ; Ŀ
 ;   Repth.                                                                
 ; 
 (DEFUN C:REPTH (/ *error* fnam xlist cpath num xname patha len nam)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
 (defun *error* (shk / fnam)
  (setq fnam (getvar "dwgname"))
  (if regist (regist (strcat "***** Error in Repth.lsp, File: " fnam)))
  (if shk (print shk))
 (princ))
 ; Ŀ
 ;   Save xref layer settings in the local drawing.                        
 ; 
  (setvar "visretain" 1)
 ; Ŀ
 ;   Get a list of all xrefs.                                              
 ; 
  (if (setq xlist (contx))
      (if regist (regist (strcat "* Xrefs repathed in drawing "
                                 (nopath) ":"))))
 ; Ŀ
 ;   Get the current drawing path.                                         
 ; 
  (setq cpath (getvar "dwgprefix"))
 ; Ŀ
 ;   Repath each one.                                                      
 ; 
  (setq num 0)
  (while (and xlist (setq xname (nth num xlist)))
         (setq num (1+ num))
 ; Ŀ
 ;   Get the block name and path.                                          
 ; 
         (setq patha (cadr xname))  ; xref path with block name .dwg
         (setq xname (car xname))   ; xref block name
 ; Ŀ
 ;   If the path is relative or not stated, don't change it.               
 ; 
         (cond ((or (= xname patha)
                    (and (> (setq len (strlen patha)) 4)
                         (setq nam (substr patha 1 (- len 4)))
                         (= nam xname))
                    (= (substr patha 1 2) "..")))
 ; Ŀ
 ;   Files located in the current directory don't need a path.             
 ; 
               ((= (strcase cpath) (strcase patha))
                (setq patha xname)
                (command "xref" "path" xname patha)
                (if regist (regist (strcat xname " pathed locally."))))
 ; Ŀ
 ;   If all else fails, make a new path.                                   
 ; 
               (t
                (setq patha (repth patha cpath))
                (command "xref" "path" xname patha)
                (if regist (regist (strcat xname
                                         " changed to relative path: "
                                           patha))))))
 ; Ŀ
 ;   Add an end marker the the register file.                              
 ; 
  (if (and xlist regist) (regist "Xrefs End"))
 (princ))