 ; Ŀ
 ;   Resc - scale the insertion points of stuff but not the entities.      
 ;   Copyright 2016 by Rocket Software Ltd.                                
 ;                                                                         
 ; 

 ; Ŀ
 ;   Gresp - remove all groups containing an entity from an ss.            
 ;   If the entity isn't in any groups then it is removed from the ss.     
 ;   Arguments: Enam, the entity name.                                     
 ;              SS, the ss.                                                
 ;   Calls Grrr, returns the ss name.                                      
 ; 
 (DEFUN GRESP (enam ss / grlist num sub nmam)
 ; Ŀ
 ;   Grrr returns a list of lists (or nil):                                
 ;   ((group_name group_ename member_ename_1 member_ename_2 ...) ...)      
 ; 
  (if (setq grlist (grrr enam))
      (progn
           (setq num 0)
           (while (setq sub (nth num grlist))
                  (setq num (1+ num))
                  (setq sub (cddr sub))
                  (while (setq mnam (car sub))
                         (setq sub (cdr sub))
                         (ssdel mnam ss))))
      (ssdel enam ss))
 ss)
 ; Ŀ
 ;   Gresp end.                                                            
 ; 

 ; Ŀ
 ;   Grrr - get group data for an entity.                                  
 ;   Takes one argument: Enam - the entity name.                           
 ;   Returns a list of lists:                                              
 ;   ((group_name group_ename member_ename_1 member_ename_2 ...) ...)      
 ;   or nil if the entity wasn't a member of any groups.                   
 ;                                                                         
 ;   Note that a group can't contain other groups as such - including one  
 ;   group in another adds its members.                                    
 ; 
 (DEFUN GRRR (enam / entt grlist master mast2)
  (setq entt (entget enam))
 ; Ŀ
 ;   There will be one 330 group in the entity data for each group to      
 ;   which the object belongs.  Note that an entity can belong to          
 ;   multiple groups.  So get a list of their enames.                      
 ;   Also: check to see if this really is a group and not some other type  
 ;   of reactor.  (The 0 group in the entity data for the 330 group in     
 ;   the original entity is (0 . "GROUP"))                                 
 ; 
  (setq num 0)
  (while (setq sub (nth num entt))
         (if (and (= (car sub) 330)
                  (= (cdr (assoc 0 (entget (cdr sub)))) "GROUP"))
             (setq grlist (cons (cdr sub) grlist)))
         (setq num (1+ num)))
 ; Ŀ
 ;   If any of the 330 enames were Groups, add the enames of the entities  
 ;   contained in each group to the appropriate sublist.                   
 ; 
  (if grlist
      (progn
           (setq num 0)
           (while (setq grenam (nth num grlist))
                  (setq subent (entget grenam))
                  (setq newlst (list grenam))
                  (setq subnum 0)
                  (while (setq sub (nth subnum subent))
                         (if (= (car sub) 340)
                             (setq newlst (append newlst (list (cdr sub)))))
                         (setq subnum (1+ subnum)))
                  (setq master (cons newlst master))
                  (setq num (1+ num)))
 ; Ŀ
 ;   Add the group name of each group to the start of its sublist.         
 ;   First get a copy of the master group list.                            
 ; 
           (setq grdict (dictsearch (namedobjdict) "acad_group"))
           (setq grnams (reverse grdict))
 ; Ŀ
 ;   Get the name for each group from the list.                            
 ; 
           (setq num 0)
           (while (setq sub (nth num master))
                  (setq grenam (car sub))
                  (setq grpnam (cadr (member (cons 350 grenam) grnams)))
                  (setq sub (cons (cdr grpnam) sub))
                  (setq mast2 (cons sub mast2))
                  (setq num (1+ num)))))
 mast2)
 ; Ŀ
 ;   Grrr end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Sclam: move an entity by a scale factor.                   
 ;   Arguments: Enam, the entity name.                                     
 ;              Bap, the base point.                                       
 ;              Scal, the scale factor.                                    
 ;   Calls Spit, Returns nothing.  Likes black and white film.             
 ; 
 (DEFUN SCLAM (enam bap scal / entt pa dist angg gnudis pa1 typp)
  (setq entt (entget enam))
  (setq typp (cdr (assoc 0 entt)))
  (cond ((member typp '("ATTDEF" "TEXT"))
         (setq pa (spit entt))
         (setq pa (cdr (assoc 10 entt))))
        ((equal typp "POLYLINE")
         (setq pa (cdr (assoc 10 (entget (entnext enam)))))))
  (setq bap (reverse (cdr (reverse bap)))) ; ignore vertical distances
  (setq pa (reverse (cdr (reverse pa))))   ; same
  (setq dist (distance bap pa))
  (setq angg (angle bap pa))
  (setq gnudis (* scal dist))
  (setq pa1 (polar bap angg gnudis))
  (command ".move" enam "" pa pa1)
 (princ))
 ; Ŀ
 ;   Subroutine Sclam end.                                                 
 ; 

 ; Ŀ
 ;   Spit - returns the insertion point of the text entity whose data was  
 ;   passed as its sole argument.  Note that this is not necessarily the   
 ;   same as the 10 association code.                                      
 ; 
 (DEFUN SPIT (entt / xjust yjust)
  (setq xjust (cdr (assoc 72 entt)))
  (if (= (cdr (assoc 0 entt)) "ATTDEF")
      (setq yjust (cdr (assoc 74 entt)))
      (setq yjust (cdr (assoc 73 entt))))
  (if (or (/= xjust 0) (/= yjust 0))
      (cdr (assoc 11 entt))
      (cdr (assoc 10 entt))))
 ; Ŀ
 ;   Spit end.                                                             
 ; 

 ; Ŀ
 ;   Resc.                                                                 
 ; 
 (DEFUN C:RESC (/ osmo *error* bap scal0 scal1 scalf ss num enam)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
  (setq osmo (getvar "osmode"))
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (setvar "osmode" osmo)
   (setvar "oleframe" 0)
   (command "imageframe" 0)
   (command "wipeout" "f" "off")
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
   (if shk (write-line shk))
  (princ))
 ; Ŀ
 ;   Get some entities.                                                    
 ; 
  (write-line "\nStuff to Pseudoscale: ")
  (setq ss (ssget))
 ; Ŀ
 ;   ... a base point ...                                                  
 ; 
  (setq bap (getpoint "\nBase Point: "))
  (princ bap)
  (setvar "osmode" 0)
 ; Ŀ
 ;   ... and a scale factor.                                               
 ; 
  (setq scal0 (getreal "\nOriginal Scale: "))
  (setq scal1 (getreal "\nNew Scale: "))
  (setq scalf (/ scal1 scal0))  
 ; Ŀ
 ;   Turn on some frames so they can be moved.                             
 ; 
  (setvar "oleframe" 1)
  (command "imageframe" 1)
  (command "wipeout" "f" "on")
 ; Ŀ
 ;   Rescale stuff.                                                        
 ; 
  (while (setq enam (ssname ss 0))
         (sclam enam bap scalf)
 ; Ŀ
 ;   Stuff in groups is moved together, which seems like a good idea,      
 ;   so must remove the whole group from the ss so it won't be moved       
 ;   once for each member.                                                 
 ; 
         (gresp enam ss))
  (*error* ())
 (princ))
