 ; Ŀ
 ;   Secup - rev up a Secure_A1_Titleblock.                                
 ;   Copyright 2002, 2005, 2008 - 2012 by Rocket Software Ltd.             
 ;   History is written by the survivors, so if you like to write,         
 ;   try to survive.                                                       
 ; 

 ; Ŀ
 ;   Blint - place new values into a block starting at a given attribute.  
 ;   Arguments: Enam, the attribute name at which to start.                
 ;              Vlasta, the list of new values.                            
 ;   Caution: doesn't check to see if there are attributes available.      
 ; 
 (DEFUN BLINT (enam vlasta / esav gnuval entt)
  (setq esav enam)
  (while (setq gnuval (car vlasta))
         (setq vlasta (cdr vlasta))
         (setq entt (entget enam))
         (entmod (subst (cons 1 gnuval) (assoc 1 entt) entt))
         (setq enam (entnext enam)))
  (entupd esav)
 (princ))
 ; Ŀ
 ;   Blint end.                                                            
 ; 

 ; Ŀ
 ;   Exstr - extract a number of attribute values, return them as a list   
 ;   with the ename of the next following attribute as the first element.  
 ;   Arguments: Enam, the entity name of the first attribute to extract.   
 ;              Num, the number of attributes to extract.                  
 ;                                                                         
 ;   Called by Fudge.  Which sounds like a Motorhead song.                 
 ;   Calls nothing.  But has good intentions otherwise.                    
 ;   Returns a list: (next_ename string1 string2 ... string_num)           
 ; 
 (DEFUN EXSTR (enam num / vall malist)
  (repeat num (setq vall (cdr (assoc 1 (entget enam))))
              (setq enam (entnext enam))
              (setq malist (cons vall malist)))
  (cons enam (reverse malist)))
 ; Ŀ
 ;   Exstr end.                                                            
 ; 

 ; Ŀ
 ;   Fudge - If a title block rev area is full, delete the lowest rev      
 ;   line of data and shuffle the rest down.                               
 ;   Arguments: Enam, the ename of the block insertion.                    
 ;              Numlin, the number of rev lines.                           
 ;              Atsper, the number of attributes per line.                 
 ;              Revpos, the 0 based position of the first rev attribute.   
 ;   Returns a list: the ename of the first attribute in the first empty   
 ;   rev line and the highest rev number.                                  
 ; 
 (DEFUN FUDGE (enam numlin atsper revpos / esav1 revlst esav rsave toprev
                                                    llist sub meta entt vall)
  (setq enam (entnext enam)) ; first ename after the insert
 ; Ŀ
 ;   Find the first rev line.                                              
 ; 
  (repeat revpos (setq enam (entnext enam)))
 ; Ŀ
 ;   Save the first rev number ename as the reapplication start point.     
 ; 
  (setq esav1 enam)
 ; Ŀ
 ;   Extract the next Numlin lines into a list of value lists.             
 ;   Ignore any rev line which does not begin with a legitimate value.     
 ; 
  (repeat numlin (setq revlst (exstr enam atsper))
 ; Ŀ
 ;   Save the ename for the first attribute on the current rev line in     
 ;   case it is the first empty rev.                                       
 ; 
                 (setq esav enam)
                 (setq enam (car revlst))     ; next ename after extracted ones
                 (setq revlst (cdr revlst))   ; list of extracted strings
                 (if (member (car revlst) '("" " " "." "-" "..."))
                     (if (null rsave) (setq rsave esav))
                     (progn
                          (setq toprev (car revlst))
                          (setq llist (append llist (list revlst))))))
 ; Ŀ
 ;   If there are Numlin lines, ditch the first one and reinstall them.    
 ;   The ename returned as the starting point for the new rev is thus the  
 ;   one in Esav.                                                          
 ;   Otherwise the correct value is the one in Rsave.                      
 ; 
  (if (= (length llist) numlin)
      (progn
           (setq llist (cdr llist))
 ; Ŀ
 ;   Make the thing back into one meta-list.                               
 ; 
           (while (setq sub (car llist))
                  (setq llist (cdr llist))
                  (setq meta (append meta sub)))
 ; Ŀ
 ;   Pad it out to full length with spaces.                                
 ; 
           (while (< (length meta) (* numlin atsper))
                  (setq meta (append meta (list ""))))
 ; Ŀ
 ;   Reinstall it.                                                         
 ; 
           (while (and (/= (cdr (assoc 0 (setq entt (entget esav1)))) "SEQEND")
                       (setq vall (car meta)))
                  (setq meta (cdr meta))
                  (entmod (subst (cons 1 vall) (assoc 1 entt) entt))
                  (setq esav1 (entnext esav1)))
           (setq rsave esav)))
 (list rsave toprev))
 ; Ŀ
 ;   Fudge end.                                                            
 ; 

 ; Ŀ
 ;   Pman - update a value at a position in a block.                       
 ;   Arguments, Enam, a block entity name.                                 
 ;              Str, a string.                                             
 ;              Pos, the position of the attribute in the block.           
 ; 
 (DEFUN PMAN (enam str pos / entt)
  (setq num 0)
  (repeat pos (setq enam (entnext enam)))
  (setq entt (entget enam))
  (entmod (subst (cons 1 str) (assoc 1 entt) entt))
 (princ))
 ; Ŀ
 ;   Pman end.                                                             
 ; 

 ; Ŀ
 ;   Tach - deduce a new rev based on the existing one and a flag.         
 ;   Arguments: Toprev, the previous (existing) rev string.                
 ;              Gnurev, the new rev or flag.                               
 ;   Returns a new rev.                                                    
 ; 
 (DEFUN TACH (toprev gnurev)
 ; Ŀ
 ;   Cond: There was no previous rev and the flag is * or "" then the      
 ;   next rev should be A.                                                 
 ; 
  (cond ((and (or (null toprev) (member toprev '("." "..." "" " ")))
              (member gnurev '("*" "." "..." "" " ")))
         (setq toprev "A"))
 ; Ŀ
 ;   There was no previous rev and the flag is # then the next rev is 0.   
 ; 
        ((and (or (null toprev) (member toprev '("." "..." "" " ")))
              (= gnurev "#"))
         (setq toprev "0"))
 ; Ŀ
 ;   There was a previous rev and the flag is * then go to the next        
 ;   letter or number.                                                     
 ; 
        ((equal gnurev "*")
         (cond ((= (strcase toprev) (strcase toprev t))
                (setq toprev (itoa (1+ (read toprev)))))
               (T (setq toprev (chr (1+ (ascii toprev)))))))
 ; Ŀ
 ;   There was a previous rev and the flag is #: if the previous rev was   
 ;   a number then go to the next number, if a letter then go to 0.        
 ; 
        ((equal gnurev "#")
         (cond ((= (strcase toprev) (strcase toprev t))
                (setq toprev (itoa (1+ (read toprev)))))
               (T (setq toprev "0"))))
 ; Ŀ
 ;   Otherwise put in the value as is.                                     
 ; 
        (T (setq toprev gnurev)))
 toprev)
 ; Ŀ
 ;   Tach end.                                                             
 ; 

 ; Ŀ
 ;   Secup.                                                                
 ; 
 (DEFUN C:SECUP (/ vlasta ss esav enam dlist mtrev prerev mainrv rev projo)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   The list Vlasta contains the seven values for the new rev.            
 ;   The first value controls how the rev will be incremented:             
 ;   If it is * then A is followed by B, 0 becomes 1, if there is no       
 ;   existing rev the first one will be A.                                 
 ;   If it is # then the next rev is forced to a number: A becomes 0,      
 ;   1 becomes 2, a first rev will be inserted as 0.                       
 ;   Anything else is used as written, no matter what the last rev was.    
 ; 
  (setq vlasta '("#" "RE-ISSUED FOR CONSTRUCTION" "2013.OCT.01" "SW"
                 "NJG" "KdU" "DRW"))
 ; Ŀ
 ;   Find the main title block.                                            
 ; 
  (if (setq ss (ssget "X" (list (cons 2 "secure_a1_titleblock"))))
      (progn
           (setq esav (setq enam (ssname ss 0)))
 ; Ŀ
 ;   Call Fudge - If a title block rev area is full, empty the lowest      
 ;   rev line and shuffle the rest down.                                   
 ;   Arguments: Enam, the ename of the block insertion.                    
 ;              Numlin, the number of rev lines.                           
 ;              Atsper, the number of attributes per line.                 
 ;              Revpos, the 0 based position of the first rev attribute.   
 ;   Returns a list: the ename of the first attribute in the first empty   
 ;   rev line and the highest rev number.                                  
 ; 
           (setq rev (fudge enam 8 7 9))
           (setq rnam (car rev))
           (setq rev (cadr rev))
 ; Ŀ
 ;   Deduce the next rev from that and the flag in vlasta.                 
 ; 
           (setq rev (tach rev (car vlasta)))
 ; Ŀ
 ;   Make a new Vlasta ...                                                 
 ; 
           (setq vlasta (cons rev (cdr vlasta)))
 ; Ŀ
 ;   Install the new rev at the top.                                       
 ; 
           (blint rnam vlasta)
           (pman esav rev 6)
 ; Ŀ
 ;   Update the block, end.                                                
 ; 
           (entupd esav)))
 (princ))