 ; Ŀ
 ;   Strat - proto string art maker, also prints a list of lists of        
 ;   pins to wire to, in order, starting from the innermost patterns.      
 ;   Copyright 1993, 2014, 2015 by Rocket Software Ltd.                    
 ;   Nobody has ever printed a checkerboard on a Klein bottle.             
 ; 

 ; Ŀ
 ;   Wraps.                                                                
 ; 
 (DEFUN WRAPS (plist jump / len pos stop nums)
  (setq len (length plist))
  (setq pos 0)
  (setq nums (list pos))
  (command ".pline" (car plist))
  (while (null stop)
         (setq pos (+ pos jump))
         (cond ((> pos len)
                (setq pos (- pos len))
                (command (nth pos plist)))
               ((= pos len)
                (command "c") 
                (setq pos 0)  ; for point list for user
                (setq stop t))
               (t
                (command (nth pos plist))))
         (setq nums (cons pos nums)))
 (reverse nums))
 ; Ŀ
 ;   Wraps.                                                                
 ; 

 ; Ŀ
 ;   NxPri - if a number is not prime, find the next higher one which is.  
 ;   Argument: Num, a number.  Accepts an integer or a real, + or -.       
 ;   Returns a prime integer.                                              
 ; 
 (DEFUN NXPRI (num / incr)
  (setq num (fix num))
  (setq incr (if (minusp num) -1 1))
  (while (null (primp num))
         (setq num (+ num incr)))
 num)
 ; Ŀ
 ;   NxPri end.                                                            
 ; 

 ; Ŀ
 ;   Primp - see if a number is prime.                                     
 ;   Argument: Num, a number.  Accepts integers or reals,                  
 ;             but returns T for all reals with a decimal part.            
 ;   Note: considers 0, 1, and 2 to be prime.                              
 ; 
 (DEFUN PRIMP (num / divi xprim)
  (setq divi (abs (fix (/ num 2))))
  (while (and (null xprim)
              (> divi 1))
         (if (zerop (rem num divi))
             (setq xprim t))
         (setq divi (1- divi)))
 (not xprim))
 ; Ŀ
 ;   Primp end.                                                            
 ; 

 ; Ŀ
 ;   Strat.                                                                
 ; 
 (DEFUN C:STRAT (/ ceco circ entt pa rad divs ang incang aa plist jump num
                                                              pts malis sub)
  (setvar "cmdecho" 0)
  (command "undo" "be")
  (setq ceco (getvar "cecolor"))
  (if (and (setq circ (entsel "Select a circle: "))
           (setq circ (car circ))
           (equal (cdr (assoc 0 (setq entt (entget circ)))) "CIRCLE"))
      (progn
           (setq pa (cdr (assoc 10 entt)))
           (setq rad (cdr (assoc 40 entt)))
           (setq divs (getint "\nNumber of points: "))
 ; Ŀ
 ;   If the number isn't prime this won't be a valid string pattern, so    
 ;   call Nxpri to find one.                                               
 ; 
           (if (null (primp divs))
               (progn
                    (setq divs (nxpri divs))
                    (write-line (strcat "\nPrime Number Used: " (itoa divs)))))
 ; Ŀ
 ;   Get a list of points.                                                 
 ; 
              (setq ang 0)
              (setq incang (/ (* 2 pi) divs))
              (repeat divs
                     (setq aa (polar pa ang rad))
 ;                   (setq aa (list (car aa) (cadr aa))) ; for vertical separation
                     (setq ang (+ ang incang))
                     (setq plist (append plist (list aa))))
 ; Ŀ
 ;   Make the position lists.                                              
 ;   Start with the largest jump.                                          
 ; 
              (setq jump (fix (/ (length plist) 2)))
 ; Ŀ
 ;   For each jump length down to 1 - a ring around the outside - call     
 ;   wraps to draw a polyline and return a list:                           
 ;   ((position  point_list) ...)                                          
 ; 
              (setq num 1)
 ;            (setq elev 1)                          ; vertical separation
              (while (> jump 0)
                     (setvar "cecolor" (itoa num))
 ;                   (setq elev (+ 10 elev))         ; vertical separation
 ;                   (setvar "elevation" elev)       ; vertical separation
                     (setq num (1+ num))
 ;                   (if (> num 7) (setq num 1))     ; only cycle through first seven colours
                     (setq pts (wraps plist jump))
                     (setq malis (cons pts malis))
                     (setq jump (1- jump)))))
  (setvar "cecolor" ceco)
  (command "undo" "end")
  (setq malis (reverse malis))
  (while (setq sub (car malis))
         (setq malis (cdr malis))
         (print sub))
 (princ))