 ; Ŀ
 ;   TbDb - maintain a database of tb data.                                
 ;   Copyright 2005, 2014 by Rocket Software Ltd.                          
 ;                                                                         
 ; 

 ; Ŀ
 ;   Bulla - write a list of lists of strings to a (csv) file.             
 ;   Arguments: Lista, a list.                                             
 ;              Filnam, a filename.                                        
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN BULLA (lista filnam / fn sub str nxtstr)
  (if (setq fn (open filnam "w"))
      (progn
           (while (setq sub (car lista))
                  (setq lista (cdr lista))
                  (setq str "")
                  (while (setq nxtstr (car sub))
                         (setq sub (cdr sub))
                         (setq str (strcat str "," nxtstr)))
                  (write-line (substr str 2) fn))
           (close fn))
       (write-line (strcat "\nCan't open " filnam " to write.")))
 (princ))
 ; Ŀ
 ;   Bulla end.                                                            
 ; 

 ; Ŀ
 ;   Comba - add a data list to a list of data lists, if there is an       
 ;   existing sublist in the list of lists with the same first element     
 ;   then replace it with the new one.                                     
 ;   Arguments: Gnulst, the new data list.                                 
 ;              Csvlst, the list of data lists from the cdf/csv file.      
 ;   Calls nothing, returns a combined data list.                          
 ; 
 (DEFUN COMBA (gnulst csvlst / num sub poso magu subnum gnucsv cara)
 ; Ŀ
 ;   If there are two sublists in gnulst then the second one takes out     
 ;   the first one as it goes past.  Have to do this in two steps: kill    
 ;   the previous sublists and then add the new ones.                      
 ;   Don't simplify this too far - the new sublists may not all have the   
 ;   same first element (i.e. they may not use the file name.)             
 ; 
  (setq num 0)
  (while (setq sub (nth num gnulst))
         (setq num (1+ num))
         (setq sub (csplit sub))
         (setq poso (car sub))
         (setq magu (cons sub magu))
 ; Ŀ
 ;   If csvlst does not have a sublist with the same first element as sub  
 ;   then append sub to csvlst.                                            
 ;   This will kill off duplicate lists if they've sneaked in.             
 ;   If there are duplicate ones in a file they'll be added back in.       
 ;   This doesn't allow for different files with the same drawing name,    
 ;   but ... lunacy is hard to allow for.                                  
 ; 
         (setq subnum 0)
         (setq gnucsv ())
         (while (and csvlst
                     (setq cara (nth subnum csvlst)))
                (setq subnum (1+ subnum))
                (if (not (equal poso (car cara)))
                    (setq gnucsv (cons cara gnucsv))))
         (setq csvlst (reverse gnucsv)))
 ; Ŀ
 ;   Add the new sublist list to malist.                                   
 ; 
  (setq csvlst (append csvlst magu))
 csvlst)
 ; Ŀ
 ;   Comba end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Csplit - divide a text string at commas, make into a list  
 ;   of substrings.                                                        
 ; 
 (DEFUN CSPLIT (linn / len pos name1 strlst)
  (while (/= (strlen linn) 0)
         (while (and (= (substr linn 1 1) " ")
                     (/= (strlen linn) 0))
                (setq linn (substr linn 2)))
         (while (= (substr linn (setq len (strlen linn))) " ")
                (setq linn (substr linn 1 (1- len))))
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) ",")
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
         (setq strlst (append strlst (list name1))))
  (if (null strlst) (setq strlst (list "")))
  strlst)
 ; Ŀ
 ;   Csplit end.                                                           
 ; 

 ; Ŀ
 ;   CsvOut - suck a cdf/scv file into a list.                             
 ;   Arguments: filnam, a filename.                                        
 ;   Calls Csplit.                                                         
 ;   Returns a list of lists of strings.                                   
 ; 
 (DEFUN CSVOUT (filnam / fn linn malist)
  (if (setq fn (open filnam "r"))
      (progn
           (while (setq linn (read-line fn))
                  (if (/= linn "")
                      (setq malist (append malist (list (csplit linn))))))
           (close fn)))
 malist)
 ; Ŀ
 ;   Csvout end.                                                           
 ; 

 ; Ŀ
 ;   TbDb.                                                                 
 ; 
 (DEFUN C:TBDB (/ fnam *error* lista fnamp exlist)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Make a data filename.                                                 
 ; 
  (setq fnam (strcat (getvar "dwgprefix") "Dwglist.csv"))
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (if shk (print shk))
   (write-line "\n*** TBDB Error ***")
  (princ))
 ; Ŀ
 ;   If the data file already exists:                                      
 ; 
  (if (setq fnamp (findfile fnam))
      (progn
 ; Ŀ
 ;   Trout is required.  Make sure it's loaded.                            
 ;   If so get the data for the current drawing.                           
 ;   If not then place an error message in the database file.              
 ;   Trout returns a list of csv strings, there may be >1 tb.              
 ; 
           (if (and (null trout)
                    (null (load "trout" ())))
               (setq lista (strcat (getvar "dwgname")
                                   ",Trout.lsp can't be loaded."))
               (setq lista (trout)))
 ; Ŀ
 ;   If the file exists, read it into its own list.                        
 ; 
           (setq exlist (csvout fnamp))
 ; Ŀ
 ;   Combine the file list and the drawing list.                           
 ; 
           (setq lista (comba lista exlist))
 ; Ŀ
 ;   Write the list to the csv file.                                       
 ; 
           (bulla lista fnamp)))
 ; Ŀ
 ;   If the data file doesn't exist then make one.                         
 ; 
 ; currently ... don't
 ;      (bulla (list lista) fnam)
 (princ))