 ; Ŀ
 ;   Tga - draw cable tags and a leader line.                              
 ;   Tg - draw cable tags and a slightly different leader line.            
 ;   Tca - draw cable tags and an arced tip leader line.                   
 ;   Copyright 1997, 2000, 2001, 2006, 2008 - 2012 by Rocket Software Ltd. 
 ;   Revised to allow modification of the block without modification       
 ;   of the routine.                                                       
 ;   Revised again to squash the text to fit.                              
 ;   Has to be able to find Puss.lsp and Misps.lsp.                        
 ;   Coffee - actually a drug which makes boredom tolerable.               
 ; 

 ; Ŀ
 ;   Tim - extract the Tdindwg sysvar (Time in Days in Drawing) in         
 ;   hours:minutes format.                                                 
 ; 
 (DEFUN TIM (/ td hours mins)
  (setq td (getvar "tdindwg"))
  (setq hours (fix (setq td (* td 24))))
  (setq mins (itoa (fix (* 60 (- td hours)))))
  (if (= (strlen mins) 1) (setq mins (strcat "0" mins)))
 (strcat (if (> hours 0) (strcat (itoa hours) "hr ") "") mins "min."))
 ; Ŀ
 ;   Tim end.                                                              
 ; 

 ; Ŀ
 ;   Subroutine Tga.                                                       
 ; 
 (DEFUN TGA (styl / tagblk *error* precol dimscl attdia orth clay plist pta
                rhdis lhdis updis dndis blhite blwid p1 p1a p2 dist1 p3 pbase
                            dir1 pa pb pc osmo pd pe ptlist vdist tagtxt entt)
  (setvar "cmdecho" 0)
  (setq osmo (getvar "osmode"))
  (setvar "osmode" 0)
 ; Ŀ
 ;   Decide which tag block to use.                                        
 ; 
  (setq tagblk "cabletag")
 ; Ŀ
 ;   Make an error handler.                                                
 ; 
 (defun *error* (shk)
  (setvar "osmode" osmo)
  (if clay (setvar "clayer" clay))
  (if precol (setvar "cecolor" precol))
  (if attdia (setvar "attdia" attdia))
  (if orth (setvar "orthomode" orth))
  (if shk (print shk))
 (princ))
 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))
 ; Ŀ
 ;   Get the scale, depending on which space we are in and other things.   
 ; 
  (if misps
      (setq dimscl (misps))
      (setq dimscl (getvar "dimscale")))
 ; Ŀ
 ;   Save and change a few sysvars.                                        
 ; 
  (setq precol (getvar "cecolor"))
  (setq attdia (getvar "attdia"))
  (setq orth (getvar "orthomode"))
  (setvar "attdia" 0)
  (setvar "attreq" 1)
  (setq clay (getvar "clayer"))
 ; Ŀ
 ;   Make the Misc layer current.                                          
 ; 
  (if malaya (malaya "misc"))
 ; Ŀ
 ;   If the tag block isn't available then insert it so the definition     
 ;   can be looked at.                                                     
 ; 
  (if (null (tblsearch "block" tagblk))
      (progn
           (command ".insert" tagblk)
           (command)))
 ; Ŀ
 ;   Load the block corner point finder from Puss.lsp.                     
 ; 
  (if (not bent) (load "puss"))
 ; Ŀ
 ;   Get a list of the insertion point and block extents.                  
 ;   Bent returns a list: (insertion_point  x-max  x-min  y-max  y-min).   
 ; 
  (setq plist (bent tagblk))
  (setq pta (car plist))
  (setq rhdis (+ (car pta) (cadr plist)))         ; right hand (+x) distance
  (setq lhdis (abs (+ (car pta) (caddr plist))))  ; left hand (-x) distance
  (setq updis (+ (cadr pta) (cadddr plist)))      ; top (+y) distance
  (setq dndis (abs (+ (cadr pta) (nth 4 plist)))) ; bottom (-y) distance
  (setq blhite (+ updis dndis))                   ; overall block height
  (setq blwid (+ rhdis lhdis))                    ; overall block width
 ; Ŀ
 ;   Get some lines to tag.                                                
 ; 
  (setvar "osmode" 512)
  (if (setq p1 (getpoint "\nLine to tie leader to or <Return> for no leader:"))
      (progn
          (command "color" "red")
           (if (setq p1a (osnap p1 "nearest"))
               (setq p1 p1a))
           (setvar "osmode" 128)
           (setq p2 (getpoint p1 "\nNext line or <Return> if only one: "))
           (setvar "osmode" 0)
           (if (null p2) (setq p2 p1))
           (setq dist1 (distance p1 p2))
           (setq p3 (getpoint p2
                           "\nPick leader line points, <Return> when done: "))
           (if (> (distance p3 p1) (distance p3 p2))
               (setq pbase p1)
               (setq pbase p2))
           (setq dir1 (angle pbase p3))
 ; Ŀ
 ;   Make the initial loop.                                                
 ; 
           (setq pa (polar pbase (+ dir1 pi) (* dimscl 2.5)))
           (setq pb (polar pa (+ dir1 (/ pi 2)) (* dimscl 1.25)))
           (setq pc (polar pb dir1 (+ dist1 (* dimscl 5))))
           (cond ((= styl "a")
                  (setq pd (polar pc (+ dir1 (* pi 1.5)) (* dimscl 2.5))))
                 ((or (= styl "b") (= styl "c"))
                  (setq pd (polar pc (+ dir1 (* pi 1.5)) (* dimscl 1.25)))))
           (setq pe (polar pd dir1 (* dimscl 1.25)))
           (cond ((= styl "a")
                  (setq ptlist (list p3 pa pb pc pd pe))
                  (command ".pline" pe pd pc pb pa p3))
                 ((= styl "b")
                  (setq ptlist (list p3 pa pb pc pd))
                  (command ".pline" pd pc pb pa p3))
                 ((= styl "c")
;                  (grdraw '(0 0) pd 1)
;                  (grdraw '(0 0) p1 2)
;                  (grdraw '(0 0) pa 3)
                  (setq ptlist (list p3 pa p1 pd))
                  (command ".pline" pd "a" "ce" p1 pa "l" p3)))
 ; Ŀ
 ;   Make the rest of the polyline - allow undo.                           
 ; 
           (initget 128)
           (while (setq pa (getpoint (car ptlist)))
                  (initget 128)
                  (if (listp pa)
                      (progn
                           (setq ptlist (cons pa ptlist))
                           (command pa))
                      (progn
                           (if (= pa "u")
                               (if (> (length ptlist) 1)
                                   (progn
                                        (setq ptlist (cdr ptlist))
                                        (command pa))
                                   (prompt "Nothing left, Bozo."))
                               (command pa)))))
           (command "")
           (setq p3 (car ptlist)))
      (progn
           (setvar "osmode" 0)
           (setq p3 (getpoint "\nTag insertion point: "))))
 ; Ŀ
 ;   Deduce the tag insertion point and flow direction.                    
 ; 
  (setvar "cecolor" precol)
  (setvar "orthomode" 0)
  (setq pa (getpoint p3 "Tag array direction: "))
  (setvar "orthomode" orth)
  (if (< (car pa) (car p3))
      (setq p3 (polar p3 pi (* dimscl rhdis)))
      (setq p3 (polar p3 0 (* dimscl lhdis))))
  (if (> (cadr pa) (cadr p3))
      (progn
           (setq vdist (* dimscl blhite))
           (setq p3 (polar p3 (* pi 0.5) (* dimscl dndis))))
      (progn
           (setq p3 (polar p3 (* pi 1.5) (* dimscl updis)))
           (setq vdist (- (* dimscl blhite)))))
 ; Ŀ
 ;   Draw the tags.                                                        
 ; 
  (setq tagtxt "a")
  (while (/= "" tagtxt)
         (setq tagtxt " ")
         (command ".insert" tagblk p3 dimscl "" 0 tagtxt)
         (setq p3 (polar p3 (/ pi 2) vdist))
         (if (= "" (setq tagtxt (getstring t "\nCable Name: ")))
             (progn
                  (entdel (entlast))
                  (redraw (entlast)))
             (progn
                  (setq enam (entnext (entlast)))
                  (entmod (subst (cons 1 tagtxt)
                                 (assoc 1 (setq entt (entget enam))) entt))
                  (veni enam (- blwid (* dimscl 4)) 1)
                  (entupd (entlast)))))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (write-line (strcat "Total time in drawing: " (tim)))
  (*error* ())
 (princ))
 ; Ŀ
 ;   Subroutine Tga end.                                                   
 ; 

 ; Ŀ
 ;   Subroutine Veni - squeeze or stretch text as required.                
 ;   Arguments: Enam, a text ename.                                        
 ;              Width, a desired width.                                    
 ;              Txtscl, a desired width scale factor.                      
 ;   Calls Wits.                                                           
 ;   Returns nothing.                                                      
 ; 
 (DEFUN Veni (enam width txtscl / entt realwd widscl prev41 scalfc)
 ; Ŀ
 ;   Get the text entity data.                                             
 ; 
  (setq entt (entget enam))
 ; Ŀ
 ;   Call Wits to find the actual string width.                            
 ; 
  (setq realwd (wits entt))
 ; Ŀ
 ;   Find the attribute width scale factor.                                
 ; 
  (setq widscl (cdr (setq prev41 (assoc 41 entt))))
 ; Ŀ
 ;   Compare the actual and desired widths.                                
 ; 
  (if (and (> realwd width) (not (equal realwd width 0.1)))
 ; Ŀ
 ;   If the actual width is greater than the allowed width in the          
 ;   sublist, then adjust the width scale factor to make it fit.           
 ;   Wait: must also check to see if the width scale is greater than the   
 ;   ideal - if the attribute is too wide and the width scale is too       
 ;   large, then shrinking the attribute to fit may result in it just      
 ;   filling the space but still being too wide.                           
 ; 
      (progn
 ; Ŀ
 ;   Compare the actual width scale to the ideal width scale.              
 ; 
           (if (<= widscl txtscl)
 ; Ŀ
 ;   If the actual is less than or equal to the ideal, then make it fit.   
 ; 
               (progn
                    (setq scalfc (/ width realwd))
                    (setq widscl (* widscl scalfc))
                    (entmod (subst (cons 41 widscl) prev41 entt)))
 ; Ŀ
 ;   If the actual width scale is greater than the ideal, see if the       
 ;   attribute will be too wide if if set to the ideal.                    
 ;   If so then squash to fit, if not then set to the ideal.               
 ; 
               (progn
                    (if (> (* realwd (/ txtscl widscl)) width)
 ; Ŀ
 ;   Squash to fit.                                                        
 ; 
                        (progn
                             (setq scalfc (/ width realwd))
                             (setq widscl (* widscl scalfc))
                             (entmod (subst (cons 41 widscl) prev41 entt)))
 ; Ŀ
 ;   Set to the ideal width scale factor.                                  
 ; 
                        (entmod (subst (cons 41 txtscl) prev41 entt))))))
 ; Ŀ
 ;   Else the actual width is narrower than or equal to the available      
 ;   space.                                                                
 ; 
      (progn
 ; Ŀ
 ;   See if the attribute is narrower than it should be - if setting the   
 ;   width scale factor to the desired value would leave the attribute     
 ;   wider than the allowable space, then increase it to fill the space.   
 ; 
           (if (> (* realwd (/ txtscl widscl)) width)
               (progn
                    (setq scalfc (/ width realwd))
                    (setq widscl (* widscl scalfc))
                    (entmod (subst (cons 41 widscl) prev41 entt)))
 ; Ŀ
 ;   Otherwise set it to the ideal width scale value.                      
 ; 
               (entmod (subst (cons 41 txtscl) prev41 entt)))))
 (princ))
 ; Ŀ
 ;   Veni end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Wits - find the width of a text entity.                    
 ;   Takes one argument: the text entity data list.  Returns a width.      
 ; 
 (DEFUN WITS (entt / tblist cc dd)
  (setq tblist (textbox entt))
  (setq cc (car tblist))                    ; ll offset from 10 of text
  (setq dd (cadr tblist))                   ; ur offset from 10 of text
  (- (car dd) (car cc)))
 ; Ŀ
 ;   Wits end.                                                             
 ; 

 ; Ŀ
 ;   Tca.                                                                  
 ; 
 (DEFUN C:TCA ()
  (tga "c")
 (princ))

 ; Ŀ
 ;   Tg.                                                                   
 ; 
 (DEFUN C:TG ()
  (tga "a")
 (princ))

 ; Ŀ
 ;   Tga.                                                                  
 ; 
 (DEFUN C:TGA ()
  (tga "b")
 (princ))
