 ; Ŀ
 ;   Trout - extract data from a variety of title blocks.                  
 ;   Copyright 1997, 2003 - 2016 by Rocket Software Ltd.                   
 ;   Required data: File name, All three title lines, Rev.  Maybe.         
 ;   Much later: this differs depending on the client and tb.              
 ;   Writes an output file: c:/Titledat.csv                                
 ;                                                                         
 ;   Multiple choice question.                                             
 ;   Fish didn't discover fire because:                                    
 ;    A. They can't hold a chunk of flint securely with their fins.        
 ;    B. The wood floats away before they can light it.                    
 ;    C. Cold blooded creatures don't like hot food.                       
 ;    D. Marshmallows melt in water, so why have a fire?                   
 ; 

 ; Ŀ
 ;   Dalbe - get data from an ss of blocks.                                
 ;   The argument Rqlst is a list which contains, in order:                
 ;        An ss of block enames.                                           
 ;        A list of attribute names to extract.  Any attribute names       
 ;          which are not found in the block insertion are used as values, 
 ;          which allows one to insert spaces and commas as required.      
 ;          (Or in fact any string which doesn't match an attribute tag.)  
 ;        The number of blocks to find, -1 = all.                          
 ;   Modified 2010.0.21 - can now use embedded functions in rqlist.        
 ;   Calls Extac and nocomma.                                              
 ;   Returns a list of comma delimited strings.                            
 ; 
 (DEFUN DALBE (rqlst / ss attlst numble count enam atnum attnam strval curlis
                                                                   typ malist)
  (setq ss (car rqlst))
  (setq attlst (cadr rqlst))
  (setq numble (nth 2 rqlst))
  (setq count 0)
  (while (and (setq enam (ssname ss count))
              (or (minusp numble) (< count numble)))
         (setq curlis ())
         (setq count (1+ count))
         (setq atnum 0)
         (while (setq attnam (nth atnum attlst))
                (setq atnum (1+ atnum))
                (setq typ (type attnam))
                (cond ((= typ 'STR)
 ; Ŀ
 ;   If no attribute is found then use the string value - this lets the    
 ;   user put in commas and spaces.                                        
 ;   This also means that a missing attribute will be replaced in the      
 ;   output file with the attribute tag.                                   
 ; 
                       (if (setq strval (extac enam attnam))
                           (setq strval (nocomma strval))
                           (setq strval attnam)))
                      (t ; (= typ 'LIST)
                       (setq strval (eval attnam))   ; strval may be a list
                       (if (= (type strval) 'LIST)
                           (setq strval (listi strval ",")))))
                (if (null strval) (setq strval "N/A"))
                (setq curlis (append curlis (list strval))))
         (setq curlis (listi curlis ""))
         (setq malist (cons curlis malist)))
 malist)
 ; Ŀ
 ;   Dalbe end.                                                            
 ; 

 ; Ŀ
 ;   Epfiln - get the filename without description from an Enerplus dwg.   
 ;   Takes no arguments.                                                   
 ;   Returns a string.                                                     
 ; 
 (DEFUN EPFILN () (car (pmatch " - " (nopath))))
 ; Ŀ
 ;   Epfiln end.                                                           
 ; 

 ; Ŀ
 ;   Eptop - get the latest rev line from an enerplus TitleA1A tb.         
 ;   Takes no arguments.                                                   
 ;   Returns a list of values.                                             
 ; 
 (DEFUN EPTOP (/ ss enam tagg entt vall stop mtrev prerev val malist)
  (if (setq ss (ssget "X" (list (cons 2 "titlea1a"))))
      (progn
           (setq enam (ssname ss 0))
 ; Ŀ
 ;   Step through the title block.                                         
 ; 
           (while (not stop)
                  (setq tagg (cdr (assoc 2 (setq entt (entget (setq enam
                                                           (entnext enam)))))))
                  (setq vall (cdr (assoc 1 entt)))
 ; Ŀ
 ;   Find the first empty rev line, the saved one before that is the last  
 ;   filled one.                                                           
 ; 
                  (if (and (= (strlen tagg) 4)
                           (= (substr tagg 1 3) "REV"))
                      (if (member vall '("" " " "." "-" "..."))
                          (progn
                               (setq stop t)
                               (setq mtrev enam))
                          (setq prerev enam))))
 ; Ŀ
 ;   If there was a filled line then extract it.                           
 ; 
           (if prerev
               (repeat 5
                       (setq val (cdr (assoc 1 (entget prerev))))
                       (setq prerev (entnext prerev))
                       (setq malist (cons val malist))))))
 (reverse malist))
 ; Ŀ
 ;   Eptop end.                                                            
 ; 

 ; Ŀ
 ;   Extac - extract an attribute value from a block insertion.            
 ;   Arguments: Enam, the block insertion entity name.                     
 ;              Atta, the attribute tag.                                   
 ;   Assumes that the block contains attributes, and that only one of      
 ;   them has that tag (or that we only want to extract the first one.)    
 ;   Returns nil if no value was found.                                    
 ;   Modified 2011.06.29 - attribute tags can now be mixed case.           
 ;   Modified 2013.01.11 - doesn't crash on a block with no attributes.    
 ; 
 (DEFUN EXTAC (enam atta / vall stop entt tagg)
  (setq blnam (cdr (assoc 2 (entget enam))))
  (setq bldat (tblsearch "block" blnam))
  (if (/= "ENDBLK" (cdr (assoc 0 (entget (cdr (assoc -2 bldat))))))
      (progn
           (setq atta (strcase atta))
           (while (and (null stop)
                       (/= "SEQEND" (cdr (assoc 0 (setq entt (entget
                                              (setq enam (entnext enam))))))))
                  (setq tagg (strcase (cdr (assoc 2 entt))))
                  (if (= tagg atta)
                      (progn
                           (setq stop t)
 ; Ŀ
 ;   Texu can get the string out of a multi-line attribute or              
 ;   a normal one.                                                         
 ; 
                           (setq vall (texu entt nil))
 ; Ŀ
 ;   Multi-line attributes may contain line feed codes: "\\p".             
 ;   Nolf removes these.  Later: also "\\w0.008;".                         
 ;   And spak kills extraneous spaces.                                     
 ; 
                           (setq vall (nolf vall))
                           (setq vall (car (spak vall))))))))
 vall)
 ; Ŀ
 ;   Subroutine Extac end.                                                 
 ; 

 ; Ŀ
 ;   Frat - get a list of all attributes from a block definition.          
 ;   Arguments: Blnam, a block name.                                       
 ;   Returns a list.                                                       
 ; 
 (DEFUN FRAT (blnam / blok enam entt str atlis)
  (setq blok (tblsearch "block" blnam))           ; head entity from table
  (setq enam (cdr (assoc -2 blok)))               ; first ename after head
  (while (and enam (setq entt (entget enam)))
         (if (= (cdr (assoc 0 entt)) "ATTDEF")
             (progn
                  (setq str (cdr (assoc 2 entt)))
                  (setq atlis (cons str atlis))))
         (setq enam (entnext enam)))
 (reverse atlis))
 ; Ŀ
 ;   Frat end.                                                             
 ; 

 ; Ŀ
 ;   Grizmo - get the module from a DCSParameters block.                   
 ;   Takes no arguments.                                                   
 ;   Returns a string, which may be "".                                    
 ; 
 (DEFUN GRIZMO (/ ss enam tagg entt vall stop modu)
  (if (setq ss (ssget "X" (list (cons 2 "dcsparameters"))))
      (progn
           (setq enam (ssname ss 0))
 ; Ŀ
 ;   Step through the title block.                                         
 ; 
           (while (not stop)
                  (setq tagg (cdr (assoc 2 (setq entt (entget (setq enam
                                                           (entnext enam)))))))
                  (setq vall (cdr (assoc 1 entt)))
 ; Ŀ
 ;   Find the Module attribute.                                            
 ; 
                  (if (= tagg "MODULE")
                      (progn
                           (setq stop t)
                           (setq modu vall))))))
 (if modu modu ""))
 ; Ŀ
 ;   Grizmo end.                                                           
 ; 

 ; Ŀ
 ;   Husk - get the latest rev from an insane Husky-D tb.                  
 ; 
 (DEFUN HUSK (/ ss enam entt tagg vala sub malist num atnam suba pos)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Find the main title block.                                            
 ; 
  (if (setq ss (ssget "X" (list (cons 2 "Husky-d"))))
      (progn
           (setq enam (ssname ss 0))
 ; Ŀ
 ;   Extract the rev number and description attributes into a list.        
 ;   They aren't in any known order so this has to be done by name.        
 ; 
           (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget (setq enam
                                                          (entnext enam)))))))
                  (setq tagg (cdr (assoc 2 entt)))
                  (if (or (= (substr tagg 1 4) "REV_")
                          (member tagg '("0" "1" "2" "3" "4")))
                      (progn
                           (setq vala (cdr (assoc 1 entt)))
                           (setq sub (cons tagg vala))
                           (setq malist (cons sub malist)))))
 ; Ŀ
 ;   Now find the first rev_n_description that isn't empty.                
 ; 
           (setq num 4)
           (while (>= num 0)
                  (setq atnam (strcat "REV_" (itoa num) "_DESCRIPTION"))
                  (if (and (setq suba (assoc atnam malist))
                           (not (member (cdr suba) '("" " " "  " "..." "-"))))
                      (progn
                           (setq pos num)
                           (setq num -12)))
                  (setq num (1- num)))
           (if (null pos) (setq pos 0))
 ; Ŀ
 ;   Get the matching rev attribute values.                                
 ; 
           (setq vala (cdr (assoc (itoa pos) malist)))))
 (if vala vala "-"))
 ; Ŀ
 ;   Husk end.                                                             
 ; 

 ; Ŀ
 ;   Inf - get the attribute at a given position from a block.             
 ;   Arguments:  Enam, a block entity name.                                
 ;               Poslis, a list of zero based attribute positions.         
 ;   Returns a string value.                                               
 ; 
 (DEFUN INF (enam poslis / num vala entt valist)
 ; Ŀ
 ;   Step through the block.                                               
 ; 
  (setq num 0)
  (while (/= "SEQEND" (cdr (assoc 0 (setq entt (entget (setq enam
                                                       (entnext enam)))))))
         (if (member num poslis)
             (progn
                  (setq vala (cdr (assoc 1 entt)))
                  (setq vala (nocomma vala))
                  (setq valist (cons vala valist))))
         (setq num (1+ num)))
 (reverse valist))
 ; Ŀ
 ;   Inf end.                                                              
 ; 

 ; Ŀ
 ;   Listi - make a list of strings into one string.                       
 ;   Arguments: Alist, a list of strings.                                  
 ;              Sepstr, the separator string.                              
 ;   Returns a string.                                                     
 ; 
 (DEFUN LISTI (alist sepstr / thestr len)
  (setq thestr "")
 ; Ŀ
 ;   You don't ever really have to have mapcar, but it is nice...          
 ; 
  (mapcar '(lambda (astr)
                   (if (= (type astr) 'INT) (setq astr (itoa astr)))
                   (setq thestr (strcat thestr sepstr astr)))
                    alist)
 ; Ŀ
 ;   Remove the extraneous copy of sepstr from the string end.             
 ; 
  (if (> (strlen thestr) (setq len (strlen sepstr)))
      (setq thestr (substr thestr (1+ len))))
 thestr)
 ; Ŀ
 ;   Listi end.                                                            
 ; 

 ; Ŀ
 ;   Nocomma - returns a string minus the commas.                          
 ; 
 (DEFUN NOCOMMA (aa / pos len bb)
  (setq pos 1)
  (setq len (strlen aa))
  (while (>= len pos)
         (setq bb (substr aa pos 1))
         (if (= bb ",")
             (setq aa (strcat (substr aa 1 (1- pos)) ";"
                              (substr aa (1+ pos)))))
         (setq pos (1+ pos)))
  aa)
 ; Ŀ
 ;   Nocomma end.                                                          
 ; 

 ; Ŀ
 ;   Nolf - returns a string minus the "\p", "\n", and "\w"s.              
 ;   Modified - also kills leading %%U.                                    
 ; 
 (DEFUN NOLF (aa / pos len bb aa0 aa1)
  (if (and (> (strlen aa) 3)
           (= (strcase (substr aa 1 3)) "%%U"))
      (setq aa (substr aa 4)))
  (setq pos 1)
  (setq len (strlen aa))
  (while (>= len (1+ pos))
         (setq bb (strcase (substr aa pos 2)))
         (cond ((= bb "\\P")
                (setq aa (strcat (substr aa 1 (1- pos))
                                 " " (substr aa (+ 2 pos))))
                (setq len (strlen aa))
                (setq pos 0))
               ((= bb "\\W")
                (setq aa0 (substr aa 1 (1- pos)))
                (setq aa1 (substr aa (+ 2 pos)))
                (while (and (/= aa1 "")
                            (/= (substr aa1 1 1) ";"))
                       (setq aa1 (substr aa1 2)))
                (if (= (substr aa1 1 1) ";")
                       (setq aa1 (substr aa1 2)))
                (setq aa (strcat aa0 aa1))
                (setq len (strlen aa))
                (setq pos 0))
               ((= (ascii (substr aa pos 1)) 10)  ; 10 = \n = newline
                (setq aa (strcat (substr aa 1 (1- pos))
                                 " " (substr aa (1+ pos))))
                (setq len (strlen aa))
                (setq pos 0)))
         (setq pos (1+ pos)))
 aa)
 ; Ŀ
 ;   Nolf end.                                                             
 ; 

 ; Ŀ
 ;   Nopath - returns the drawing name without the path or the extension.  
 ; 
 (DEFUN NOPATH (/ tt pos ff)
 ; Ŀ
 ;   Save this next bit in case we want the path too.                      
 ; 
 ; (setq tt (strcat (getvar "dwgprefix") (getvar "dwgname")))
 ; Ŀ
 ;   Get drawing name with path and set pointer Pos to end of string.      
 ; 
  (setq pos (strlen (setq tt (getvar "dwgname"))))  ; start at end of string
 ; Ŀ
 ;   Remove the path.                                                      
 ; 
  (while (< 0 pos)
          (if (or (= (substr tt pos 1) (chr 92))    ; if char = \
                  (= (substr tt pos 1) ":"))        ; if char = :
             (progn
                   (setq tt (substr tt (1+ pos)))   ; then set tt to all after
                   (setq pos 1)))                   ;  and set pos to first
         (setq pos (1- pos)))                       ; set pos to previous
 ; Ŀ
 ;   Remove the extension.                                                 
 ; 
  (if (= (substr (strcase tt t) (- (setq len (strlen tt)) 3)) ".dwg")
          (setq tt (substr tt 1 (- len 4))))
  tt)
 ; Ŀ
 ;   Nopath end.                                                           
 ; 

 ; Ŀ
 ;   Subroutine Pmatch - split a string around a match string.             
 ;   Arguments: Pat, the string to find.                                   
 ;              Str, the overall string.                                   
 ;   Returns a list: (before  match_string  after) or nil = no match.      
 ;   Calls nothing.                                                        
 ;   Assumes that the match string occurs no more than once.               
 ;   Doesn't kill spaces.                                                  
 ;   Is case sensitive.                                                    
 ; 
 (DEFUN PMATCH (pat str / patlen pos sub before after)
  (setq patlen (strlen pat))
  (setq pos 1)
  (while (= patlen (strlen (setq sub (substr str pos patlen))))
         (if (= sub pat)
             (progn
                  (setq before (substr str 1 (1- pos)))
                  (setq after (substr str (+ pos patlen)))
                  (setq pos (strlen str))))
         (setq pos (1+ pos)))
 (if before (list before pat after) nil))
 ; Ŀ
 ;   Pmatch end.                                                           
 ; 

 ; Ŀ
 ;   Spak - Remove leading and trailing and multiple internal spaces.      
 ;   Takes one argument, a string.                                         
 ;   Returns a list: the modified-p string and the no. of spaces removed.  
 ; 
 (defun spak (bstr / monk len pos bsub gnustr)
  (setq monk 0)
  (while (and (/= bstr "") (= (substr bstr 1 1) " "))
         (setq monk (1+ monk))
         (setq bstr (substr bstr 2)))
  (while (and (/= bstr "") (= (substr bstr (setq len (strlen bstr))) " "))
         (setq monk (1+ monk))
         (setq bstr (substr bstr 1 (1- len))))
  (setq pos 1)
  (setq len (strlen bstr))
  (while (and (>= len pos)
              (setq bsub (substr bstr pos 1)))
         (cond ((null gnustr)
                (setq gnustr bsub))
               ((or (/= (substr gnustr (strlen gnustr)) " ")
                    (/= bsub " "))
                (setq gnustr (strcat gnustr bsub)))
               (T
                (setq monk (1+ monk))))
         (setq pos (1+ pos)))
 (list (if gnustr gnustr "") monk))
 ; Ŀ
 ;   Spak end.                                                             
 ; 

 ; Ŀ
 ;   Texu - perhaps replace a string in a textlike entity.                 
 ;   Arguments: Entt, an entity data list.                                 
 ;              Stra, a new string or nil of no change is required.        
 ;   Calls nothing.                                                        
 ;   Returns the original string.                                          
 ;   Builds an antiaircraft cannon out of chocolate chip cookies.          
 ; 
 (DEFUN TEXU (entt stra / num sub ook strb gnu)
  (if (member '(101 . "Embedded Object") entt)
      (progn
           (setq num 0)
           (while (setq sub (nth num entt))
                  (setq num (1+ num))
                  (cond ((equal sub '(101 . "Embedded Object"))
                         (setq ook t))
                        ((and stra ook (= (car sub) 1))
                         (setq strb (cdr sub))
                         (setq sub (cons 1 stra)))
                        ((and ook (= (car sub) 1))
                         (setq strb (cdr sub))))
                  (setq gnu (append gnu (list sub))))
           (if stra (entmod gnu)))
      (progn
           (setq strb (cdr (assoc 1 entt)))
           (if stra (entmod (subst (cons 1 stra) (assoc 1 entt) entt)))))
 strb)
 ; Ŀ
 ;   Texu end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Trout - get a list of title block data strings.            
 ;   Takes no arguments.                                                   
 ;   Uses an internal list of title blocks and desired data.               
 ;   Calls Nopath, Dalbe/Extac & Nocomma.                                  
 ;   Returns a list of strings, one for each tb in the drawing.            
 ; 
 (DEFUN TROUT (/ fileno tblisi blnam num blist ss datlst stop)
 ; Ŀ
 ;   Get the current drawing file name.                                    
 ; 
  (setq fileno (nopath))
 ; Ŀ
 ;   Make the list of blocks and data to extract.  Note that anything      
 ;   which isn't an an attribute tag (i.e. one found in the block) is      
 ;   added directly to the data string returned by Dalbe.  This includes   
 ;   commas and the second element which is the file name.                 
 ;   If we are keying csv lines off the file name then multiple title      
 ;   blocks extracted from one drawing won't be written to the file.       
 ;   Must make the drawing name the second field, which means passing it   
 ;   to Dalbe as a string and hoping that it doesn't duplicate an          
 ;   attribute tag, which admittedly looks unlikely.                       
 ; 
  (setq tblisi (list
   (list "1"          (list fileno "," "TITLE_LINE_1" " " "TITLE_LINE_2"
                                   " " "TITLE_LINE_3" " " "TITLE_LINE_4"
                                   "," "REV_NO." "," "DWG_NO.") -1)
   (list "8.5x11 TITLE BLOCK" (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                           "," "REV#_DWG") -1)
   (list "A1_ENG-BAS-SD-0001-R1" (LIST "DRAWINGNO" "," "TITLE-3" " " "TITLE-4"
                                       " " "TITLE-5" "," "DWG_REV") -1)
   (list "a1blk"      (list fileno "," "%%UTITLE_1" " " "%%UTITLE_2"
                                   " " "%%UTITLE_3" " " "%%UTITLE_4"
                                   "," "REV#" "," "DRAWING_NO.") -1)
   (list "a1tblk"     (list "DRAWING_NO" "," fileno "," 
                            "TITLE_3" " " "TITLE_4" "," "REV") -1)
   (list "Aker Proto" (list fileno "," "TITLE4" " " "TITLE5" "," "REV") -1)
   (list "ATTR-D"     (list fileno "," "DWG#" "-" "SHEET#" ","
                            "TITLE_2_OF_3" " " "TITLE_3_OF_3" "," "REV#") -1)
   (list "amw_title"  (list fileno "," "REV_NO" "," "TITLE2" " " "TITLE3") -1)
   (list "BG-D"       (list fileno "," "TITLE3" " " "TITLE4" "," "REVNO") -1)
   (list "bord_a1"    (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG") -1)
   (list "bord_a4"    (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG" " " "TITLE3_TAG") -1)
   (list "border"     (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                   " " "TITLE3_TAG" "," "REV") -1)
   (list "brdr1"      (list fileno "," "DWG_DESCRIPTION"
                                   " " "DWG_CLASSIFICATION") -1)
   (list "CARDTBK2"   (list fileno "," "TITLE1" " " "TITLE2" " " "TITLE3"
                                   "," "REV") -1)
   (list "CatchTB"    (list fileno "," "TITLE3" " " "TITLE4" "," "REV") -1)
   (list "cnrl tb text" (list fileno "," "TITLE_2ND_LINE" "," "REVISION_1") -1)
   (list "cnrl-d-tblk" (list fileno "," "TITLE3" " " "TITLE4" "," "REVNO") -1)
   (list "connachertb" (list fileno "," "DRAWING_NUMBER" "," "TITLE3" " " "TITLE4"
                             "," "REV") -1)
   (list "connacher-tb" (list fileno "," "TITLE3" " " "TITLE4" "," "REV") -1)
   (list "coltma1c"   (list fileno "," "TITLE1" "," "TITLE2" "," "TITLE3" "," "REV") -1)
   (list "DETAIL TB"  (list fileno "," "TITLE_LINE_1" " " "TITLE_LINE_2"
                                   " " "TITLE_LINE_3" "," "REV_NUMBER") -1)
   (list "DEVON-D"    (list fileno "," "DRAWINGNAME" "," "REV") -1)
   (list "drawing_desc." (list fileno "," '(ttt3) "," "X") -1)
   (list "EXC-805-DWG-0001-A1" (list fileno "," "title3" " " "title4" "," "rev#") -1)
   (list "fci-encana" (list fileno "," "TITLE2" "," "TITLE" "," "TITLE3"
                                   "," "REVISION") -1)
   (list "gca1tb"     (list fileno "," "LINE1" "," "LINE2" "," "LINE3"
                                   "," "TBREV") -1)
   (list "gca3ltb"    (list fileno "," "LINE1" "," "LINE2" "," "LINE3"
                                   "," "Line1-2" "," "Line2-2" "," "TBREV") -1)
   (list "geielctb"   (list fileno "," "TITLE_1ST_LINE" "," "TITLE_2ND_LINE"
                                   "," "TITLE_3RD_LINE" "," "REV") -1)
   (list "gela1tb"    (list fileno "," "TITLE_1ST_LINE" "," "TITLE_2ND_LINE"
                                   "," "TITLE_3RD_LINE" "," "REV") -1)
   (list "geia1tb"    (list fileno "," "TITLE_1ST_LINE" "," "TITLE_2ND_LINE"
                                   "," "TITLE_3RD_LINE" "," "REV") -1)
   (list "geia0tb"    (list fileno "," "TITLE_1ST_LINE" "," "TITLE_2ND_LINE"
                                   "," "TITLE_3RD_LINE" "," "REV") -1)
   (list "gca3-cad"   (list fileno "," "LINE1" "," "LINE2" "," "LINE3"
                                   "," "Line1-2" "," "Line2-2" "," "TBREV") -1)
   (list "geia3itb"   (list fileno "," "TITLE_1ST_3LINE" "," "TITLE_1ST_2LINE"
                                   "," "TITLE_2ND_3LINE" "," "TITLE_2ND_2LINE"
                                   "," "TITLE_3RD_3LINE" "," "REV") -1)
   (list "geia3ltb"   (list fileno "," "TITLE_1ST_3LINE" "," "TITLE_1ST_2LINE"
                                   "," "TITLE_2ND_3LINE" "," "TITLE_2ND_2LINE"
                                   "," "TITLE_3RD_3LINE" "," "REV") -1)
   (list "grizzly-tb" (list fileno "," '(grizmo) "," "TITLE1" " " "TITLE2" " "
                                       "TITLE3" " " "TITLE4" "," "REV") -1)
   (list "hocs attribute text" (list fileno "," "PROJECT" "," "TITLE"
                                            "," "DWG_REV") -1)
   (list "hocs attributes text" (list fileno "," "TITLE" "," "DWG_REV") -1)
   (list "attributes text new"  (list fileno "," "TITLE" "," "DWG_REV") -1)
   (list "HUSKYTB"    (list fileno "," "TITLE_3" " " "TITLE_4"
                                   "," "REV") -1)
   (list "HUSKY-D"    (list fileno "," "DRAWING_TITLE_LINE_01"
                                   " " "DRAWING_TITLE_LINE_02" "," (husk)) -1)
   (list "Husky_TitleBlock_Text"
                      (list fileno "," "DRAWING_TITLE_LINE_01"
                                   " " "DRAWING_TITLE_LINE_02"
                                   "," "CURRENT_REV_NUMBER") -1)
   (list "HUSKY ANSI D Tblock Attributes"
                      (list fileno "," "DRAWING_TITLE_LINE_01"
                                   " " "DRAWING_TITLE_LINE_02"
                                   " " "DRAWING_TITLE_LINE_03"
                                   "," "REV_NO") -1)
   (list "HUSKY HOOL ANSI D Tblock Attributes"
                      (list fileno "," "DRAWING_TITLE_LINE_02"
                                   " " "DRAWING_TITLE_LINE_03"
                                   "," "REV_NO") -1)
   (list "inf" (list fileno "," '(inf enam (list 1 2 3)) "," "REVISION") -1)
   (list "Laricina_TB" (list fileno "," "LINE3" " " "LINE4" "," "TBREV") -1)
   (list "Laricina-D" (list fileno "," "TITLE3" " " "TITLE4" "," "REVNO") -1)
   (list "maintb"     (list fileno "," "TITLE_1ST_LINE" "," "TITLE_2ND_LINE"
                                   "," "TITLE_3RD_LINE" "," "REV") -1)

   (list "MEG_ALLIANCE_A1_ENG-BAS-SD-0001-R1" (LIST "DRAWINGNO" "," "TITLE-3"
                                                    " " "TITLE-4" " " "TITLE-5"
                                                    "," "DWG_REV") -1)
   (list "MEGENERGYALLIANCE-A1" (list "DRAWINGNO" "," "TITLE-3" " " "TITLE-4"
                                      " " "TITLE-5" "," "REV0") -1)
   (list "MEG_Energy_A1" (list fileno "," "TITLE-3" " " "TITLE-4" "," "A") -1)
   (list "moneta tblk" (list fileno "," "PROJECT_1" " " "PROJECT_2" " "
                                   "PROJECT_3" " " "PROJECT_4"
                                   "," "REVISION_NUMBER") -1)
   (list "new blockCIV" (list fileno "," "TITLE-3" " " "TITLE-4" "," "A") -1)
   (list "NCLTTLD"    (list fileno "," "TITLE1" " " "TITLE2" " " "TITLE3"
                                   "," "REV") -1)
   (list "NCLTTLD1"   (list fileno "," "TITLE1" " " "TITLE2" " " "TITLE3"
                                   "," "REV") -1)
   (list "PARAMOUNT_D(metric)"
                      (list fileno "," "PROJECT_2" "," "REV_NUMBER") -1)
   (list "PennWest TB_D Size-MS"
                      (list fileno "," "Title2" " " "Title4" "," "REV") -1)
   (list "PERPETUAL_D(metric)TEMP"
                      (list fileno "," "PROJECT_2" "," "REVISION_NUMBER") -1)
   (list "PEL D Size" (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                   " " "TITLE3_TAG"
;                                   " * If Missing Third Line - Check Drawing for Text *"
                                   "," "REV#_DWG") -1)
   (list "NEW PEL D Size" (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                       "," "REV#_DWG") -1)
   (list "PEL D Size(Color)" (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                   "," "REV#_DWG") -1)
   (list "PPC-Dsize" (list fileno "," "REV#" "," "TITLE_2_OF_3" " " "TITLE_3_OF_3") -1)
   (list "PPL-Dsize"  (list fileno "," "DWG#" "-" "SHEET#" ","
                            "TITLE_2_OF_3" " " "TITLE_3_OF_3" "," "REV#") -1)
   (list "Provident TB D Size" (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                            " " "TITLE3_TAG" "," "REV#_DWG") -1)
   (list "Revdata"  (list fileno "," "N" "," "DESCRIPTION" "," "SUBPROJ" ","
                                 "DATE" "," "NAME" "," APP) -1)
   (list "RWTitleblock" (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG"
                                     "," "REV#_DWG") -1)
   (list "shawn"      (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG" " " "TITLE3_TAG") -1)
   (list "Secure_A1_Titleblock" (list fileno "," "DWG.DESCRIPTION" " " "DWG.TYPE"
                                      "," "DWG.REV") -1)
   (list "STD D TITLE BLOCK" (list fileno "," "TITLE_1" " " "TITLE_2"
                                      " " "TITLE_3" "," "REV") -1)
   (list "t2a-2"      (list fileno "," "TITLE1" " " "TITLE2" "," "RC1") -1)
   (list "T-A002A"    (list fileno "," "TITLE1" " " "TITLE2" "," "RC1") -1)
   (list "T-A002A_9"  (list fileno "," "ECA-TITLE1" " " "ECA-TITLE2" "," "RC1") -1)
   (list "t2a-7"      (list fileno "," "ECA-TITLE1" " " "ECA-TITLE2" "," "RC1") -1)
   (list "t2a-8"      (list fileno "," "ECA-TITLE1" " " "ECA-TITLE2" "," "RC1") -1)
   (list "tb_d"       (list fileno "," "TITLE3" " - " "TITLE4" "," "REV1" "," "REV2" ","
                                   "REV3" "," "REV4" "," "REV5" "," "REV6" "," "REV7" ","
                                   "REV8" "," "REV9") -1)
   (list "tcmtb"      (list fileno "," "TITLE1_TAG" " " "TITLE2_TAG" " " "TITLE3_TAG"
                            "," "REV#_DWG") -1)
   (list "titl_a1"    (list fileno "," "LOCATION" " " "DWG_TYPE"
                            "," "REV" "," "DWG_NUMBER") -1)
   (list "titl-d2"    (list "DRAWINGNO" "," fileno "," "TITLE1" " " "TITLE2"
                            "," "MAINREV") -1)
   (list "titlea1a"   (list '(epfiln) "," "DRAWING_NAME" "," '(eptop)) -1)
   (list "title_block" (list fileno "," "DRAWING_TITLE_LINE_01"
                                    " " "DRAWING_TITLE_LINE_02"
                                    " " "DRAWING_TITLE_LINE_03"
                                    "," "DWG_REVISION") -1)
   (list "tiwwtb"     (list fileno "," "TITLE1" " " "TITLE2" "," "REVISION") -1)
   (list "Transworld-D" (list "TWACAD_DRAWING_NO" "," "TITLE2" " " "TITLE3" "," "REV") -1)
   (list "TRIDYNE_TB_D" (list "DRAWING_NUMBER" "," "TITLE_2ND_LINE" " " "TITLE_3RD_LINE"
                              " " "TITLE_4TH_LINE" "," "REVISION") -1)
   (list "WEC DSIZE METRIC" (list fileno "," "TITLE1" " " "TITLE2"
                                         " " "TITLE3"
                                         " " "TITLE4" "," "REV_NO") -1)))
 ; Ŀ
 ;   Reprocess the list to allow for different title blocks with the same  
 ;   name.  This will probably be expanded later.                          
 ; 
  (cond ((tblsearch "block" "a1tblk")
         (setq atts (frat "a1tblk"))
         (if (= (car atts) "JOB")
             (progn
                  (setq sub (list "a1tblk" (list fileno
                                   "," "%%UTITLE_1" " " "%%UTITLE_2"
                                   " " "%%UTITLE_3" " " "%%UTITLE_4"
                                   "," "REV#" "," "DRAWING_NO.") -1))
                  (setq tblisi (subst sub (assoc (car sub) tblisi) tblisi)))))
        ((tblsearch "block" "border")
         (setq atts (frat "border"))
         (if (= (length atts) 1)
             (setq tblisi (subst '("null") (assoc "border" tblisi) tblisi)))))
 ; Ŀ
 ;   Try to find a title block, call the data extraction subroutine.       
 ; 
  (setq num 0)
  (while (and (null stop)
              (setq blist (nth num tblisi)))
         (setq blnam (car blist))
         (setq num (1+ num))
         (if (and (tblsearch "block" blnam)
                  (setq ss (ssget "X" (list (cons 2 blnam))))
                  (= 1 (cdr (assoc 66 (entget (ssname ss 0))))))
             (progn
                  (setq datlst (dalbe (cons ss (cdr blist))))
                  (setq stop T))))
 ; Ŀ
 ;   If no block was found, make an error string.                          
 ; 
  (if (null stop)
      (setq datlst (list (strcat fileno ",No Data Available"))))
 datlst)
 ; Ŀ
 ;   Subroutine Trout end.                                                 
 ; 

 ; Ŀ
 ;   Ttt3 - get title values when the atts are all called Title.           
 ;   Takes no arguments.                                                   
 ;   Returns a list of values.                                             
 ; 
 (DEFUN TTT3 (/ ss enam tagg entt vall mastr)
  (setq mastr "")
  (if (setq ss (ssget "X" (list (cons 2 "drawing_desc."))))
      (progn
           (setq enam (ssname ss 0))
           (repeat 2
                   (setq enam (entnext enam)))
           (repeat 3 
                   (setq entt (entget (setq enam (entnext enam))))
                   (setq vall (cdr (assoc 1 entt)))
                   (setq mastr (strcat mastr " " vall)))))
 (substr mastr 2))
 ; Ŀ
 ;   Ttt3 end.                                                             
 ; 

 ; Ŀ
 ;   Trout.                                                                
 ; 
 (DEFUN C:TROUT (/ datlst fn)
 ; Ŀ
 ;   Get a list of title block data strings.                               
 ; 
   (setq datlst (trout))
 ; Ŀ
 ;   Write the data strings from datlst to the file.                       
 ; 
  (setq fn (open "c:/save/Titledat.csv" "a"))
  (mapcar '(lambda (str) (write-line str fn)) datlst)
  (close fn)
 (princ))