 ; Ŀ
 ;   Unbind - make a block (back) into an xref.                            
 ;   Copyright 2003, 2004, 2013 by Rocket Software Ltd.                    
 ;   See Xeno, which can re-clip a block, and which may be finished.       
 ;   One could make a program to check layers for $n$ bound names and      
 ;   then unbind those blocks.                                             
 ;   Also to block out blocks for which there are no files.                
 ;                                                                         
 ;                                                                         
 ; 

 ; Ŀ
 ;   Blada - get block insertion data.                                     
 ;   Arguments: ss, a selection set of block insertions.                   
 ;   Ignores attributes because these are bound xrefs and there shouldn't  
 ;   be any.                                                               
 ;   Calls nothing.                                                        
 ;   Returns a list of lists of entity data:                               
 ;   ((insertion_point  rotation_degrees  xscale  yscale  zscale  layer    
 ;     layout) ...)                                                        
 ; 
 (DEFUN BLADA (ss / num enam entt pa rota xsc ysc zsc layy tabb subdat malist)
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (setq entt (entget enam))
 ; Ŀ
 ;   Get its insertion point, scales, layer, and rotation.                 
 ; 
         (setq pa (cdr (assoc 10 entt)))
         (setq rota (cdr (assoc 50 entt)))
         (if rota
            (setq rota (/ (* 180 rota) pi))
            (setq rota 0))
         (setq xsc (cdr (assoc 41 entt)))
         (if (null xsc) (setq xsc 1))
         (setq ysc (cdr (assoc 42 entt)))
         (if (null ysc) (setq ysc 1))
         (setq zsc (cdr (assoc 43 entt)))
         (if (null zsc) (setq zsc 1))
         (setq layy (cdr (assoc 8 entt)))
         (setq tabb (cdr (assoc 410 entt)))
 ; Ŀ
 ;   Make a data sublist, add it to the overall list.                      
 ; 
         (setq subdat (list pa rota xsc ysc zsc layy tabb))
         (setq malist (cons subdat malist)))
 malist)
 ; Ŀ
 ;   Blada end.                                                            
 ; 

 ; Ŀ
 ;   Delop - delete an ss of entities which may be in ps or ms.            
 ; 
 (DEFUN DELOP (ss mark / rad num enam pa)
  (setq rad (/ (getvar "viewsize") 95))
  (setq num 0)
  (while (setq enam (ssname ss num))
         (setq num (1+ num))
         (if mark
             (progn
                  (setq pa (cdr (assoc 10 (entget enam))))
                  (grdraw (polar pa (/ pi 4) rad)
                          (polar pa (* 1.25 pi) rad) 7)
                  (grdraw (polar pa (* pi 0.75) rad)
                          (polar pa (* pi 1.75) rad) 7)))
         (entdel enam))
 (princ))
 ; Ŀ
 ;   Delop end.                                                            
 ; 

 ; Ŀ
 ;   Isxref: see if a given block is an xref.                              
 ;   Arguments: Blnam, either an entity name or a block name string.       
 ;   Returns T if the block was an xref, else nil.                         
 ;   Note: returns nil if the block isn't inserted, so be careful.         
 ; 
 (DEFUN ISXREF (blnam / dat xp isxrf)
  (if (= (type blnam) 'ename)
      (setq blnam (cdr (assoc 2 (entget blnam)))))
  (if (setq dat (tblsearch "block" blnam))
      (progn
           (setq xp (cdr (assoc 70 dat)))
           (setq isxrf (logand xp 4))))
 (if (= isxrf 4) T ()))
 ; Ŀ
 ;   Isxref end.                                                           
 ; 

 ; Ŀ
 ;   Rinxs - insert xrefs.                                                 
 ;   Arguments: Blnam, the block name.                                     
 ;              Malist, the list of lists returned by Blada:               
 ;   ((ins rota_degrees xscale yscale zscale layer layout) ...)            
 ;   Calls nothing, Returns nothing.                                       
 ; 
 (DEFUN RINXS (blnam malist / num sub tabb entt)
  (setq num 0)
  (while (setq sub (nth num malist))
         (setq num (1+ num))
         (setq tabb (last sub))
         (setq layy (nth 5 sub))
         (setvar "ctab" tabb)
         (if (/= tabb "Model")
             (command "pspace"))
         (command "-xref" "attach" blnam (car sub)
                  "xyz" (caddr sub) (cadddr sub) (nth 4 sub)
                  (cadr sub))
 ; Ŀ
 ;   Put the block on the correct layer.                                   
 ; 
         (setq entt (entget (entlast)))
         (entmod (subst (cons 8 layy) (assoc 8 entt) entt)))
 (princ))
 ; Ŀ
 ;   Rinxs end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Unbind.  Make a block into an xref.                        
 ;   Arguments: Blnam, a block name.                                       
 ;   Calls Isxref, Blada, Delop, and Rinxs.                                
 ; 
 (DEFUN UNBIND (blnam / ss bldat)
 ; Ŀ
 ;   Ignore the block if it's not defined in the drawing.                  
 ; 
  (cond ((null (tblsearch "block" blnam))
         (write-line (strcat "\n" blnam
                             " Is Not Defined In This Drawing.")))
 ; Ŀ
 ;   Or if it's already an xref.                                           
 ; 
        ((isxref blnam)
         (write-line (strcat "\n" blnam " Is Already An Xref.")))
 ; Ŀ
 ;   Of if the file by the same name isn't available.                      
 ;   Could extract it from the current file if it isn't available.         
 ;   This could be automated, or made to be an option.                     
 ; 
        ((null (findfile (strcat blnam ".dwg")))
         (write-line (strcat "\n" blnam
                             " Is Not Available As a File."))
         (write-line "\nBut you could block out the one in this drawing."))
 ; Ŀ
 ;   Or if there are no insertions.                                        
 ; 
        ((null (setq ss (ssget "x" (list (cons 2 blnam)))))
         (write-line (strcat "\n" blnam ": No Insertions Found.")))
 ; Ŀ
 ;   Otherwise go on.                                                      
 ; 
        (t
         (write-line (strcat "\nXrefing " blnam))
 ; Ŀ
 ;   Get a list of all data for each block insertion.                      
 ; 
         (setq bldat (blada ss))
 ; Ŀ
 ;   Erase all of them.  Note that they may not all be in the same space.  
 ; 
         (delop ss t)
 ; Ŀ
 ;   Purge the block definition.                                           
 ; 
         (command "-purge" "block" blnam "n")
 ; Ŀ
 ;   Call Rinxs to reinsert the xrefs.                                     
 ; 
         (rinxs blnam bldat)))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
 (princ))
 ; Ŀ
 ;   Subroutine Unbind end.                                                
 ; 

 ; Ŀ
 ;   Unbind.                                                               
 ; 
 (DEFUN C:UNBIND (/ *error* ublis num blnam)
  (setvar "cmdecho" 0)
  (command ".undo" "be")
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk /)
   (if (type command-s)
       (command-s ".undo" "end")
       (command ".undo" "end"))
   (write-line shk)
  (princ))
 ; Ŀ
 ;   Make a list of blocks to unbind.                                      
 ; 
  (setq ublis '("64200-model-sm-00-18" "GD1-013-M-MDL-028-01"))
 ; Ŀ
 ;   Call Unbind to deal with them.                                        
 ; 
  (setq num 0)
  (while (setq blnam (nth num ublis))
         (setq num (1+ num))
         (unbind blnam))
 ; Ŀ
 ;   Clean up and end.                                                     
 ; 
  (command ".undo" "end")
 (princ))
