 ; Ŀ
 ;   Vcut - insert a block (upright) on a vertical line, break the line.   
 ;   Copyright 2000, 2010 by Rocket Software Ltd.                          
 ;   Asks for four inputs: bottom and top break distances, block name,     
 ;   and insertion point.                                                  
 ;   But: if the first input is a point and the global variables still     
 ;   contain values for the other three required inputs, the block is      
 ;   inserted at that point using those values.                            
 ;                                                                         
 ; 

 ; Ŀ
 ;   Pf: find the lwpline, pline, or line segment a point lies on.         
 ;   Takes two arguments: the point and the pline ename.                   
 ;   Returns a list: the number of vertices and the endpoints of the       
 ;   segment containing the point, or the number of vertices and nil.      
 ;   If the entity is a line then the number of vertices is usually 2,     
 ;   other entity types return nil.                                        
 ;   Caution: doesn't check for closed polylines.                          
 ; 
 (DEFUN PF (pa enam / typ entt num vnum entt2 end1 end2 angg pb pint entt1)
  (setq typ (cdr (assoc 0 (setq entt (entget enam)))))
  (cond ((= typ "POLYLINE")
         (setq num 0)
         (while (/= "SEQEND" (cdr (assoc 0 (setq entt2
                                       (entget (setq enam (entnext enam)))))))
                (setq num (1+ num))
                (if (and entt1 entt2 (null pint))
                    (progn
                         (setq end1 (cdr (assoc 10 entt1)))
                         (setq end2 (cdr (assoc 10 entt2)))
                         (setq angg (angle end1 end2))
                         (setq pb (polar pa (+ angg (/ pi 2)) 0.000001))
                         (setq pint (inters end1 end2 pa pb))))
                (if (null pint)
                    (setq entt1 entt2)))
         (if pint
            (list num end1 end2)
            (list num ())))
        ((= typ "LWPOLYLINE")
         (setq vnum (setq num 0))
         (while (setq sub (nth num entt))
                (setq num (1+ num))
                (if (= (car sub) 10)
                    (progn
                         (setq vnum (1+ vnum))
                         (setq end2 (cdr sub))
                         (if (and end1 end2 (null pint))
                             (progn
                                  (setq angg (angle end1 end2))
                                  (setq pb (polar pa (+ angg (/ pi 2))
                                                                     0.000001))
                                  (if (inters end1 end2 pa pb)
                                      (setq pint (list end1 end2)))))
                         (if (null pint) (setq end1 end2)))))
         (cons vnum pint))
        ((= typ "LINE")
         (setq end1 (cdr (assoc 10 entt)))
         (setq end2 (cdr (assoc 11 entt)))
         (setq angg (angle end1 end2))
         (setq pb (polar pa (+ angg (/ pi 2)) 0.000001))
         (setq pint (inters end1 end2 pa pb))
         (if pint
            (list 2 end1 end2)
            (list 2 ())))
        (T nil)))
 ; Ŀ
 ;   Pf end.                                                               
 ; 

 ; Ŀ
 ;   Subroutine Ptor - get either a point or a set of inputs.              
 ;   If the required inputs other than the point have acceptable values,   
 ;   ask for a point but if a number is offered take it as the first       
 ;   and ask for three more, the last being the point.                     
 ;   If any of the required inputs are not acceptable, ask for all of      
 ;   them.                                                                 
 ;   The whole process is intended to be transparent to the user: if the   
 ;   program is called from the menu all arguments are supplied except     
 ;   the point.  The command can be repeated with a <Return> in which      
 ;   case it uses the previous block name and break distance values.       
 ;                                                                         
 ;   Takes no arguments, calls nothing, returns a list: two numbers,       
 ;   intended to be the left and right side break distances, a string      
 ;   (block name) and a point list.                                        
 ;                                                                         
 ; 
 (DEFUN PTOR (/ pa pb)
  (if (and (= (type ldist) 'REAL)
           (= (type rdist) 'REAL)
           (= (type name) 'STR))
      (progn
           (initget 128)
           (setq pa (getpoint "Insertion point: "))
           (if (not (listp pa))
               (progn
                    (setq ldist (* 1.0 (read pa)))
                    (setq rdist (getdist))
                    (setq name (getstring))
                    (setq pa (getpoint)))))
      (progn
           (setq ldist (getdist))
           (setq rdist (getdist))
           (setq name (getstring))
           (setq pa (getpoint "Insertion point: "))))
 ; Ŀ
 ;   See if can snap the insertion point to anything.                      
 ; 
  (setq pb (osnap pa "Nearest"))
  (if pb (setq pa pb))
 (list ldist rdist name pa))
 ; Ŀ
 ;   Ptor end.                                                             
 ; 


 ; Ŀ
 ;   Vcut.                                                                 
 ; 
 (DEFUN C:VCUT (/ *error* blip osmo scal devlst devls2 pa pb ss typ entt rota
                                             rrota xa xb enam blins blscal pc)
  (setvar "cmdecho" 0)
  (setq blip (getvar "blipmode"))
  (setvar "blipmode" 0)
  (setq osmo (getvar "osmode"))
 ; Ŀ
 ;   Make an error handler.                                                
 ; 
 (defun *error* (shk)
  (setvar "blipmode" blip)
  (setvar "osmode" osmo)
  (if shk (print shk))
 (princ))
 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))
 ; Ŀ
 ;   Get the scale, depending on which space we are in and other things.   
 ; 
  (if misps
      (setq scal (misps))
      (setq scal (getvar "dimscale")))
 ; Ŀ
 ;   And get a block name and cut distances and insertion point.           
 ; 
  (setq lisa (ptor))
  (setq ldist (car lisa))
  (setq rdist (cadr lisa))
  (setq name (strcase (caddr lisa)))
  (setq pa (cadddr lisa))
  (setq ldisst (* ldist scal))
  (setq rdisst (* rdist scal))
 ; Ŀ
 ;   If there is a line or a pline at the insertion point...               
 ; 
  (if (setq ss (ssget pa))
      (progn
           (setq typ (cdr (assoc 0 (entget (setq enam (ssname ss 0))))))
 ; Ŀ
 ;   Get the angle of the segment containing the pick point.               
 ; 
           (if (or (= typ "LINE") (= typ "POLYLINE") (= typ "LWPOLYLINE"))
               (progn
                    (setq rota (pf pa enam))
                    (setq rota (angle (cadr rota) (caddr rota)))
                    (if (or (> rota pi) (= rota 0))
                        (setq rota (- rota pi)))))))
 ; Ŀ
 ;   Now decide what to do.  The block rotation matches the line           
 ;   rotation, if there is no line then the rotation is 0.                 
 ; 
  (setvar "osmode" 0)
  (cond ((and (or (= typ "LINE") (= typ "POLYLINE") (= typ "LWPOLYLINE")))
         (setq rrota (- (/ (* 180 rota) pi) 90))
         (setq xa (polar pa rota rdisst))
         (setq xb (polar pa rota (- ldisst)))
         (command "break" pa "f" xa xb))
        (T (setq rrota 0.0)))
  (command "insert" name pa scal "" rrota)
 ; Ŀ
 ;   Reset and end.  This should leave the insert command active so that   
 ;   the command prompts for attribute values normally without further     
 ;   code.                                                                 
 ; 
  (setvar "blipmode" blip)
  (setvar "osmode" osmo)
 (princ))
