 ; Ŀ
 ;   Wiretag - put an appropriate block on a horizontal or vertical wire.  
 ;   Wiretagr - the same but on the right side.                            
 ;   Copyright 2000, 2001, 2010, 2013 by Rocket Software Ltd.              
 ;                                                                         
 ; 

 ; Ŀ
 ;   Subroutine Wiretag.                                                   
 ;   Argument: rl - "r" = right side, "l" = left.                          
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN WIRETAG (rl / *error* scal pa pb ss enam entt ten elv angg osm)
  (setvar "cmdecho" 0)
  (if (= rl "l")
      (progn
           (setq vtag "vwiretag")
           (setq htag "hwiretag"))
      (progn
           (setq vtag "vwiretagr")
           (setq htag "hwiretagr")))
 ; Ŀ
 ;   Don't use a dialog box.  Maybe this should be set globally.           
 ; 
  (setvar "attdia" 0)
  (setq osm (getvar "osmode"))
  (setvar "osmode" 0)
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (setvar "osmode" osm)
   (if shk (write-line shk))
  (princ))
 ; Ŀ
 ;   Load Misps.lsp, which contains the ps/ms scaling subroutines.         
 ; 
  (if (or (null wasp) (null misps))
      (if (null (load "misps" ()))
          (prompt "\n** The File Misps.lsp Is Not Available. **\n")))
 ; Ŀ
 ;   Get the scale, depending on which space we are in and other things.   
 ; 
  (if misps
      (setq scal (misps))
      (setq scal (getvar "dimscale")))
 ; Ŀ
 ;   Ask for an insertion point.                                           
 ; 
  (setq pa (getpoint "Line: "))
  (if (setq pb (osnap pa "near"))                  ; look for a near point
      (setq pa pb))                                ; if one then use it
  (setq psav pa)
 ; Ŀ
 ;   Decide which type of entity we are dealing with, find its angle.      
 ;   A line...                                                             
 ; 
  (cond ((setq ss (ssget pa (list (cons 0 "line"))))
         (setq enam (ssname ss 0))
         (setq entt (entget enam))
         (setq ten (cdr (assoc 10 entt)))
         (setq elv (cdr (assoc 11 entt)))
         (setq angg (angle ten elv))
         (setq angg (/ (* angg 180) pi)))
 ; Ŀ
 ;   Or a polyline.                                                        
 ; 
        ((setq ss (ssget pa '((-4 . "<or") (0 . "polyline")
                              (0 . "lwpolyline") (-4 . "or>"))))
         (setq pb (osnap pa "end"))
         (if (equal pa pb) (setq pa (osnap pa "mid")))
         (setq angg (angle pa pb))
         (setq angg (/ (* angg 180) pi))))
 ; Ŀ
 ;   ...or empty space.                                                    
 ; 
  (if (or (equal angg 90 22) (equal angg 270 22))
      (command "insert" vtag psav scal "" "0")
      (command "insert" htag psav scal "" "0"))
 ; Ŀ
 ;   End but leave the insert command active.                              
 ; 
   (setvar "osmode" osm)
 (princ))
 ; Ŀ
 ;   Subroutine Wiretag end.                                               
 ; 

 ; Ŀ
 ;   Wiretag.                                                              
 ; 
 (DEFUN c:WIRETAG ()
  (wiretag "l")
 (princ))

 ; Ŀ
 ;   Wiretagr.                                                             
 ; 
 (DEFUN c:WIRETAGR ()
  (wiretag "r")
 (princ))