 ; Ŀ
 ;   Xreg: utility function.  Write a line to a csv log file.              
 ;   Copyright 2012 by Rocket Software Ltd.                                
 ;   This will delete one comma from the end of a csv line each time       
 ;   it's run, which shouldn't be a problem - commas with values are safe. 
 ; 

 ; Ŀ
 ;   Bulla - write a list of lists to a csv file.                          
 ;   Arguments: Lista, a list.                                             
 ;              Filnam, a filename.                                        
 ;   Calls nothing, returns nothing.                                       
 ; 
 (DEFUN BULLA (lista filnam / fn sub str nxtstr)
  (setq fn (open filnam "w"))
  (while (setq sub (car lista))
         (setq lista (cdr lista))
         (setq str "")
         (while (setq nxtstr (car sub))
                (setq sub (cdr sub))
                (setq str (strcat str "," nxtstr)))
         (write-line (substr str 2) fn))
  (close fn)
 (princ))
 ; Ŀ
 ;   Bulla end.                                                            
 ; 

 ; Ŀ
 ;   Fcut - suck a cdf file into a list of lists.                          
 ;   Arguments: filnam, a filename.                                        
 ;   Calls Splat.                                                          
 ;   Returns a list of lists of strings.                                   
 ; 
 (DEFUN FCUT (filnam / fn linn llist malist)
  (if (setq fn (open filnam "r"))
      (progn
 ; Ŀ
 ;   Read each line.                                                       
 ; 
           (while (setq linn (read-line fn))
                  (if (/= linn "")
                      (progn
 ; Ŀ
 ;   Divide it into a list of strings.                                     
 ; 
                           (setq llist (splat "," linn))
 ; Ŀ
 ;   Add the new sublist to malist.                                        
 ; 
                           (setq malist (append malist (list llist))))))
           (close fn)))
 malist)
 ; Ŀ
 ;   Fcut end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Finsub - find the appropriate sublist in a list.           
 ;   Arguments: Alist, a list of lists of strings.                         
 ;              Match, the string used to identify the correct line.       
 ;              Pos, the list position to search for the match string.     
 ;   Calls nothing.                                                        
 ;   Returns the first (and ideally only) matching sublist or nil.         
 ; 
 (DEFUN FINSUB (alist match pos / notyet num linn strp)
  (setq match (strcase match))
  (setq notyet t)
 ; Ŀ
 ;   Search each sublist for the match string.                             
 ; 
  (setq num 0)
  (while (and notyet alist (setq linn (nth num alist)))
         (setq num (1+ num))
         (if (and (setq strp (nth pos linn))
                  (= match (strcase strp)))
             (setq notyet ())))
 (if notyet () linn))
 ; Ŀ
 ;   Finsub end.                                                           
 ; 

 ; Ŀ
 ;   Loofah - make a stub C:\Register.csv file.                            
 ;   Arguments: None.                                                      
 ;   Calls nothing, returns a closed (?) file handle or nil.               
 ; 
 (DEFUN LOOFAH (/ header fn)
  (setq header (strcat "Drawing Name,New Drawing Name,Name Collision,"
    "Duplicate Files,No Name in TB,# of Title Blocks,TB Swapped,"
    "TB Data Installed,Second TB CSV Pass,Attribute Widths,File Name S&R,"
    "Xdata Block Inserted,Odd Xrefs Bound,Schematic Xrefs,Drawing Split,"
    "Excess Tabs Killed,Title Block to MS,Extra Sheets in MS Deleted,"
    "Limits Set to Extents,Clouds Removed,Excess Vports Killed,"
    "Xrefs Renamed in Drawing,Xrefs Repathed to Relative,Trash Blocks Erased,"
    "Pline Boxes Erased,3D Model Renamed and TB'd,"
    "Xref Re-Pointed to Renamed File,Absent TB Added,Xref Moved to Directory,"
    "Xref on Bad Drive Fixed"))
  (if (setq fn (open "c:\\Register.csv" "w"))
      (progn
           (write-line header fn)
           (close fn)))
 fn)
 ; Ŀ
 ;   Loofah end.                                                           
 ; 

 ; Ŀ
 ;   Quank - update a value at a position in a list.                       
 ;   If the list isn't long enough, pad it with empty strings.             
 ;   Arguments: Linn, a list of strings.                                   
 ;              Str, a new string value.                                   
 ;              p0s, the zero-based position for the new string.           
 ;              Addp - if the old and new values are numbers and this      
 ;                     is T then add them together.                        
 ;   Returns an updated list of strings.                                   
 ; 
 (DEFUN QUANK (linn str p0s addp / num sub ssub sstr gnu)
  (while (< (length linn) p0s)
         (setq linn (append linn (list ""))))
  (cond ((= (length linn) p0s)
         (setq linn (append linn (list str))))
        (t
         (setq num 0)
         (while (setq sub (nth num linn))
                (cond ((and addp
                            (= p0s num)
                            (member (type (setq ssub (read sub))) '(int real))
                            (member (type (setq sstr (read str))) '(int real)))
                       (setq sub (+ ssub sstr))
                       (if (or (= (type ssub) 'real)
                               (= (type sstr) 'real))
                           (setq sub (rtos sub))
                           (setq sub (itoa sub))))
                      ((= p0s num)
                       (setq sub str)))
                (setq gnu (cons sub gnu))
                (setq num (1+ num)))
         (setq linn (reverse gnu))))
 linn)
 ; Ŀ
 ;   Quank end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Splat - divide a text string into a list of substrings.    
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ;                                                                         
 ;   Completely rewritten 11.19.2000.                                      
 ; 
 (DEFUN SPLAT (sepchr linn / pos len name1 strlst)
  (while (/= (strlen linn) 0)
 ; Ŀ
 ;   Find the first separator character, save everything before it into    
 ;   the Name1 variable, remove it from the start of the string Linn.      
 ; 
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (setq linn (substr linn (1+ pos)))
 ; Ŀ
 ;   Remove spaces from the front and back of Name1.                       
 ; 
         (while (and (> (strlen name1) 0)
                     (= (substr name1 (setq len (strlen name1))) " "))
                (setq name1 (substr name1 1 (1- len))))
         (while (and (> (strlen name1) 0)
                     (= (substr name1 1 1) " "))
                (setq name1 (substr name1 2)))
 ; Ŀ
 ;   Add Name1 to the substring list Strlst.                               
 ; 
         (setq strlst (append strlst (list name1))))
 ; Ŀ
 ;   If the string contained no separator characters then Strlst will be   
 ;   nil, so return a list containing the original string.                 
 ; 
  (if (null strlst) (setq strlst (list linn)))
 strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Xreg.                                                                 
 ; 
 (defun xreg (str p0s addp / filnam nam len *error* tlist linn suba)
  (setq filnam "c:\\Register.csv")
 ; Ŀ
 ;   Make the string which will identify the correct csv line.             
 ; 
  (setq nam (getvar "dwgname"))
  (if (= (strcase (substr nam (- (setq len (strlen nam)) 3))) ".DWG")
      (setq nam (substr nam 1 (- len 4))))
 ; Ŀ
 ;   Make a local error handler.                                           
 ; 
  (defun *error* (shk)
   (if regist (regist (strcat "***** Error in Xreg.lsp, File: " nam)))
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Find the data file, get the appropriate line as a list of strings.    
 ; 
  (cond ((null (findfile filnam))
         (if (loofah)
             (regist (strcat nam ": Data File Created."))
             (regist (strcat nam ": Can't Create C:\\Register.csv.")))
         (setq tlist (fcut filnam)))
 ; Ŀ
 ;   Suck the file into a list of lists of strings.                        
 ; 
        ((null (setq tlist (fcut filnam)))
         (if (loofah)
             (progn
                  (regist (strcat nam ": Data File Header Added."))
                  (setq tlist (fcut filnam)))
             (regist (strcat nam ": Can't Make C:\\Register.csv.")))))
 ; Ŀ
 ;   If no matching line was found, make one.                              
 ; 
  (cond ((and (null (setq linn (finsub tlist nam 0)))
              (null (setq linn (finsub tlist nam 1))))
         (setq linn (list nam))
         (repeat (1- p0s) (setq linn (append linn (list ""))))
         (setq linn (append linn (list str)))
         (regist (strcat nam ": Data Line added to " filnam ".")))
 ; Ŀ
 ;   If we made it here, update the string list.                           
 ; 
        (t
         (setq linn (quank linn str p0s addp))))
 ; Ŀ
 ;   Replace the list in the master list (or add it.)                      
 ; 
  (if (setq suba (assoc (car linn) tlist))
      (setq tlist (subst linn suba tlist))
      (setq tlist (append tlist (list linn))))
 ; Ŀ
 ;   Overwrite the file with the new list.                                 
 ; 
  (bulla tlist filnam)
 (princ))