 ; Ŀ
 ;   Ze - Zoom Extents with (usually) no regen.                            
 ;   Zoob - Zoom Extents in a viewport, which Ze won't do.                 
 ;   Zxp - Zoom paper space to a desired scale.                            
 ;   Copyright 1990, 2001, 2003, 2010, 2011 by Rocket Software Ltd.        
 ;   When don't seconds count?                                             
 ; 

 ; Ŀ
 ;   Subroutine Vu - unlock all viewports.                                 
 ;   Takes no arguments, calls nothing.                                    
 ;   Returns t if everything went ok, nil if not.                          
 ; 
 (DEFUN VU (/ unlo)
  (if (= (strcase (getvar "ctab") t) "model")
      (prompt "\n* You Can't Unlock Viewports From Model Space. *")
      (progn
           (command ".mview" "L" "off" "all" "")
           (setq unlo t)))
 unlo)
 ; Ŀ
 ;   Subroutine Vu end.                                                    
 ; 

 ; Ŀ
 ;   Subroutine Vl - lock all viewports.                                   
 ;   Takes no arguments, calls nothing.                                    
 ;   Returns t if everything went ok, nil if not.                          
 ; 
 (DEFUN VL ()
  (if (= (strcase (getvar "ctab") t) "model")
      (prompt "\n* You Can't Lock Viewports From Model Space. *")
      (progn
           (command ".mview" "L" "on" "all" "")
           (setq vlo t)))
 vlo)
 ; Ŀ
 ;   Subroutine Vl end.                                                    
 ; 

 ; Ŀ
 ;   Vprt - see if we are in model space, paper space, or a viewport.      
 ;   Returns a list: a descriptive string: model, paper, or vport          
 ;   and the corner points of either the current screen, if we are in      
 ;   paper or model space, or the corners of the current vport in overall  
 ;   screen coordinates, not those of the viewport itself.                 
 ;   Calls nothing, takes no arguments.                                    
 ; 
 (DEFUN VPRT (/ cvport spac a vs ctr w cornr1 cornr2 cornrs)
  (setq cvport (getvar "cvport"))
 ; Ŀ
 ;   Possibilities: we are in the model tab.                               
 ; 
  (cond ((= (getvar "ctab") "Model")
         (setq spac "Model")
         (setq a (getvar "screensize"))        ; view height & width (pixels)
         (setq a (/ (car a) (cadr a)))         ; view width/height ratio
         (setq vs (* (getvar "viewsize") 0.5)) ; view height in drawing units
         (setq ctr (getvar "viewctr"))         ; centre point of screen
         (setq w (* vs a ))                    ; view half width
         (setq cornr1 (list (- (car ctr) w) (- (cadr ctr) vs)))
         (setq cornr2 (list (+ (car ctr) w) (+ (cadr ctr) vs))))
 ; Ŀ
 ;   In any layout (ps) tab, paper space is always viewport number 1.      
 ; 
        ((= 1 cvport)
         (setq spac "Paper"))
 ; Ŀ
 ;   The default: We are in a viewport in paper space.                     
 ; 
        (t
         (setq spac "Vport")))
 ; Ŀ
 ;   If we are in paper space or a viewport, get the corners.              
 ; 
  (if (= spac "Model")
      (setq cornrs (list cornr1 cornr2))
      (setq cornrs (cdr (assoc cvport (vports)))))
 (list spac cornrs))
 ; Ŀ
 ;   Vprt end.                                                             
 ; 

 ; Ŀ
 ;   Zxp.                                                                  
 ; 
 (DEFUN C:ZXP (/ *error* scal)
  (setvar "cmdecho" 0)
 ; Ŀ
 ;   Make an error handler.                                                
 ; 
  (defun *error* (shk)
   (vl)
   (if shk (print shk))
  (princ))
 ; Ŀ
 ;   Make sure we're in a viewport in paper space.                         
 ; 
  (if (/= (car (vprt)) "Vport")
      (prompt "\nYou must be in a viewport in paper space")
      (if (setq scal (getreal "Desired Scale: "))
          (progn
               (setq scal (rtos (/ 1 scal) 2 12))
               (vu)
               (command "zoom" (strcat scal "xp"))
               (vl))))
 (princ))

 ; Ŀ
 ;   Z1.                                                                   
 ; 
 (DEFUN C:Z1 ()
  (setvar "cmdecho" 0)
  (defun *error* (shk)
   (write-line shk)
   (vl)
  (princ))
  (if (= (car (vprt)) "Vport")
      (progn
           (vu)
           (command ".zoom" "0.1x")
           (vl))
      (command ".zoom" "0.1x"))
 (princ))

 ; Ŀ
 ;   Z9.                                                                   
 ; 
 (DEFUN C:Z9 ()
  (setvar "cmdecho" 0)
  (defun *error* (shk)
   (write-line shk)
   (vl)
  (princ))
  (if (= (car (vprt)) "Vport")
      (progn
           (vu)
           (command ".zoom" "0.95x")
           (vl))
      (command ".zoom" "0.95x"))
 (princ))

 ; Ŀ
 ;   Z5.                                                                   
 ; 
 (DEFUN C:Z5 ()
  (setvar "cmdecho" 0)
  (defun *error* (shk)
   (write-line shk)
   (vl)
  (princ))
  (if (= (car (vprt)) "Vport")
      (progn
           (vu)
           (command ".zoom" "0.5x")
           (vl))
      (command ".zoom" "0.5x"))
 (princ))

 ; Ŀ
 ;   Zi.                                                                   
 ; 
 (DEFUN C:ZI ()
  (setvar "cmdecho" 0)
  (defun *error* (shk)
   (write-line shk)
   (vl)
  (princ))
  (if (= (car (vprt)) "Vport")
      (progn
           (vu)
           (command ".zoom" "10x")
           (vl))
      (command ".zoom" "10x"))
 (princ))

 ; Ŀ
 ;   Zoob.                                                                 
 ; 
 (DEFUN C:ZOOB ()
  (setvar "cmdecho" 0)
  (defun *error* (shk)
   (if (= shk "invalid point")
       (progn
            (setq a (getvar "extmin"))
            (setq b (getvar "extmax"))
            (write-line "*Error* - Extents may be invalid.  Values are:")
            (prin1 (list a b)))
       (write-line (strcat "\n" shk)))
   (vl)
  (princ))
  (if (= (car (vprt)) "Vport")
      (progn
           (vu)
           (command ".zoom" "e")
           (vl))
      (command ".zoom" "e"))
 (princ))

 ; Ŀ
 ;   Ze.                                                                   
 ; 
 (DEFUN C:ZE ( / a b *error*)
  (setvar "cmdecho" 0)
  (defun *error* (shk)
   (if (= shk "invalid point")
       (progn
            (write-line "*Error* - Extents may be invalid.  Values are:")
            (prin1 (list a b)))
       (write-line (strcat "\n" shk)))
  (princ))
  (if (= (car (vprt)) "Vport")
      (command "pspace"))
  (command "zoom" "e")
  (setq a (getvar "extmin"))
  (setq b (getvar "extmax"))
  (command "zoom" "w" a b)
 (princ))